<?php
require_once(__DIR__ . '/../config/config.php');

function logError($pdo, $message, $context = '', $errorLevel = 'error') {
    try {
        $stmt = $pdo->prepare("INSERT INTO errors (message, context, error_level) VALUES (?, ?, ?)");
        $stmt->execute([$message, $context, $errorLevel]);
        return $pdo->lastInsertId();
    } catch (PDOException $e) {
        error_log('Error Logging Error: ' . $e->getMessage());
        return false;
    }
}

function getError($pdo, $errorId) {
    try {
        $stmt = $pdo->prepare("SELECT * FROM errors WHERE id = ?");
        $stmt->execute([$errorId]);
        return $stmt->fetch(PDO::FETCH_ASSOC);
    } catch (PDOException $e) {
        error_log('Get Error Error: ' . $e->getMessage());
        return false;
    }
}

function listErrors($pdo, $filters = []) {
    try {
        $query = "SELECT * FROM errors WHERE 1";
        $params = [];
        if (!empty($filters['error_level'])) {
            $query .= " AND error_level = ?";
            $params[] = $filters['error_level'];
        }
        if (!empty($filters['start_date'])) {
            $query .= " AND created_at >= ?";
            $params[] = $filters['start_date'];
        }
        if (!empty($filters['end_date'])) {
            $query .= " AND created_at <= ?";
            $params[] = $filters['end_date'];
        }
        $query .= " ORDER BY created_at DESC";
        $stmt = $pdo->prepare($query);
        $stmt->execute($params);
        return $stmt->fetchAll(PDO::FETCH_ASSOC);
    } catch (PDOException $e) {
        error_log('List Errors Error: ' . $e->getMessage());
        return [];
    }
}

function deleteError($pdo, $errorId) {
    try {
        $stmt = $pdo->prepare("DELETE FROM errors WHERE id = ?");
        $stmt->execute([$errorId]);
        return $stmt->rowCount() > 0;
    } catch (PDOException $e) {
        error_log('Delete Error Error: ' . $e->getMessage());
        return false;
    }
}

function handleException($pdo, Exception $exception) {
    logError($pdo, $exception->getMessage(), $exception->getTraceAsString(), 'exception');
}

try {
    $pdo = new PDO('sqlite:' . DB_FILE);
    $pdo->setAttribute(PDO::ATTR_ERRMODE, PDO::ERRMODE_EXCEPTION);
} catch (PDOException $e) {
    error_log('Database Connection Error: ' . $e->getMessage());
}
?>
