<?php
require_once('../core/audit_logs.php');
require_once('components/auth.php');
require_once('components/header.php');

$errors = [];
$success = '';

if (!checkRole(['audit_logs_add'])) {
    $errors[] = $_SESSION['error_message'] ?? 'Access denied. You do not have permission to add audit logs.';
    unset($_SESSION['error_message']);
    ?>
    <h1>Add Audit Log</h1>
    <?php if (!empty($errors)): ?>
    <div class="alert">
        <?php foreach ($errors as $error): ?>
            <p><?php echo htmlspecialchars($error); ?></p>
        <?php endforeach; ?>
    </div>
    <?php endif; ?>
    <?php require_once('components/footer.php'); ?>
    <?php
    exit;
}

if ($_SERVER['REQUEST_METHOD'] === 'POST') {
    $action = trim($_POST['action']);
    $entity = trim($_POST['entity']);
    $entityId = (int)$_POST['entity_id'];
    $additionalInfo = trim($_POST['additional_info']);

    // Validate inputs
    if (empty($action) || empty($entity) || $entityId <= 0) {
        $errors[] = 'Action, Entity, and valid Entity ID are required.';
    } elseif (strlen($action) > 100) {
        $errors[] = 'Action must be less than 100 characters.';
    } elseif (strlen($entity) > 100) {
        $errors[] = 'Entity must be less than 100 characters.';
    } elseif (strlen($additionalInfo) > 255) {
        $errors[] = 'Additional Info must be less than 255 characters.';
    }

    if (empty($errors)) {
        $userId = $_SESSION['user_id'];
        // Set is_system to 0 for user-created logs
        $result = createAuditLog($pdo, $userId, $action, $entity, $entityId, $additionalInfo, 0);
        if ($result) {
            $success = 'Audit log has been added successfully.';
        } else {
            $errors[] = 'Failed to add audit log.';
        }
    }
}
?>
<h1>Add Audit Log</h1>
<?php if (!empty($errors)): ?>
<div class="alert">
    <?php foreach ($errors as $error): ?>
        <p><?php echo htmlspecialchars($error); ?></p>
    <?php endforeach; ?>
</div>
<?php endif; ?>
<?php if ($success): ?>
<div class="success">
    <p><?php echo htmlspecialchars($success); ?></p>
    <a href="audit_logs.php" class="button">Back to Audit Logs</a>
</div>
<?php else: ?>
<div class="form-container">
    <form action="add_audit_logs.php" method="POST">
        <label for="action">Action</label>
        <input type="text" id="action" name="action" required value="<?php echo isset($_POST['action']) ? htmlspecialchars($_POST['action']) : ''; ?>">

        <label for="entity">Entity</label>
        <input type="text" id="entity" name="entity" required value="<?php echo isset($_POST['entity']) ? htmlspecialchars($_POST['entity']) : ''; ?>">

        <label for="entity_id">Entity ID</label>
        <input type="number" id="entity_id" name="entity_id" required min="1" value="<?php echo isset($_POST['entity_id']) ? htmlspecialchars($_POST['entity_id']) : ''; ?>">

        <label for="additional_info">Additional Info</label>
        <textarea id="additional_info" name="additional_info"><?php echo isset($_POST['additional_info']) ? htmlspecialchars($_POST['additional_info']) : ''; ?></textarea>

        <div class="button-group">
            <button type="submit">Add Audit Log</button>
            <a href="audit_logs.php" class="button">Back</a>
        </div>
    </form>
</div>
<?php endif; ?>
<?php require_once('components/footer.php'); ?>
