<?php
require_once('../core/financial_periods.php');
require_once('components/auth.php');
require_once('components/header.php');
require_once('components/form_component.php');

$errors = [];
$success = '';

if (!checkRole(['financial_period_add'])) {
    $errors[] = $_SESSION['error_message'];
    unset($_SESSION['error_message']);
    ?>
    <h1>Add Financial Period</h1>
    <?php if (!empty($errors)): ?>
        <div class="alert">
            <?php foreach ($errors as $error): ?>
                <p><?php echo htmlspecialchars($error); ?></p>
            <?php endforeach; ?>
        </div>
    <?php endif; ?>
    <?php require_once('components/footer.php'); ?>
    <?php
    exit;
}

if ($_SERVER['REQUEST_METHOD'] === 'POST') {
    $name = trim($_POST['name']);
    $startDate = trim($_POST['start_date']);
    $endDate = trim($_POST['end_date']);

    if (empty($name) || empty($startDate) || empty($endDate)) {
        $errors[] = 'Name, Start Date, and End Date are required.';
    }

    if (strtotime($startDate) === false || strtotime($endDate) === false) {
        $errors[] = 'Invalid date format.';
    }

    if (strtotime($startDate) > strtotime($endDate)) {
        $errors[] = 'Start Date cannot be after End Date.';
    }

    if (empty($errors)) {
        $periodId = createFinancialPeriod($pdo, $name, $startDate, $endDate,$_SESSION['user_id']);
        if ($periodId) {
            $success = 'Financial period added successfully.';
        } else {
            $errors[] = 'Failed to add financial period.';
        }
    }
}

?>
<h1>Add Financial Period</h1>
<?php if (!empty($errors)): ?>
    <div class="alert">
        <?php foreach ($errors as $error): ?>
            <p><?php echo htmlspecialchars($error); ?></p>
        <?php endforeach; ?>
    </div>
<?php endif; ?>
<?php if ($success): ?>
    <div class="success">
        <p><?php echo htmlspecialchars($success); ?></p>
        <a href="financial_periods.php" class="button">Back to Financial Periods</a>
    </div>
<?php else: ?>
    <div class="form-container">
        <form action="add_financial_period.php" method="POST">
            <?php
            renderInput('text', 'name', 'Name', '', true);
            renderInput('date', 'start_date', 'Start Date', '', true);
            renderInput('date', 'end_date', 'End Date', '', true);
            ?>
            <div class="button-group">
                <button type="submit">Add Financial Period</button>
                <a href="financial_periods.php" class="button">Cancel</a>
            </div>
        </form>
    </div>
<?php endif; ?>
<?php require_once('components/footer.php'); ?>

