<?php
require_once('../core/journal_entries.php');
require_once('../core/accounts.php');
require_once('../core/financial_periods.php');
require_once('../core/transactions.php');
require_once('components/auth.php');
require_once('components/header.php');
require_once('components/form_component.php');

$errors = [];
$success = [];
$allowedDateRanges = getAllowedTransactionDateRanges($pdo);

if (!checkRole(['journal_entries_add'])) {
    $errors[] = 'Access denied. You do not have permission to add journal entries.';
    echo '<h1>Add Journal Entry</h1>';
    if (!empty($errors)) {
        echo '<div class="alert">';
        foreach ($errors as $error) {
            echo '<p>' . htmlspecialchars($error) . '</p>';
        }
        echo '</div>';
    }
    require_once('components/footer.php');
    exit;
}

$userId = $_SESSION['user_id'] ?? 0;

if ($_SERVER['REQUEST_METHOD'] === 'POST') {
    $description = trim($_POST['description']);
    $date = trim($_POST['date']);
    $financialPeriodId = (int)$_POST['financial_period_id'];

    // Validate inputs
    if (empty($description) || empty($date) || empty($financialPeriodId)) {
        $errors[] = 'All fields are required.';
    }

    if (empty($errors)) {
        $journalEntryId = createJournalEntry($pdo, $description, $date, $financialPeriodId, $userId);
        if ($journalEntryId) {
            // Redirect to edit page to add transactions
            $_SESSION['success_message'] = 'Journal entry created successfully. Please add transactions.';
            header('Location: edit_journal_entry.php?id=' . $journalEntryId);
            exit;
        } else {
            $errors[] = 'Failed to create journal entry. Please try again.';
        }
    }
}

// Get list of financial periods
$financialPeriods = listFinancialPeriods($pdo);
?>
<h1>Add Journal Entry</h1>
<?php
if (!empty($errors)) {
    echo '<div class="alert">';
    foreach ($errors as $error) {
        echo '<p>' . htmlspecialchars($error) . '</p>';
    }
    echo '</div>';
}
?>
<div class="form-container">
    <form action="add_journal_entry.php" method="POST">
        <?php
        renderInput('text', 'description', 'Description', '', true);
        renderInput('date', 'date', 'Date', date('Y-m-d'), true);

        // Prepare financial periods options
        $financialPeriodOptions = [];
        foreach ($financialPeriods as $period) {
            $financialPeriodOptions[$period['id']] = $period['name'] . ' (' . $period['start_date'] . ' - ' . $period['end_date'] . ')';
        }
        renderSelect('financial_period_id', 'Financial Period', $financialPeriodOptions, '', true);
        ?>
        <div class="button-group">
            <button type="submit">Create Journal Entry</button>
            <a href="journal_entries.php" class="button">Back to Journal Entries</a>
        </div>
    </form>
</div>
<script>
document.addEventListener('DOMContentLoaded', function() {
    var allowedRanges = <?php echo json_encode($allowedDateRanges); ?>;
    function isDateAllowed(dateStr) {
        var test = new Date(dateStr + 'T00:00:00');
        for(var i=0; i<allowedRanges.length; i++){
            var rStart = new Date(allowedRanges[i].start + 'T00:00:00');
            var rEnd   = new Date(allowedRanges[i].end   + 'T00:00:00');
            if(test >= rStart && test <= rEnd){
                return true;
            }
        }
        return false;
    }
    var dateInputs = [
        document.getElementById('date'),
    ].filter(Boolean); // remove null if an element doesn't exist

    dateInputs.forEach(function(inp){
        inp.addEventListener('change', function(){
            var val = inp.value;
            if(val && !isDateAllowed(val)){
                alert('That date is not allowed because the financial period or VAT period is closed.');
                inp.value = ''; // or revert to old value
            }
        });
    });

});
</script>
<?php require_once('components/footer.php'); ?>
