<?php
require_once('../core/users.php');
require_once('components/auth.php');
require_once('components/header.php');

// Initialize session error, success, and form data messages if not set
if (!isset($_SESSION['user_error'])) {
    $_SESSION['user_error'] = [];
}
if (!isset($_SESSION['user_success'])) {
    $_SESSION['user_success'] = '';
}
if (!isset($_SESSION['form_data'])) {
    $_SESSION['form_data'] = [];
}

$errors = [];
$success = '';

// Update role check to 'roles_add'
if (!checkRole(['roles_add'])) {
    $errors[] = $_SESSION['error_message'] ?? 'Unauthorized access.';
    unset($_SESSION['error_message']);
    ?>
    <h1>Add Role</h1>
    <?php if (!empty($errors)): ?>
    <div class="alert">
        <?php foreach ($errors as $error): ?>
            <p><?php echo htmlspecialchars($error); ?></p>
        <?php endforeach; ?>
    </div>
    <?php endif; ?>
    <?php require_once('components/footer.php'); ?>
    <?php
    exit;
}

if ($_SERVER['REQUEST_METHOD'] === 'POST') {
    $roleName = trim($_POST['role_name']);
    if (!empty($roleName)) {
        // Call the updated createRole function
        $createResult = createRole($pdo, $roleName, $_SESSION['user_id']);
        if ($createResult['success']) {
            $_SESSION['success_message'] = "Role '{$roleName}' added successfully.";
            // Clear any previously stored form data
            $_SESSION['form_data'] = [];
            header('Location: roles.php'); // Redirect to prevent form resubmission
            exit;
        } else {
            // Check if the error is due to duplicate role name
            if ($createResult['message'] === 'Role name already exists.') {
                $errors[] = 'Role name already exists.';
            } else {
                $errors[] = $createResult['message'];
            }
            // Store the submitted form data to repopulate the form
            $_SESSION['form_data'] = [
                'role_name' => $roleName
            ];
        }
    } else {
        $errors[] = 'Role name cannot be empty.';
    }

    // If there are errors, store them in the session
    if (!empty($errors)) {
        $_SESSION['user_error'] = $errors;
        header('Location: add_roles.php'); // Redirect to display errors and repopulate the form
        exit;
    }
}
?>
<h1>Add Role</h1>
<?php if (!empty($_SESSION['user_error'])): ?>
<div class="alert">
    <?php foreach ($_SESSION['user_error'] as $error): ?>
        <p><?php echo htmlspecialchars($error); ?></p>
    <?php endforeach; ?>
</div>
<?php
    // Clear the error messages after displaying
    unset($_SESSION['user_error']);
endif;
?>
<?php if (!empty($_SESSION['user_success'])): ?>
<div class="success">
    <p><?php echo htmlspecialchars($_SESSION['user_success']); ?></p>
</div>
<?php
    // Clear the success message after displaying
    unset($_SESSION['user_success']);
endif;
?>
<div class="form-container">
    <form action="add_roles.php" method="POST">
        <label for="role_name">Role Name</label>
        <input type="text" id="role_name" name="role_name" required value="<?php echo isset($_SESSION['form_data']['role_name']) ? htmlspecialchars($_SESSION['form_data']['role_name']) : ''; ?>">
        <div class="button-group">
            <button type="submit" name="add_role">Add Role</button>
            <a href="roles.php" class="button">Back</a>
        </div>
    </form>
</div>
<?php
    // Clear the form data after repopulating the form
    unset($_SESSION['form_data']);
?>
<?php require_once('components/footer.php'); ?>
