<?php
require_once('../core/vat_periods.php');
require_once('components/auth.php');
require_once('components/header.php');

$errors = [];
$success = '';

if (!checkRole(['vat_period_add'])) {
    $errors[] = 'Access denied. You do not have permission to add VAT periods.';
    ?>
    <h1>Add VAT Period</h1>
    <?php if (!empty($errors)): ?>
    <div class="alert">
        <?php foreach ($errors as $error): ?>
        <p><?php echo htmlspecialchars($error); ?></p>
        <?php endforeach; ?>
    </div>
    <?php endif; ?>
    <?php
    require_once('components/footer.php');
    exit;
}

if ($_SERVER['REQUEST_METHOD'] === 'POST') {
    $name = trim($_POST['name']);
    $start_date = trim($_POST['start_date']);
    $end_date = trim($_POST['end_date']);

    if (empty($name) || empty($start_date) || empty($end_date)) {
        $errors[] = 'All fields are required.';
    } elseif (strtotime($start_date) > strtotime($end_date)) {
        $errors[] = 'Start date cannot be after end date.';
    }

    // Check for overlapping VAT periods
    $existingPeriods = listVATPeriods($pdo);
    foreach ($existingPeriods as $period) {
        if ((strtotime($start_date) <= strtotime($period['end_date'])) && (strtotime($end_date) >= strtotime($period['start_date']))) {
            $errors[] = 'The date range overlaps with an existing VAT period.';
            break;
        }
    }

    if (empty($errors)) {
        $vatPeriodId = createVATPeriod($pdo, $name, $start_date, $end_date,$_SESSION['user_id']);
        if ($vatPeriodId) {
            $_SESSION['success_message'] = 'VAT Period added successfully.';
            header('Location: vat_periods.php');
            exit;
        } else {
            $errors[] = 'Failed to add VAT Period.';
        }
    }
}

?>

<h1>Add VAT Period</h1>
<?php if (!empty($errors)): ?>
<div class="alert">
    <?php foreach ($errors as $error): ?>
    <p><?php echo htmlspecialchars($error); ?></p>
    <?php endforeach; ?>
</div>
<?php endif; ?>
<div class="form-container">
    <form action="add_vat_periods.php" method="POST">
        <label for="name">VAT Period Name</label>
        <input type="text" id="name" name="name" required>

        <label for="start_date">Start Date</label>
        <input type="date" id="start_date" name="start_date" required>

        <label for="end_date">End Date</label>
        <input type="date" id="end_date" name="end_date" required>

        <div class="button-group">
            <button type="submit">Add VAT Period</button>
            <a href="vat_periods.php" class="button">Back</a>
        </div>
    </form>
</div>
<?php require_once('components/footer.php'); ?>

