<?php
require_once('../core/vat_rates.php');
require_once('components/auth.php');
require_once('components/header.php');

$errors = [];
$success = '';

// Check if the user has the required role
if (!checkRole(['vat_rates_add'])) {
    $errors[] = $_SESSION['error_message'];
    unset($_SESSION['error_message']);
    ?>
    <h1>Add VAT Rate</h1>
    <?php if (!empty($errors)): ?>
        <div class="alert">
            <?php foreach ($errors as $error): ?>
                <p><?php echo htmlspecialchars($error); ?></p>
            <?php endforeach; ?>
        </div>
    <?php endif; ?>
    <?php require_once('components/footer.php'); ?>
    <?php
    exit;
}

// Retrieve the user ID from the session
$userId = $_SESSION['user_id'];

if ($_SERVER['REQUEST_METHOD'] === 'POST') {
    $rateName = trim($_POST['rate_name']);
    $rateValue = trim($_POST['rate_value']);
    $isDefault = isset($_POST['is_default']) ? 1 : 0;

    // Validate form inputs
    if (empty($rateName)) {
        $errors[] = 'Rate Name is required.';
    }

    if (!is_numeric($rateValue)) {
        $errors[] = 'Rate Value must be a numeric value.';
    }

    if (empty($errors)) {
        // Call the createVATRate function with the required parameters
        $vatRateId = createVATRate($pdo, $rateName, $rateValue, $userId, $isDefault);
        if ($vatRateId) {
            $_SESSION['success_message'] = 'VAT Rate added successfully.';
            header('Location: vat_rates.php');
            exit;
        } else {
            // Retrieve and display any error messages from the session
            if (isset($_SESSION['vat_rate_error']) && !empty($_SESSION['vat_rate_error'])) {
                $errors[] = $_SESSION['vat_rate_error'];
                unset($_SESSION['vat_rate_error']);
            } else {
                $errors[] = 'An error occurred while adding the VAT Rate. Please try again.';
            }
        }
    }
}
?>
<h1>Add VAT Rate</h1>
<?php if (!empty($errors)): ?>
    <div class="alert">
        <?php foreach ($errors as $error): ?>
            <p><?php echo htmlspecialchars($error); ?></p>
        <?php endforeach; ?>
    </div>
<?php endif; ?>
<div class="form-container">
    <form action="add_vat_rates.php" method="POST">
        <label for="rate_name">Rate Name</label>
        <input type="text" id="rate_name" name="rate_name" value="<?php echo isset($_POST['rate_name']) ? htmlspecialchars($_POST['rate_name']) : ''; ?>" required>
        <label for="rate_value">Rate Value (%)</label>
        <input type="text" id="rate_value" name="rate_value" value="<?php echo isset($_POST['rate_value']) ? htmlspecialchars($_POST['rate_value']) : ''; ?>" required>
        <label for="is_default">Set as Default</label>
        <input type="checkbox" id="is_default" name="is_default" <?php echo isset($_POST['is_default']) ? 'checked' : ''; ?>>
        <div class="button-group">
            <button type="submit">Add VAT Rate</button>
            <a href="vat_rates.php" class="button">Back</a>
        </div>
    </form>
</div>
<?php require_once('components/footer.php'); ?>
