<?php
// public/components/form_component.php

/**
 * Renders a select dropdown with optional additional attributes.
 *
 * @param string $name The name attribute of the select.
 * @param string $label The label text for the select.
 * @param array $options The options for the select (value => display text).
 * @param string $selected The selected value.
 * @param bool $required Whether the select is required.
 * @param array $attributes Additional HTML attributes (e.g., ['disabled' => 'disabled']).
 */
function renderSelect($name, $label, $options, $selected = '', $required = false, $attributes = []) {
    echo '<label for="' . htmlspecialchars($name) . '">' . htmlspecialchars($label) . '</label>';
    echo '<select id="' . htmlspecialchars($name) . '" name="' . htmlspecialchars($name) . '"' . ($required ? ' required' : '');

    // Add additional attributes
    foreach ($attributes as $attr => $value) {
        echo ' ' . htmlspecialchars($attr) . '="' . htmlspecialchars($value) . '"';
    }

    echo '>';
    echo '<option value="">Select ' . htmlspecialchars($label) . '</option>';
    foreach ($options as $value => $text) {
        $isSelected = ($value == $selected) ? ' selected' : '';
        echo '<option value="' . htmlspecialchars($value) . '"' . $isSelected . '>' . htmlspecialchars($text) . '</option>';
    }
    echo '</select>';
}

function renderInput($type, $name, $label, $value = '', $required = false, $attributes = []) {
    echo '<label for="' . htmlspecialchars($name) . '">' . htmlspecialchars($label) . '</label>';
    echo '<input type="' . htmlspecialchars($type) . '" id="' . htmlspecialchars($name) . '" name="' . htmlspecialchars($name) . '" value="' . htmlspecialchars($value) . '"' . ($required ? ' required' : '');

    // Add any additional attributes like step or min.
    foreach ($attributes as $key => $attrValue) {
        echo ' ' . htmlspecialchars($key) . '="' . htmlspecialchars($attrValue) . '"';
    }

    echo '>';
}

function renderTextarea($name, $label, $value = '', $required = false) {
    echo '<label for="' . htmlspecialchars($name) . '">' . htmlspecialchars($label) . '</label>';
    echo '<textarea id="' . htmlspecialchars($name) . '" name="' . htmlspecialchars($name) . '"' . ($required ? ' required' : '') . '>' . htmlspecialchars($value) . '</textarea>';
}

function renderCheckbox($name, $label, $checked = false) {
    echo '<label><input type="checkbox" name="' . htmlspecialchars($name) . '"' . ($checked ? ' checked' : '') . '> ' . htmlspecialchars($label) . '</label>';
}
?>
