<?php
// /public/dashboard/income_statement_widget.php

ini_set('display_errors', 1);
ini_set('display_startup_errors', 1);
error_reporting(E_ALL);

require_once(__DIR__ . '/../../core/financial_reports.php');
require_once(__DIR__ . '/../../core/accounts.php');
require_once(__DIR__ . '/../../core/financial_periods.php');
require_once(__DIR__ . '/../../config/config.php');

// We'll replicate the logic to get total Revenue vs. Expense for a default date range
$startDate = '2024-01-01';
$endDate   = date('Y-m-d');

// This function might be named differently: "generateIncomeStatementReport" or "generateIncomeStatement"
$incomeStatementData = generateIncomeStatementReport($pdo, $startDate, $endDate);

// Tally up totals
$totalIncome = 0;
$totalExpense= 0;
if (!empty($incomeStatementData)) {
    foreach ($incomeStatementData as $row) {
        $acctType = strtolower($row['account_type'] ?? '');
        $amt = floatval($row['total_amount'] ?? 0);
        if ($acctType === 'income' || $acctType === 'revenue') {
            $totalIncome += $amt;
        } elseif ($acctType === 'expense') {
            $totalExpense += $amt;
        }
    }
}

// If both 0, show a 50-50 pie
$pieIncome  = $totalIncome;
$pieExpense = $totalExpense;
if ($pieIncome == 0 && $pieExpense == 0) {
    $pieIncome  = 0.5;
    $pieExpense = 0.5;
}
?>
<div style="width:300px; height:300px;">
  <canvas id="incomeExpensePie"></canvas>
  <div id="widgetTotals" style="font-size: 0.85em; margin-top:5px;">
    Income: <?php echo number_format($totalIncome, 2); ?> |
    Expenses: <?php echo number_format($totalExpense, 2); ?>
    <?php if ($totalIncome == 0 && $totalExpense == 0): ?>
      <br>(No transactions found. Pie is 50/50 fallback.)
    <?php endif; ?>
  </div>
</div>

<!-- Chart.js for pie chart -->
<script src="https://cdn.jsdelivr.net/npm/chart.js@3.9.1/dist/chart.min.js"></script>
<script>
(function(){
    const ctx = document.getElementById('incomeExpensePie').getContext('2d');
    const data = {
      labels: ['Income', 'Expenses'],
      datasets: [{
        data: [<?php echo $pieIncome; ?>, <?php echo $pieExpense; ?>],
        backgroundColor: ['#2ecc71','#e74c3c']
      }]
    };
    new Chart(ctx, {
      type: 'pie',
      data: data,
      options: {
        responsive: true,
        maintainAspectRatio: false
      }
    });
})();
</script>
