<?php
ini_set('display_errors', 1);
ini_set('display_startup_errors', 1);
error_reporting(E_ALL);
require_once('../core/accounts.php');
require_once('components/auth.php');

// Add role check for 'accounts_delete'
if (!checkRole(['accounts_delete'])) {
    $_SESSION['error_message'] = $_SESSION['error_message'] ?? 'Access denied. You do not have permission to delete accounts.';
    header('Location: accounts.php');
    exit;
}

if (!isset($_GET['id']) || empty(trim($_GET['id']))) {
    $_SESSION['error_message'] = "Invalid account ID.";
    header('Location: accounts.php');
    exit;
}
$code = trim($_GET['id']);
$account = getAccount($pdo, $code);
if (!$account) {
    $_SESSION['error_message'] = "Account with code [$code] does not exist.";
    header('Location: accounts.php');
    exit;
}
if ($account['is_system'] == 1) {
    $_SESSION['error_message'] = "System accounts cannot be deleted.";
    header('Location: accounts.php');
    exit;
}
$hasChildren = checkIfAccountHasChildren($pdo, $code);
if ($hasChildren) {
    $_SESSION['error_message'] = "Cannot delete account [$code] {$account['name']} because it has child accounts.";
    header('Location: accounts.php');
    exit;
}
$hasTransactions = checkIfAccountHasTransactions($pdo, $code);
if ($hasTransactions) {
    $_SESSION['error_message'] = "Cannot delete account [$code] {$account['name']} because it has associated transactions.";
    header('Location: accounts.php');
    exit;
}
$deleted = deleteAccount($pdo, $code,$_SESSION['user_id']);
if ($deleted) {
    $_SESSION['success_message'] = "Account [{$account['code']}] {$account['name']} has been successfully deleted!";
} else {
    $_SESSION['error_message'] = "Failed to delete account [{$account['code']}] {$account['name']}. Please try again.";
}
header('Location: accounts.php');
exit;

function checkIfAccountHasChildren(PDO $pdo, string $code): bool {
    try {
        $stmt = $pdo->prepare("SELECT COUNT(*) FROM accounts WHERE parent=?");
        $stmt->execute([$code]);
        $count = $stmt->fetchColumn();
        return $count > 0;
    } catch (PDOException $e) {
        error_log('Check Children Accounts Error: ' . $e->getMessage(), 3, LOG_FILE);
        return false;
    }
}

function checkIfAccountHasTransactions(PDO $pdo, string $code): bool {
    try {
        $stmt = $pdo->prepare("SELECT COUNT(*) FROM transactions WHERE debit_account=? OR credit_account=?");
        $stmt->execute([$code, $code]);
        $count = $stmt->fetchColumn();
        return $count > 0;
    } catch (PDOException $e) {
        error_log('Check Transactions Error: ' . $e->getMessage(), 3, LOG_FILE);
        return false;
    }
}
?>
