<?php
ini_set('display_errors', 1);
ini_set('display_startup_errors', 1);
error_reporting(E_ALL);
require_once('../config/config.php');
require_once('components/auth.php');

try {
    $pdo = new PDO('sqlite:' . DB_FILE);
    $pdo->setAttribute(PDO::ATTR_ERRMODE, PDO::ERRMODE_EXCEPTION);
} catch (PDOException $e) {
    die("DB Error: " . $e->getMessage());
}

$layoutDir = __DIR__ . '/documents';

function listLayoutFiles($dir) {
    $out = [];
    if (is_dir($dir)) {
        $scan = scandir($dir);
        foreach ($scan as $f) {
            if (strtolower(pathinfo($f, PATHINFO_EXTENSION)) === 'layout') {
                $out[] = $f;
            }
        }
    }
    return $out;
}

// Handle AJAX actions
if (isset($_GET['action'])) {
    if ($_GET['action'] === 'saveLayout') {
        $raw = file_get_contents('php://input');
        $res = ['success' => false, 'message' => 'Unknown error', 'filename' => ''];
        if ($raw) {
            $json = json_decode($raw, true);
            if (!empty($json['filename']) && !empty($json['data'])) {
                if (empty($json['data']['type'])) {
                    $res['message'] = 'Cannot save: no type selected.';
                    header('Content-Type: application/json');
                    echo json_encode($res);
                    exit;
                }
                $filename = basename($json['filename']);
                if (strtolower(substr($filename, -7)) !== '.layout') {
                    $filename .= '.layout';
                }
                $savePath = $layoutDir . '/' . $filename;
                $encoded  = json_encode($json['data'], JSON_PRETTY_PRINT);
                if (@file_put_contents($savePath, $encoded) !== false) {
                    $res['success']  = true;
                    $res['message']  = 'Saved successfully.';
                    $res['filename'] = $filename;
                } else {
                    $res['message'] = 'Failed to write file.';
                }
            } else {
                $res['message'] = 'Bad JSON data.';
            }
        } else {
            $res['message'] = 'No input.';
        }
        header('Content-Type: application/json');
        echo json_encode($res);
        exit;
    }
    if ($_GET['action'] === 'listImages') {
        $imgDir = __DIR__ . '/documents/images';
        $files  = [];
        if (is_dir($imgDir)) {
            $scan = scandir($imgDir);
            foreach ($scan as $f) {
                if ($f[0] === '.') continue;
                $ext = strtolower(pathinfo($f, PATHINFO_EXTENSION));
                if (in_array($ext, ['jpg','jpeg','png','gif'])) {
                    $files[] = $f;
                }
            }
        }
        header('Content-Type: application/json');
        echo json_encode($files);
        exit;
    }
    if ($_GET['action'] === 'deleteLayout') {
        $res = ['success' => false, 'message' => 'Unknown', 'filename' => ''];
        $layoutName = isset($_GET['filename']) ? basename($_GET['filename']) : '';
        if (!$layoutName) {
            $res['message'] = 'No filename.';
        } else {
            $fullPath = $layoutDir . '/' . $layoutName;
            if (file_exists($fullPath)) {
                if (@unlink($fullPath)) {
                    $res['success']  = true;
                    $res['message']  = 'Deleted layout.';
                    $res['filename'] = $layoutName;
                } else {
                    $res['message'] = 'Failed to delete.';
                }
            } else {
                $res['message'] = 'Layout not found.';
            }
        }
        header('Content-Type: application/json');
        echo json_encode($res);
        exit;
    }
    if ($_GET['action'] === 'listTypes') {
        $baseDir = __DIR__ . '/documents/type';
        $types = [];
        if (is_dir($baseDir)) {
            $folders = scandir($baseDir);
            foreach ($folders as $fld) {
                if ($fld === '.' || $fld === '..') continue;
                $typeJson = $baseDir . '/' . $fld . '/type.json';
                if (is_dir($baseDir . '/' . $fld) && file_exists($typeJson)) {
                    $info = json_decode(file_get_contents($typeJson), true);
                    if (!empty($info['name'])) {
                        $types[] = [
                            'folderName' => $fld,
                            'name'       => $info['name']
                        ];
                    }
                }
            }
        }
        header('Content-Type: application/json');
        echo json_encode($types);
        exit;
    }
    if ($_GET['action'] === 'showTypeDialog') {
        $typeFolder = isset($_GET['typeFolder']) ? basename($_GET['typeFolder']) : '';
        if (!$typeFolder) {
            echo json_encode(['success'=>false,'message'=>'No type folder specified']);
            exit;
        }
        $dialogFile = __DIR__ . "/documents/type/$typeFolder/dialog.json";
        if (!file_exists($dialogFile)) {
            echo json_encode(['success'=>false,'message'=>'No dialog.json found']);
            exit;
        }
        $dialogData = json_decode(file_get_contents($dialogFile), true);
        if (!$dialogData) {
            echo json_encode(['success'=>false,'message'=>'dialog.json invalid']);
            exit;
        }
        // OPTIONAL: load external CSS
        $dialogCss = '';
        $cssFile = __DIR__ . "/documents/type/$typeFolder/dialog.css";
        if (file_exists($cssFile)) {
            $dialogCss = file_get_contents($cssFile);
        }
        echo json_encode(['success'=>true,'dialog'=>$dialogData, 'dialogCss'=>$dialogCss]);
        exit;
    }
}

require_once('components/header.php');
$layoutFiles = listLayoutFiles($layoutDir);
?>
<div id="main-container">
  <div class="toolbar" id="toolbar">
    <button class="minimize-button" id="toolbar-toggle"><</button>
    <div id="toolbar-contents" style="padding:10px;">
      <button data-element-type="Line">Line</button>
      <button data-element-type="Rect">Rect</button>
      <button data-element-type="Cell">Cell</button>
      <button data-element-type="MultiCell">MultiCell</button>
      <button data-element-type="Image">Image</button>
      <button data-element-type="Text">Text</button>
      <hr style="margin-bottom:7px;">
      <!-- Type-specific toolbar buttons get appended here -->
    </div>
  </div>
  <div class="center-area">
    <div class="top-bar">
      <div class="top-bar-row">
        <label for="type-select">Type:</label>
        <select id="type-select">
          <option value="">(none)</option>
        </select>

        <label for="layout-select">Layout:</label>
        <select id="layout-select">
          <option value="">-- Select Layout --</option>
          <?php foreach($layoutFiles as $lf): ?>
            <option value="<?php echo htmlspecialchars($lf); ?>">
              <?php echo htmlspecialchars($lf); ?>
            </option>
          <?php endforeach; ?>
        </select>

        <label for="page-size-select">Size:</label>
        <select id="page-size-select">
          <option value="A4">A4</option>
          <option value="A3">A3</option>
          <option value="Letter">Letter</option>
          <option value="Custom">Custom (210x297)</option>
        </select>

        <label for="orientation-select">Orientation:</label>
        <select id="orientation-select">
          <option value="P">Portrait</option>
          <option value="L">Landscape</option>
        </select>

        <button id="zoom-out">-</button>
        <span id="zoom-level">100%</span>
        <button id="zoom-in">+</button>
      </div>
      <div class="top-bar-row">
        <button id="btn-new-save" disabled>New Save</button>
        <button id="btn-delete-layout" disabled>Delete Layout</button>
        <button id="btn-save" disabled>Save Layout</button>
        <button id="btn-generate" disabled>Generate PDF</button>
        <button id="btn-image-management" disabled>Image Management</button>
      </div>
    </div>

    <div class="editor" id="editor">
      <div id="page"></div>
    </div>
  </div>

  <div class="property-box" id="property-box">
    <button class="minimize-button" id="property-toggle">></button>
    <h3>Properties</h3>
    <label for="object-list">Object List:</label>
    <select id="object-list">
      <option value="">(none)</option>
    </select>
    <div id="no-selection" style="display:block; font-style:italic;">( no object selected )</div>

    <div id="line-properties" style="display:none;">
      <label>X1:</label><input type="number" id="line-x1" step="0.01">
      <label>Y1:</label><input type="number" id="line-y1" step="0.01">
      <label>X2:</label><input type="number" id="line-x2" step="0.01">
      <label>Y2:</label><input type="number" id="line-y2" step="0.01">
      <label>LineWidth:</label><input type="number" id="line-width" step="0.01">
      <label>Color:</label><input type="color" id="line-color">
      <label>Bind:</label>
      <select id="line-bind">
        <option value="none">none</option>
        <option value="header">header</option>
        <option value="footer">footer</option>
        <option value="watermark">watermark</option>
      </select>
    </div>

    <div id="rect-properties" style="display:none;">
      <label>X:</label><input type="number" id="rect-x" step="0.01">
      <label>Y:</label><input type="number" id="rect-y" step="0.01">
      <label>W:</label><input type="number" id="rect-w" step="0.01">
      <label>H:</label><input type="number" id="rect-h" step="0.01">
      <label>Style:</label>
      <select id="rect-style">
        <option value="D">D (draw)</option>
        <option value="F">F (fill)</option>
        <option value="DF">DF (draw+fill)</option>
      </select>
      <label>LineWidth:</label><input type="number" id="rect-linewidth" step="0.01">
      <label>DrawColor:</label><input type="color" id="rect-drawcolor">
      <label>FillColor:</label><input type="color" id="rect-fillcolor">
      <label>Bind:</label>
      <select id="rect-bind">
        <option value="none">none</option>
        <option value="header">header</option>
        <option value="footer">footer</option>
        <option value="watermark">watermark</option>
      </select>
    </div>

    <div id="cell-properties" style="display:none;">
      <label>X:</label><input type="number" id="cell-x" step="0.01">
      <label>Y:</label><input type="number" id="cell-y" step="0.01">
      <label>W:</label><input type="number" id="cell-w" step="0.01">
      <label>H:</label><input type="number" id="cell-h" step="0.01">
      <label>Txt:</label><input type="text" id="cell-txt">
      <label>Border sides:</label>
      <div style="display:flex; gap:10px; margin-bottom:5px;">
        <label><input type="checkbox" id="cell-border-l">L</label>
        <label><input type="checkbox" id="cell-border-t">T</label>
        <label><input type="checkbox" id="cell-border-r">R</label>
        <label><input type="checkbox" id="cell-border-b">B</label>
      </div>
      <label>ln:</label>
      <select id="cell-ln">
        <option value="0">0 (Right)</option>
        <option value="1">1 (Next line)</option>
        <option value="2">2 (Below)</option>
      </select>
      <label>Align:</label>
      <select id="cell-align">
        <option value="">L (Left)</option>
        <option value="C">C (Center)</option>
        <option value="R">R (Right)</option>
      </select>
      <label>Fill:</label><input type="checkbox" id="cell-fill">
      <label>Font Family:</label>
      <select id="cell-fontfam">
        <option value="Courier">Courier</option>
        <option value="Helvetica">Helvetica / Arial</option>
        <option value="Times">Times</option>
        <option value="Symbol">Symbol</option>
        <option value="ZapfDingbats">ZapfDingbats</option>
      </select>
      <label>Font Style:</label>
      <select id="cell-fontsty">
        <option value="">Regular</option>
        <option value="B">Bold</option>
        <option value="I">Italic</option>
        <option value="U">Underline</option>
      </select>
      <label>FontSz:</label><input type="number" id="cell-fontsz" step="0.5">
      <label>DrawColor:</label><input type="color" id="cell-drawcolor">
      <label>FillColor:</label><input type="color" id="cell-fillcolor">
      <label>TextColor:</label><input type="color" id="cell-textcolor">
      <label>LineWidth:</label><input type="number" id="cell-linewidth" step="0.01">
      <label>Bind:</label>
      <select id="cell-bind">
        <option value="none">none</option>
        <option value="header">header</option>
        <option value="footer">footer</option>
        <option value="watermark">watermark</option>
      </select>
    </div>

    <div id="multicell-properties" style="display:none;">
      <label>X:</label><input type="number" id="mc-x" step="0.01">
      <label>Y:</label><input type="number" id="mc-y" step="0.01">
      <label>W:</label><input type="number" id="mc-w" step="0.01">
      <label>H:</label><input type="number" id="mc-h" step="0.01">
      <label>Txt:</label><input type="text" id="mc-txt">
      <label>Border sides:</label>
      <div style="display:flex; gap:10px; margin-bottom:5px;">
        <label><input type="checkbox" id="mc-border-l">L</label>
        <label><input type="checkbox" id="mc-border-t">T</label>
        <label><input type="checkbox" id="mc-border-r">R</label>
        <label><input type="checkbox" id="mc-border-b">B</label>
      </div>
      <label>Align:</label>
      <select id="mc-align">
        <option value="">L</option>
        <option value="C">C</option>
        <option value="R">R</option>
      </select>
      <label>Fill:</label><input type="checkbox" id="mc-fill">
      <label>Font Family:</label>
      <select id="mc-fontfam">
        <option value="Courier">Courier</option>
        <option value="Helvetica">Helvetica / Arial</option>
        <option value="Times">Times</option>
        <option value="Symbol">Symbol</option>
        <option value="ZapfDingbats">ZapfDingbats</option>
      </select>
      <label>Font Style:</label>
      <select id="mc-fontsty">
        <option value="">Regular</option>
        <option value="B">Bold</option>
        <option value="I">Italic</option>
        <option value="U">Underline</option>
      </select>
      <label>FontSz:</label><input type="number" id="mc-fontsz" step="0.5">
      <label>DrawColor:</label><input type="color" id="mc-drawcolor">
      <label>FillColor:</label><input type="color" id="mc-fillcolor">
      <label>TextColor:</label><input type="color" id="mc-textcolor">
      <label>LineWidth:</label><input type="number" id="mc-linewidth" step="0.01">
      <label>Bind:</label>
      <select id="mc-bind">
        <option value="none">none</option>
        <option value="header">header</option>
        <option value="footer">footer</option>
        <option value="watermark">watermark</option>
      </select>
    </div>

    <div id="image-properties" style="display:none;">
      <label>File:</label><select id="image-file"></select>
      <label>X:</label><input type="number" id="image-x" step="0.01">
      <label>Y:</label><input type="number" id="image-y" step="0.01">
      <label>W:</label><input type="number" id="image-w" step="0.01">
      <label>H:</label><input type="number" id="image-h" step="0.01">
      <label>Type:</label>
      <select id="image-type">
        <option value="">Auto</option>
        <option value="JPG">JPG</option>
        <option value="PNG">PNG</option>
        <option value="GIF">GIF</option>
      </select>
      <label>Bind:</label>
      <select id="image-bind">
        <option value="none">none</option>
        <option value="header">header</option>
        <option value="footer">footer</option>
        <option value="watermark">watermark</option>
      </select>
    </div>

    <div id="text-properties" style="display:none;">
      <label>X:</label><input type="number" id="text-x" step="0.01">
      <label>Y:</label><input type="number" id="text-y" step="0.01">
      <label>Text:</label><input type="text" id="text-value">

      <label>Font Family:</label>
      <select id="text-fontfam">
        <option value="Courier">Courier</option>
        <option value="Helvetica">Helvetica / Arial</option>
        <option value="Times">Times</option>
        <option value="Symbol">Symbol</option>
        <option value="ZapfDingbats">ZapfDingbats</option>
      </select>
      <label>Font Style:</label>
      <select id="text-fontsty">
        <option value="">Regular</option>
        <option value="B">Bold</option>
        <option value="I">Italic</option>
        <option value="U">Underline</option>
      </select>

      <label>FontSz:</label><input type="number" id="text-fontsz" step="0.5">
      <label>Color:</label><input type="color" id="text-color">
      <label>Bind:</label>
      <select id="text-bind">
        <option value="none">none</option>
        <option value="header">header</option>
        <option value="footer">footer</option>
        <option value="watermark">watermark</option>
      </select>
    </div>

    <div style="display: flex; gap: 10px;" id="prop-buttons">
      <button id="prop-delete">Delete</button>
    </div>
  </div>
</div>

<div id="type-dialog-modal" style="display:none; position:fixed; z-index:1000; left:0; top:0; width:100%; height:100%; background-color:rgba(0,0,0,0.5); align-items:center; justify-content:center;">
  <div id="type-dialog-inner" style="background:#fff; padding:20px; border-radius:4px; min-width:300px;">
    <h2 id="type-dialog-title">Parameters</h2>
    <div id="type-dialog-body"></div>
    <div style="margin-top:15px; text-align:right;">
      <button id="type-dialog-cancel">Cancel</button>
      <button id="type-dialog-ok">Generate</button>
    </div>
  </div>
</div>

<style>
html, body {
  min-height:700px;
  min-width:1200px;
  margin:0;
  padding:0;
  box-sizing:border-box;
}
#main-container {
  display:flex;
  flex-direction:row;
  width:100%;
  height:calc(100vh - 130px);
  overflow:hidden;
}
.toolbar {
  width:180px;
  min-width:30px;
  background:#f9f9f9;
  border-right:1px solid #ccc;
  overflow:hidden;
  display:flex;
  flex-direction:column;
  height:100%;
}
.toolbar button {
  margin-bottom:6px;
  width:100%;
  cursor:pointer;
}
.center-area {
  display:flex;
  flex-direction:column;
  flex:1;
}
.top-bar {
  width:100%;
  background:#f4f4f4;
  border-bottom:1px solid #ccc;
  display:flex;
  flex-direction:column;
  gap:8px;
  padding:10px;
}
.top-bar-row {
  display:flex;
  align-items:center;
  gap:8px;
  flex-wrap:wrap;
}
.editor {
  flex:1;
  position:relative;
  display:flex;
  background:#ddd;
  overflow:auto;
  padding:10px;
}
#page {
  background:#fff;
  border:1px solid #999;
  transform-origin:top left;
  position:relative;
  margin:40px;
}
.property-box {
  width:220px;
  min-width:30px;
  background:#f9f9f9;
  border-left:1px solid #ccc;
  overflow:auto;
  display:flex;
  flex-direction:column;
  box-sizing:border-box;
  padding:10px;
  gap:8px;
  align-items:flex-start;
  height:100%;
}
.property-box label {
  text-align:right;
  padding-right:5px;
}
.property-box input,
.property-box select {
  width:100%;
}
.property-box button {
  width:100px;
  margin-top:10px;
  cursor:pointer;
}
button#btn-new-save:not(:disabled) {
  background: #66dd66;
}
button#btn-delete-layout:not(:disabled) {
  background: #ff6666;
}
button#btn-save:not(:disabled) {
  background: #66dd66;
}
button#btn-generate:not(:disabled) {
  background: orange;
}
button#btn-image-management:not(:disabled) {
  background: #1e90ff;
}
button:disabled {
  background: #ccc;
  color: #666;
  cursor: not-allowed;
}
.minimize-button {
  align-self:flex-end;
  background:#ddd;
  border:none;
  cursor:pointer;
  padding:2px 6px;
  margin:4px;
  font-size:12px;
}
.element {
  position:absolute;
  border:1px dashed #aaa;
  cursor:move;
  padding:2px;
  font-size:12px;
  background:rgba(255,255,255,0.7);
}
.line-handle {
  position:absolute;
  width:8px;
  height:8px;
  background:#ff3333;
  border-radius:50%;
  cursor:pointer;
}
.line-body {
  pointer-events:none;
}
.resize-br-handle {
  position:absolute;
  width:10px;
  height:10px;
  right:-5px;
  bottom:-5px;
  background:#00f;
  border:1px solid #fff;
  cursor:nwse-resize;
  z-index:99;
}
</style>
<script>
let isDragging=false, offsetX=0, offsetY=0;
let activeElementDiv=null, activeElementIndex=-1;
let scale=1.0;
let draggingLineEnd=null;
let resizingElementIndex=-1, resizingCorner='';
let nextObjectId=1;

// Type data
let loadedType=null;
let oldType=null;
let currentTypeConfig=null;

let lastTypedObjectId = null;
let lastTypedObjectType = null;

let currentLayout={
  pageWidth:210,
  pageHeight:297,
  orientation:'P',
  elements:[],
  type:'',
  dialogData:{}
};

const layoutSelect=document.getElementById('layout-select');
const pageSizeSelect=document.getElementById('page-size-select');
const orientationSel=document.getElementById('orientation-select');
const pageDiv=document.getElementById('page');
const zoomLevelSpan=document.getElementById('zoom-level');
const zoomInBtn=document.getElementById('zoom-in');
const zoomOutBtn=document.getElementById('zoom-out');
const toolbar=document.getElementById('toolbar');
const toolbarToggle=document.getElementById('toolbar-toggle');
let toolbarMinimized=false;
const propertyBox=document.getElementById('property-box');
const propertyToggle=document.getElementById('property-toggle');
let propertyMinimized=false;
const btnImageMgmt=document.getElementById('btn-image-management');
const btnSave=document.getElementById('btn-save');
const btnGenerate=document.getElementById('btn-generate');
const btnNewSave=document.getElementById('btn-new-save');
const btnDeleteLayout=document.getElementById('btn-delete-layout');
const objectListSel=document.getElementById('object-list');
const propDelete=document.getElementById('prop-delete');
const typeDialogModal=document.getElementById('type-dialog-modal');
const typeDialogInner=document.getElementById('type-dialog-inner');
const typeDialogTitle=document.getElementById('type-dialog-title');
const typeDialogBody=document.getElementById('type-dialog-body');
const typeDialogCancel=document.getElementById('type-dialog-cancel');
const typeDialogOk=document.getElementById('type-dialog-ok');

window.typeObjectCreators=window.typeObjectCreators||{};
let unsavedChanges=false;
let pendingProps={};

// A helper function to remove only typed objects from the old type
function removeOldTypedObjects(oldT) {
  if(!oldT) return;
  currentLayout.elements = currentLayout.elements.filter(el=>{
    return ['Line','Rect','Cell','MultiCell','Image','Text'].includes(el.type);
  });
  drawElements();
  refreshObjectList();
}

// Load all available "types"
document.addEventListener('DOMContentLoaded',()=>{
  loadAvailableTypes();
});
async function loadAvailableTypes(){
  try{
    const resp=await fetch('document_editor.php?action=listTypes&_='+Date.now());
    const data=await resp.json();
    const typeSelect=document.getElementById('type-select');
    data.forEach(t=>{
      const opt=document.createElement('option');
      opt.value=t.folderName;
      opt.textContent=t.name;
      typeSelect.appendChild(opt);
    });
  }catch(err){
    console.error('Error loading types:',err);
  }
}

// Programmatically load a type without user prompt
async function loadTypeProgrammatically(newType){
  if(!newType) return;
  if(loadedType && loadedType!==newType){
    oldType=loadedType;
    removeOldTypedObjects(oldType);
  }
  try{
    const resp=await fetch(`documents/type/${newType}/type.json?_=`+Date.now());
    const cfg=await resp.json();
    loadedType=newType;
    currentTypeConfig=cfg;
    currentLayout.type=newType;
    await loadTypeToolbar(cfg.toolbar);
    if(cfg.objectsJs){
      await loadTypeObjectsJs(newType,cfg.objectsJs);
    }
  } catch(err){
    console.error('loadTypeProgrammatically Error:', err);
  }
}

document.getElementById('type-select').addEventListener('change', async function(){
  const newType=this.value.trim();
  // Function to remove type-specific toolbar buttons
  function clearTypeToolbar() {
    const toolbarContents = document.getElementById('toolbar-contents');
    const hrIndex = [...toolbarContents.children].findIndex(el => el.tagName === 'HR');
    while (toolbarContents.children.length > hrIndex + 1) {
      toolbarContents.removeChild(toolbarContents.lastChild);
    }
  }
  if(!newType){
    if(loadedType){
      if (!confirm("Switching to no type will remove all objects from type '" + loadedType + "'. Continue?")) {
        this.value=loadedType;
        return;
      }
      oldType=loadedType;
      removeOldTypedObjects(oldType);
      drawElements();
      clearTypeToolbar();
      currentLayout.type='';
      loadedType=null;
      currentTypeConfig=null;
    }
    return;
  }
  // We have a newType, possibly different from loadedType
  if(loadedType && loadedType!==newType){
    if (!confirm("Changing from type '" + loadedType + "' to '" + newType + "' will remove old typed objects. Proceed?")) {
      this.value=loadedType;
      return;
    }
    oldType=loadedType;
    removeOldTypedObjects(oldType);
    drawElements();
    clearTypeToolbar();
  }
  try{
    const resp=await fetch(`documents/type/${newType}/type.json?_=`+Date.now());
    const cfg=await resp.json();
    loadedType=newType;
    currentTypeConfig=cfg;
    currentLayout.type=newType;
    await loadTypeToolbar(cfg.toolbar);
    if(cfg.objectsJs){
      await loadTypeObjectsJs(newType,cfg.objectsJs);
    }
  }catch(err){
    console.error('Error loading type config:',err);
    alert("Failed to load type config for '"+newType+"'");
    this.value=loadedType||'';
  }
});

// Dynamically load the type’s toolbar
async function loadTypeToolbar(toolbarFile){
  const toolbarContents=document.getElementById('toolbar-contents');
  const hrIndex=[...toolbarContents.children].findIndex(el=>el.tagName==='HR');
  if(hrIndex<0)return;
  while(toolbarContents.children.length>hrIndex+1){
    toolbarContents.removeChild(toolbarContents.lastChild);
  }
  if(!toolbarFile)return;
  try{
    const resp=await fetch(`documents/type/${loadedType}/${toolbarFile}?_=`+Date.now());
    const tbData=await resp.json();
    tbData.forEach(btnDef=>{
      const b=document.createElement('button');
      b.textContent=btnDef.label;
      b.style.width='100%';
      b.style.marginBottom='6px';
      b.onclick=()=>addTypeObject(btnDef.type);
      toolbarContents.appendChild(b);
    });
  }catch(err){
    console.error('Error loading type toolbar:',err);
  }
}
async function loadTypeObjectsJs(folder,file){
  return new Promise((resolve,reject)=>{
    const s=document.createElement('script');
    s.src=`documents/type/${folder}/${file}?_=`+Date.now();
    s.onload=()=>resolve();
    s.onerror=()=>reject("Failed to load objects.js");
    document.body.appendChild(s);
  });
}

function addTypeObject(objectType){
  if(!currentLayout.type){
    alert("No type selected.");
    return;
  }
  if(typeof window.typeObjectCreators[objectType]!=='function'){
    alert("No creator for object type: "+objectType);
    return;
  }
  let newObj=window.typeObjectCreators[objectType](nextObjectId);
  nextObjectId++;
  currentLayout.elements.push(newObj);
  drawElements();
  activeElementIndex=currentLayout.elements.length-1;
  loadProperties(activeElementIndex);
  updateButtonStates();
  refreshObjectList();
}

// Default object creation for line, rect, etc.:
document.querySelectorAll('#toolbar-contents button[data-element-type]').forEach(btn=>{
  btn.addEventListener('click',()=>{
    const t=btn.getAttribute('data-element-type');
    let newEl={type:t,id:nextObjectId++};
    switch(t){
      case 'Line':
        newEl.x1=10; newEl.y1=10; newEl.x2=60; newEl.y2=10;
        newEl.lineWidth=1; newEl.r=0; newEl.g=0; newEl.b=0;
        break;
      case 'Rect':
        newEl.x=20; newEl.y=20; newEl.width=40; newEl.height=30;
        newEl.style='D'; newEl.lineWidth=0.2; newEl.r=0; newEl.g=0; newEl.b=0;
        newEl.fillR=255; newEl.fillG=255; newEl.fillB=255;
        break;
      case 'Cell':
        newEl.x=20; newEl.y=20; newEl.width=40; newEl.height=10;
        newEl.value='New Cell'; newEl.fontFamily='Arial'; newEl.fontStyle='';
        newEl.fontSize=12; newEl.r=0; newEl.g=0; newEl.b=0;
        newEl.fillR=255; newEl.fillG=255; newEl.fillB=255;
        newEl.borderLeft=true; newEl.borderTop=true; newEl.borderRight=true; newEl.borderBottom=true;
        break;
      case 'MultiCell':
        newEl.width=40; newEl.height=15; newEl.value='New MultiCell';
        newEl.fontFamily='Arial'; newEl.fontStyle=''; newEl.fontSize=12;
        newEl.borderLeft=true; newEl.borderTop=true; newEl.borderRight=true; newEl.borderBottom=true;
        newEl.fillR=255; newEl.fillG=255; newEl.fillB=255;
        newEl.r=0; newEl.g=0; newEl.b=0;
        break;
      case 'Image':
        newEl.image='';
        newEl.x=25; newEl.y=25; newEl.width=30; newEl.height=20; newEl.imgType='';
        newEl.r=0; newEl.g=0; newEl.b=0;
        break;
      case 'Text':
        newEl.x=30; newEl.y=30; newEl.value='New Text';
        newEl.fontFamily='Arial'; newEl.fontStyle=''; newEl.fontSize=12;
        newEl.r=0; newEl.g=0; newEl.b=0;
        break;
    }
    currentLayout.elements.push(newEl);
    drawElements();
    activeElementIndex=currentLayout.elements.length-1;
    loadProperties(activeElementIndex);
    updateButtonStates();
    refreshObjectList();
  });
});

// Fill #object-list
function refreshObjectList(){
  objectListSel.innerHTML='<option value="">(none)</option>';
  currentLayout.elements.forEach(el=>{
    if(!el.id) el.id=nextObjectId++;
    const opt=document.createElement('option');
    opt.value=el.id;
    opt.textContent=`ID=${el.id} [${el.type}]`;
    objectListSel.appendChild(opt);
  });
  objectListSel.value='';
}

objectListSel.addEventListener('change', e=>{
  const val=e.target.value.trim();
  if(unsavedChanges){
    let choice=confirm("You have unsaved property changes. OK=Save, Cancel=Discard?");
    if(choice) { applyPendingProps(); }
    else { revertPendingProps(); }
  }
  if(!val){
    activeElementIndex=-1;
    loadProperties(-1);
    return;
  }
  let idx=currentLayout.elements.findIndex(o=>(o.id+''===val));
  if(idx>=0){
    activeElementIndex=idx;
    loadProperties(idx);
    drawElements();
  } else {
    activeElementIndex=-1;
    loadProperties(-1);
  }
});

// Minimizer toggles
toolbarToggle.addEventListener('click',()=>{
  if(!toolbarMinimized){
    toolbar.style.width='30px';
    toolbarToggle.textContent='>';
    toolbarMinimized=true;
  }else{
    toolbar.style.width='180px';
    toolbarToggle.textContent='<';
    toolbarMinimized=false;
  }
});
propertyToggle.addEventListener('click',()=>{
  if(!propertyMinimized){
    propertyBox.style.width='30px';
    propertyToggle.textContent='<';
    propertyMinimized=true;
  }else{
    propertyBox.style.width='220px';
    propertyToggle.textContent='>';
    propertyMinimized=false;
  }
});
zoomInBtn.addEventListener('click',()=>{
  scale+=0.1;if(scale>3)scale=3;
  zoomLevelSpan.textContent=Math.round(scale*100)+'%';
  pageDiv.style.transform='scale('+scale+')';
});
zoomOutBtn.addEventListener('click',()=>{
  scale-=0.1;if(scale<1)scale=1;
  zoomLevelSpan.textContent=Math.round(scale*100)+'%';
  pageDiv.style.transform='scale('+scale+')';
});

// Page size & orientation
pageSizeSelect.addEventListener('change',updatePageSizeFromSelects);
orientationSel.addEventListener('change',updatePageSizeFromSelects);
function updatePageSizeFromSelects(){
  let size=pageSizeSelect.value;
  let orientation=orientationSel.value;
  let w=210,h=297;
  if(size==='A3'){w=297;h=420;}
  else if(size==='Letter'){w=216;h=279;}
  else if(size==='Custom'){w=210;h=297;}
  if(orientation==='L'){[w,h]=[h,w];}
  currentLayout.pageWidth=w;
  currentLayout.pageHeight=h;
  currentLayout.orientation=orientation;
  pageDiv.style.width=(w*3)+'px';
  pageDiv.style.height=(h*3)+'px';
  drawElements();
}

// Load a .layout
layoutSelect.addEventListener('change',()=>{
  let v=layoutSelect.value.trim();
  if(!v){
    btnDeleteLayout.disabled=true;
    currentLayout={pageWidth:210,pageHeight:297,orientation:'P',elements:[],type:'',dialogData:{}};
    drawElements();
    loadProperties(-1);
    updateButtonStates();
    refreshObjectList();
    return;
  }
  btnDeleteLayout.disabled=false;
  fetch('documents/'+v+'?_='+Date.now())
    .then(r=>r.json())
    .then(async data=>{
      // If the layout has a "type", load that type first:
      if(data.type){
        await loadTypeProgrammatically(data.type);
        document.getElementById('type-select').value=data.type;
      } else {
        loadedType=null;
        currentTypeConfig=null;
        currentLayout.type='';
      }
      currentLayout=data;
      if(!currentLayout.dialogData){
        currentLayout.dialogData={};
      }
      let maxId=0;
      (data.elements||[]).forEach(el=>{
        if(el.id&&el.id>maxId)maxId=el.id;
      });
      nextObjectId=maxId+1;
      pageSizeSelect.value=guessSizeFromLayout(data.pageWidth,data.pageHeight);
      orientationSel.value=(data.orientation==='L')?'L':'P';
      updatePageSizeFromSelects();

      drawElements();
      loadProperties(-1);
      updateButtonStates();
      refreshObjectList();
    })
    .catch(err=>{
      alert("Error loading layout: "+err);
    });
});

function guessSizeFromLayout(w,h){
  if((w===210&&h===297)||(w===297&&h===210))return 'A4';
  if((w===297&&h===420)||(w===420&&h===297))return 'A3';
  if((w===216&&h===279)||(w===279&&h===216))return 'Letter';
  return 'Custom';
}

// Buttons: new save, delete, save layout, generate PDF
btnNewSave.addEventListener('click',()=>{
  if(!currentLayout.type){
    alert("Cannot save layout: please select a Type first.");
    return;
  }
  let layoutName=prompt("Enter new layout name (no extension):","my_layout");
  if(!layoutName)return;
  layoutName=layoutName.replace(/[^a-zA-Z0-9_-]/g,'_');
  fetch('document_editor.php?action=saveLayout',{
    method:'POST',
    headers:{'Content-Type':'application/json'},
    body:JSON.stringify({filename:layoutName,data:currentLayout})
  })
  .then(r=>r.json())
  .then(j=>{
    if(j.success){
      alert("Layout ["+j.filename+"] saved!");
      let found=false;
      [...layoutSelect.options].forEach(o=>{if(o.value===j.filename)found=true;});
      if(!found){
        let opt=document.createElement('option');
        opt.value=j.filename;
        opt.textContent=j.filename;
        layoutSelect.appendChild(opt);
      }
      layoutSelect.value=j.filename;
      btnDeleteLayout.disabled=false;
      updateButtonStates();
    }else{
      alert("Error saving: "+j.message);
    }
  })
  .catch(err=>{
    alert("Save error: "+err);
  });
});
btnSave.addEventListener('click',()=>{
  if(!currentLayout.type){
    alert("Cannot save layout: please select a Type first.");
    return;
  }
  let layoutName=layoutSelect.value.trim();
  if(!layoutName){
    layoutName=prompt("Enter new layout name (no extension):","my_layout");
    if(!layoutName)return;
    layoutName=layoutName.replace(/[^a-zA-Z0-9_-]/g,'_');
  }
  fetch('document_editor.php?action=saveLayout',{
    method:'POST',
    headers:{'Content-Type':'application/json'},
    body:JSON.stringify({filename:layoutName,data:currentLayout})
  })
  .then(r=>r.json())
  .then(j=>{
    if(j.success){
      alert("Layout ["+j.filename+"] saved!");
      let found=false;
      [...layoutSelect.options].forEach(o=>{if(o.value===j.filename)found=true;});
      if(!found){
        let opt=document.createElement('option');
        opt.value=j.filename;
        opt.textContent=j.filename;
        layoutSelect.appendChild(opt);
      }
      layoutSelect.value=j.filename;
      btnDeleteLayout.disabled=false;
      updateButtonStates();
    }else{
      alert("Error saving: "+j.message);
    }
  })
  .catch(err=>{
    alert("Save error: "+err);
  });
});
btnDeleteLayout.addEventListener('click',()=>{
  let v=layoutSelect.value.trim();
  if(!v)return;
  if(!confirm("Delete layout '"+v+"'? This cannot be undone."))return;
  fetch('document_editor.php?action=deleteLayout&filename='+encodeURIComponent(v))
  .then(r=>r.json())
  .then(j=>{
    if(j.success){
      alert("Deleted layout ["+j.filename+"]!");
      [...layoutSelect.options].forEach(o=>{
        if(o.value===j.filename)o.remove();
      });
      layoutSelect.value="";
      currentLayout={pageWidth:210,pageHeight:297,orientation:'P',elements:[],type:'',dialogData:{}};
      drawElements();
      loadProperties(-1);
      updateButtonStates();
      refreshObjectList();
    }else{
      alert("Error deleting layout: "+j.message);
    }
  })
  .catch(err=>{
    alert("Delete error: "+err);
  });
});
btnGenerate.addEventListener('click',async ()=>{
  if(!currentLayout.type){
    alert("No type selected. Cannot generate PDF.");
    return;
  }
  try{
    const resp=await fetch(`document_editor.php?action=showTypeDialog&typeFolder=${encodeURIComponent(currentLayout.type)}&_=`+Date.now());
    const j=await resp.json();
    if(!j.success){
      if(!j.message)j.message='No type dialog.';
      console.warn("No type dialog or error:",j.message);
      postLayoutToPdf();
      return;
    }
    showTypeDialog(j.dialog, j.dialogCss||'');
  }catch(err){
    console.error("Error retrieving dialog:",err);
    postLayoutToPdf();
  }
});

// Functions to handle final PDF generation
function postLayoutToPdf(){
  const f=document.createElement('form');
  f.method='POST';
  f.action='document_display_pdf.php';
  f.target='_blank';
  const inp=document.createElement('input');
  inp.type='hidden';
  inp.name='layoutJSON';
  inp.value=JSON.stringify(currentLayout);
  f.appendChild(inp);
  document.body.appendChild(f);
  f.submit();
}

function showTypeDialog(dialogObj, dialogCss){
  if(window.typeFunctions[currentLayout.type] && typeof window.typeFunctions[currentLayout.type].showTypeDialog === 'function'){
    window.typeFunctions[currentLayout.type].showTypeDialog(dialogObj, dialogCss);
  } else {
    console.warn(`No showTypeDialog defined for type: ${currentLayout.type}`);
    postLayoutToPdf(); // fallback
  }
}

// Image management
btnImageMgmt.addEventListener('click',()=>{
  window.location.href='document_image_management.php';
});

// List images
fetch('document_editor.php?action=listImages')
.then(r=>r.json())
.then(files=>{
  const sel=document.getElementById('image-file');
  files.forEach(f=>{
    const opt=document.createElement('option');
    opt.value=f;
    opt.textContent=f;
    sel.appendChild(opt);
  });
})
.catch(err=>console.log("Error listing images:",err));

// Draw items on page
function drawElements(){
  pageDiv.innerHTML='';
  currentLayout.elements.forEach((el,idx)=>{
    if(el.type==='Line'){
      const container=document.createElement('div');
      container.className='element line-container';
      container.style.border='none';
      container.style.background='transparent';
      container.setAttribute('data-index',idx);
      const minX=Math.min(el.x1,el.x2),maxX=Math.max(el.x1,el.x2);
      const minY=Math.min(el.y1,el.y2),maxY=Math.max(el.y1,el.y2);
      container.style.left=(minX*3)+'px';
      container.style.top=(minY*3)+'px';
      container.style.width=((maxX-minX)*3)+'px';
      container.style.height=((maxY-minY)*3)+'px';

      const lineDiv=document.createElement('div');
      lineDiv.style.position='absolute';
      lineDiv.style.border='none';
      lineDiv.style.background='transparent';
      const xA=(el.x1-minX)*3,yA=(el.y1-minY)*3;
      const xB=(el.x2-minX)*3,yB=(el.y2-minY)*3;
      const dx=(xB-xA),dy=(yB-yA);
      const lengthPx=Math.sqrt(dx*dx+dy*dy);
      const angleDeg=Math.atan2(dy,dx)*180/Math.PI;

      lineDiv.style.left=xA+'px';
      lineDiv.style.top=(yA-(el.lineWidth*3)/2)+'px';
      lineDiv.style.width=lengthPx+'px';
      lineDiv.style.height=(el.lineWidth*3)+'px';
      lineDiv.style.transformOrigin='0 50%';
      lineDiv.style.transform=`rotate(${angleDeg}deg)`;
      lineDiv.style.background=`rgb(${el.r||0},${el.g||0},${el.b||0})`;
      container.appendChild(lineDiv);

      const ptA=document.createElement('div');
      ptA.className='line-handle';
      ptA.style.left=(xA-4)+'px';
      ptA.style.top=(yA-4)+'px';

      const ptB=document.createElement('div');
      ptB.className='line-handle';
      ptB.style.left=(xB-4)+'px';
      ptB.style.top=(yB-4)+'px';

      container.appendChild(ptA);
      container.appendChild(ptB);

      container.addEventListener('mousedown',e=>{
        activeElementDiv=container;
        isDragging=true;
        offsetX=e.offsetX;offsetY=e.offsetY;
        activeElementIndex=idx;
        loadProperties(idx);
        e.preventDefault();
        objectListSel.value=(el.id||'')+'';
      });
      ptA.addEventListener('mousedown',e=>{
        e.stopPropagation(); e.preventDefault();
        activeElementIndex=idx;
        draggingLineEnd='A';
        offsetX=0; offsetY=0;
        objectListSel.value=(el.id||'')+'';
      });
      ptB.addEventListener('mousedown',e=>{
        e.stopPropagation(); e.preventDefault();
        activeElementIndex=idx;
        draggingLineEnd='B';
        offsetX=0; offsetY=0;
        objectListSel.value=(el.id||'')+'';
      });
      pageDiv.appendChild(container);
    }
    else if(['Rect','Cell','MultiCell','Image'].includes(el.type)){
      const div=document.createElement('div');
      div.className='element';
      div.setAttribute('data-index',idx);

      if(el.type==='Rect'){
        div.textContent='Rect';
      } else if(el.type==='Cell' || el.type==='MultiCell'){
        div.innerHTML=el.value||'??';
      } else if(el.type==='Image'){
        if(el.image){
          div.style.background=`url('documents/images/${el.image}') center center / cover no-repeat`;
        } else {
          div.textContent='(Image?)';
          div.style.background='#eee';
        }
      }

      div.style.left=((el.x||0)*3)+'px';
      div.style.top=((el.y||0)*3)+'px';
      div.style.width=((el.width||0)*3)+'px';
      div.style.height=((el.height||0)*3)+'px';

      if(el.type==='Rect'){
        let drawColor=`rgb(${el.r||0},${el.g||0},${el.b||0})`;
        let fillColor=`rgb(${el.fillR||255},${el.fillG||255},${el.fillB||255})`;
        const lw=(el.lineWidth||0.2)*3;
        if(el.style==='D'){
          div.style.background='transparent';
          div.style.border=`${lw}px solid ${drawColor}`;
        } else if(el.style==='F'){
          div.style.background=fillColor;
          div.style.border='1px dashed #aaa';
        } else{
          div.style.background=fillColor;
          div.style.border=`${lw}px solid ${drawColor}`;
        }
      }
      if(el.type==='Cell' || el.type==='MultiCell'){
        const txtColor=`rgb(${el.textR||0},${el.textG||0},${el.textB||0})`;
        div.style.color=txtColor;
        div.style.fontFamily=el.fontFamily||'Arial';
        div.style.fontSize=(el.fontSize||12)+'px';
        const bw=(el.lineWidth||0.2)*3;
        let borderColor=`rgb(${el.r||0},${el.g||0},${el.b||0})`;
        div.style.borderLeft=el.borderLeft?(bw+'px solid '+borderColor):'none';
        div.style.borderTop=el.borderTop?(bw+'px solid '+borderColor):'none';
        div.style.borderRight=el.borderRight?(bw+'px solid '+borderColor):'none';
        div.style.borderBottom=el.borderBottom?(bw+'px solid '+borderColor):'none';
        if(el.fill){
          const bc=`rgb(${el.fillR||255},${el.fillG||255},${el.fillB||255})`;
          div.style.background=bc;
        } else {
          div.style.background='transparent';
        }
        // Add alignment logic:
        if (el.align === 'C') {
          div.style.textAlign = 'center';
        } else if (el.align === 'R') {
          div.style.textAlign = 'right';
        } else {
          // Default to left if "L" or empty
          div.style.textAlign = 'left';
        }
      }

      const resizeHandle=document.createElement('div');
      resizeHandle.className='resize-br-handle';
      div.appendChild(resizeHandle);
      resizeHandle.addEventListener('mousedown',e=>{
        e.stopPropagation(); e.preventDefault();
        activeElementIndex=idx;
        resizingElementIndex=idx;
        resizingCorner='BR';
        offsetX=e.offsetX;
        offsetY=e.offsetY;
      });

      div.addEventListener('mousedown',e=>{
        activeElementDiv=div;
        isDragging=true;
        offsetX=e.offsetX;
        offsetY=e.offsetY;
        activeElementIndex=idx;
        loadProperties(idx);
        e.preventDefault();
        objectListSel.value=(el.id||'')+'';
      });

      pageDiv.appendChild(div);
    }
    else if(el.type==='Text'){
      const div=document.createElement('div');
      div.className='element';
      div.setAttribute('data-index',idx);
      div.innerHTML=el.value||'Text';
      div.style.left=((el.x||0)*3)+'px';
      div.style.top=((el.y||0)*3)+'px';
      div.style.width='auto';
      div.style.height='auto';
      div.style.fontFamily=el.fontFamily||'Arial';
      div.style.fontSize=(el.fontSize||12)+'px';
      div.style.color=`rgb(${el.r||0},${el.g||0},${el.b||0})`;
      div.addEventListener('mousedown',e=>{
        activeElementDiv=div;
        isDragging=true;
        offsetX=e.offsetX;
        offsetY=e.offsetY;
        activeElementIndex=idx;
        loadProperties(idx);
        e.preventDefault();
        objectListSel.value=(el.id||'')+'';
      });
      pageDiv.appendChild(div);
    }
    else {
      // Might be a typed object with custom preview:
      if(typeof window.typeObjectCreators['_renderElementPreview']==='function'){
        const customDiv=window.typeObjectCreators['_renderElementPreview'](el,idx);
        if(customDiv){
          pageDiv.appendChild(customDiv);
        }
      }
    }
  });
}

document.addEventListener('mousemove',e=>{
  if(!isDragging && !draggingLineEnd && resizingElementIndex<0) return;
  if(activeElementIndex<0||!currentLayout.elements[activeElementIndex])return;

  const el=currentLayout.elements[activeElementIndex];
  const rect=pageDiv.getBoundingClientRect();
  const mouseX=(e.clientX-rect.left)/scale;
  const mouseY=(e.clientY-rect.top)/scale;

  if(draggingLineEnd && el.type==='Line'){
    if(draggingLineEnd==='A'){
      el.x1=round2(mouseX/3);
      el.y1=round2(mouseY/3);
    } else {
      el.x2=round2(mouseX/3);
      el.y2=round2(mouseY/3);
    }
    drawElements();
    loadProperties(activeElementIndex);
    return;
  }
  if(resizingElementIndex>=0 && resizingCorner==='BR'){
    let reEl=currentLayout.elements[resizingElementIndex];
    if(!reEl)return;
    let newW=((mouseX/3)-(reEl.x||0));
    let newH=((mouseY/3)-(reEl.y||0));
    if(newW<1)newW=1;
    if(newH<1)newH=1;
    reEl.width=round2(newW);
    reEl.height=round2(newH);
    drawElements();
    loadProperties(resizingElementIndex);
    return;
  }
  if(isDragging && el.type!=='Line'){
    const newLeft=(mouseX-offsetX);
    const newTop=(mouseY-offsetY);
    el.x=round2(newLeft/3);
    el.y=round2(newTop/3);
    drawElements();
    loadProperties(activeElementIndex);
  }
});
document.addEventListener('mouseup',()=>{
  isDragging=false;
  draggingLineEnd=null;
  resizingElementIndex=-1;
  resizingCorner='';
});

// ---------- PROPERTY EDITING / UNSAVED CHANGES ----------
function loadTypeProperties(objectType) {
    if (['Line','Rect','Cell','MultiCell','Image','Text'].includes(objectType)) {return;}

    // 1) Check if the currently selected object is the same as last time
    //    If so, do NOT remove and recreate the typed properties

    // We'll maintain a global or higher-scope 'lastTypedObjectId' and 'lastTypedObjectType'
    // If the ID and type match, we skip the entire property teardown
    let currentObjId = (activeElementIndex >= 0) ? currentLayout.elements[activeElementIndex].id : null;
    if (currentObjId && lastTypedObjectId === currentObjId && lastTypedObjectType === objectType) {
        // just update the fields, do not remove them
        if (typeof window.typeFunctions[loadedType]?.populateProperties === 'function') {
            const el = currentLayout.elements[activeElementIndex];
            window.typeFunctions[loadedType].populateProperties(el);
        }
        return;
    }

    // Otherwise, we do the normal property creation
    lastTypedObjectId = currentObjId;
    lastTypedObjectType = objectType;

    const propertyPath = `documents/type/${loadedType}/${currentTypeConfig.property}?_=${Date.now()}`;

    fetch(propertyPath)
        .then(res => res.json())
        .then(propData => {
            // Remove old dynamic properties
            const dynamicProps = document.querySelectorAll('.dynamic-property');
            dynamicProps.forEach(dp => dp.remove());

            // If no matching object type in property.json, do nothing
            if (!propData.properties || !propData.properties[objectType]) {
                console.warn("No property definition found for", objectType);
                return;
            }

            const objProps = propData.properties[objectType];

            // Insert new dynamic property fields
            const propButtons = document.getElementById('prop-buttons');
            objProps.forEach(prop => {
                const wrapper = document.createElement('div');
                wrapper.className = 'dynamic-property';
                // Make a label
                const label = document.createElement('label');
                label.textContent = prop.label || prop.id;
                label.style.display = 'block';
                wrapper.appendChild(label);

                // Input element creation
                let inputEl;
                if (prop.type === 'select') {
                    inputEl = document.createElement('select');
                    inputEl.id = prop.id;
                    if (Array.isArray(prop.options)) {
                        prop.options.forEach(opt => {
                            const optionNode = document.createElement('option');
                            optionNode.value = opt.value;
                            optionNode.textContent = opt.text;
                            inputEl.appendChild(optionNode);
                        });
                    }
                } else if (prop.type === 'checkbox') {
                    inputEl = document.createElement('input');
                    inputEl.type = 'checkbox';
                    inputEl.id = prop.id;
                } else {
                    // number, color, text, etc.
                    inputEl = document.createElement('input');
                    inputEl.type = prop.type || 'text';
                    inputEl.id = prop.id;
                    if (prop.step) inputEl.step = prop.step;
                }

                // Append input to wrapper
                wrapper.appendChild(inputEl);
                // Insert into property-box
                document.getElementById('property-box').insertBefore(wrapper, propButtons);
            });

            // Now populate the property fields from the active element’s data:
            if (activeElementIndex >= 0) {
                const el = currentLayout.elements[activeElementIndex];
                if (el && typeof window.typeFunctions[loadedType]?.populateProperties === 'function') {
                    window.typeFunctions[loadedType].populateProperties(el);
                }
            }
        })
        .catch(err => console.error("Error loading dynamic property:", err));
}

 function removeDynamicProperties() {
   const dynamicProps = document.querySelectorAll('.dynamic-property');
   dynamicProps.forEach(dp => dp.remove());
   lastTypedObjectId = null;
   lastTypedObjectType = null;
 }

function loadProperties(idx){


  const forms=[
    'line-properties','rect-properties','cell-properties','multicell-properties','image-properties','text-properties'
  ];
  forms.forEach(id=>{
    let f=document.getElementById(id);
    if(f)f.style.display='none';
  });

  if(idx < 0){
    document.getElementById('no-selection').style.display='block';
    document.getElementById('prop-buttons').style.display='none';
    return;
  }
  document.getElementById('no-selection').style.display='none';
  document.getElementById('prop-buttons').style.display='flex';
  const el = currentLayout.elements[idx];
  if(!el) return;

  if (el.type != lastTypedObjectType){
    removeDynamicProperties();
  }

  switch(el.type){
    case 'Line':
      document.getElementById('line-properties').style.display='block';
      document.getElementById('line-x1').value=el.x1||0;
      document.getElementById('line-y1').value=el.y1||0;
      document.getElementById('line-x2').value=el.x2||0;
      document.getElementById('line-y2').value=el.y2||0;
      document.getElementById('line-width').value=el.lineWidth||1;
      document.getElementById('line-color').value=rgbToHex(el.r||0,el.g||0,el.b||0);
      document.getElementById('line-bind').value=el.bind||'none';
      break;
    case 'Rect':
      document.getElementById('rect-properties').style.display='block';
      document.getElementById('rect-x').value=el.x||0;
      document.getElementById('rect-y').value=el.y||0;
      document.getElementById('rect-w').value=el.width||0;
      document.getElementById('rect-h').value=el.height||0;
      document.getElementById('rect-style').value=el.style||'D';
      document.getElementById('rect-linewidth').value=el.lineWidth||0.2;
      document.getElementById('rect-drawcolor').value=rgbToHex(el.r||0,el.g||0,el.b||0);
      document.getElementById('rect-fillcolor').value=rgbToHex(el.fillR||255,el.fillG||255,el.fillB||255);
      document.getElementById('rect-bind').value=el.bind||'none';
      break;
    case 'Cell':
      document.getElementById('cell-properties').style.display='block';
      document.getElementById('cell-x').value = el.x||0;
      document.getElementById('cell-y').value = el.y||0;
      document.getElementById('cell-w').value=el.width||40;
      document.getElementById('cell-h').value=el.height||10;
      document.getElementById('cell-txt').value=el.value||'';
      document.getElementById('cell-border-l').checked=!!el.borderLeft;
      document.getElementById('cell-border-t').checked=!!el.borderTop;
      document.getElementById('cell-border-r').checked=!!el.borderRight;
      document.getElementById('cell-border-b').checked=!!el.borderBottom;
      document.getElementById('cell-ln').value=el.ln||'0';
      document.getElementById('cell-align').value=el.align||'';
      document.getElementById('cell-fill').checked=!!el.fill;
      document.getElementById('cell-fontfam').value=el.fontFamily||'Arial';
      document.getElementById('cell-fontsty').value=el.fontStyle||'';
      document.getElementById('cell-fontsz').value=el.fontSize||12;
      document.getElementById('cell-drawcolor').value=rgbToHex(el.r||0,el.g||0,el.b||0);
      document.getElementById('cell-fillcolor').value=rgbToHex(el.fillR||255,el.fillG||255,el.fillB||255);
      document.getElementById('cell-textcolor').value=rgbToHex(el.textR||0,el.textG||0,el.textB||0);
      document.getElementById('cell-linewidth').value=el.lineWidth||0.2;
      document.getElementById('cell-bind').value=el.bind||'none';
      break;
    case 'MultiCell':
      document.getElementById('multicell-properties').style.display='block';
      document.getElementById('mc-x').value = el.x||0;
      document.getElementById('mc-y').value = el.y||0;
      document.getElementById('mc-w').value=el.width||0;
      document.getElementById('mc-h').value=el.height||0;
      document.getElementById('mc-txt').value=el.value||'';
      document.getElementById('mc-border-l').checked=!!el.borderLeft;
      document.getElementById('mc-border-t').checked=!!el.borderTop;
      document.getElementById('mc-border-r').checked=!!el.borderRight;
      document.getElementById('mc-border-b').checked=!!el.borderBottom;
      document.getElementById('mc-align').value=el.align||'';
      document.getElementById('mc-fill').checked=!!el.fill;
      document.getElementById('mc-fontfam').value=el.fontFamily||'Arial';
      document.getElementById('mc-fontsty').value=el.fontStyle||'';
      document.getElementById('mc-fontsz').value=el.fontSize||12;
      document.getElementById('mc-drawcolor').value=rgbToHex(el.r||0,el.g||0,el.b||0);
      document.getElementById('mc-fillcolor').value=rgbToHex(el.fillR||255,el.fillG||255,el.fillB||255);
      document.getElementById('mc-textcolor').value=rgbToHex(el.textR||0,el.textG||0,el.textB||0);
      document.getElementById('mc-linewidth').value=el.lineWidth||0.2;
      document.getElementById('mc-bind').value=el.bind||'none';
      break;
    case 'Image':
      document.getElementById('image-properties').style.display='block';
      document.getElementById('image-file').value=el.image||'';
      document.getElementById('image-x').value=el.x||0;
      document.getElementById('image-y').value=el.y||0;
      document.getElementById('image-w').value=el.width||0;
      document.getElementById('image-h').value=el.height||0;
      document.getElementById('image-type').value=el.imgType||'';
      document.getElementById('image-bind').value=el.bind||'none';
      break;
    case 'Text':
      document.getElementById('text-properties').style.display='block';
      document.getElementById('text-x').value=el.x||0;
      document.getElementById('text-y').value=el.y||0;
      document.getElementById('text-value').value=el.value||'';

    // Font Family + Style
      document.getElementById('text-fontfam').value=el.fontFamily||'Arial';
      document.getElementById('text-fontsty').value=el.fontStyle||'';
      document.getElementById('text-fontsz').value=el.fontSize||12;
      document.getElementById('text-color').value=rgbToHex(el.r||0,el.g||0,el.b||0);
      document.getElementById('text-bind').value=el.bind||'none';
      break;
    default:
      loadTypeProperties(el.type);
      break;
  }
}
// Listen for property changes to update the currentLayout
propertyBox.addEventListener('input', (e) => {
  if(activeElementIndex < 0) return;
  const el = currentLayout.elements[activeElementIndex];
  if(!el) return;

  const target = e.target;
  // Fix: define val properly, handle checkboxes vs normal
  let val;
  if (target.type === 'checkbox') {
    val = target.checked ? 'on' : '';
  } else {
    val = target.value;
  }
  if(el.type === 'Line'){
    switch(target.id){
      case 'line-x1': el.x1 = parseFloat(target.value)||0; break;
      case 'line-y1': el.y1 = parseFloat(target.value)||0; break;
      case 'line-x2': el.x2 = parseFloat(target.value)||0; break;
      case 'line-y2': el.y2 = parseFloat(target.value)||0; break;
      case 'line-width': el.lineWidth = parseFloat(target.value)||1; break;
      case 'line-color':
        const lc = hexToRgb(target.value);
        el.r = lc.r; el.g = lc.g; el.b = lc.b;
        break;
      case 'line-bind': el.bind = target.value; break;
    }
  }
  else if(el.type === 'Rect'){
    switch(target.id){
      case 'rect-x': el.x = parseFloat(target.value)||0; break;
      case 'rect-y': el.y = parseFloat(target.value)||0; break;
      case 'rect-w': el.width = parseFloat(target.value)||0; break;
      case 'rect-h': el.height = parseFloat(target.value)||0; break;
      case 'rect-style': el.style = target.value; break;
      case 'rect-linewidth': el.lineWidth = parseFloat(target.value)||0.2; break;
      case 'rect-drawcolor':
        const rd = hexToRgb(target.value);
        el.r = rd.r; el.g = rd.g; el.b = rd.b;
        break;
      case 'rect-fillcolor':
        const rf = hexToRgb(target.value);
        el.fillR = rf.r; el.fillG = rf.g; el.fillB = rf.b;
        break;
      case 'rect-bind': el.bind = target.value; break;
    }
  }
  else if(el.type === 'Cell'){
    switch(target.id){
      case 'cell-x': el.x = parseFloat(target.value)||0; break;
      case 'cell-y': el.y = parseFloat(target.value)||0; break;
      case 'cell-w': el.width = parseFloat(target.value)||0; break;
      case 'cell-h': el.height = parseFloat(target.value)||0; break;
      case 'cell-txt': el.value = target.value; break;
      case 'cell-border-l': el.borderLeft = target.checked; break;
      case 'cell-border-t': el.borderTop = target.checked; break;
      case 'cell-border-r': el.borderRight = target.checked; break;
      case 'cell-border-b': el.borderBottom = target.checked; break;
      case 'cell-ln': el.ln = target.value; break;
      case 'cell-align': el.align = target.value; break;
      case 'cell-fill': el.fill = target.checked; break;
      case 'cell-fontfam': el.fontFamily = target.value; break;
      case 'cell-fontsty': el.fontStyle = target.value; break;
      case 'cell-fontsz': el.fontSize = parseFloat(target.value)||12; break;
      case 'cell-drawcolor':
        const cdc = hexToRgb(target.value);
        el.r = cdc.r; el.g = cdc.g; el.b = cdc.b;
        break;
      case 'cell-fillcolor':
        const cfc = hexToRgb(target.value);
        el.fillR = cfc.r; el.fillG = cfc.g; el.fillB = cfc.b;
        break;
      case 'cell-textcolor':
        const ctc = hexToRgb(target.value);
        el.textR = ctc.r; el.textG = ctc.g; el.textB = ctc.b;
        break;
      case 'cell-linewidth': el.lineWidth = parseFloat(target.value)||0.2; break;
      case 'cell-bind': el.bind = target.value; break;
    }
  }
  else if(el.type === 'MultiCell'){
    switch(target.id){
      case 'mc-x': el.x = parseFloat(target.value)||0; break;
      case 'mc-y': el.y = parseFloat(target.value)||0; break;
      case 'mc-w': el.width = parseFloat(target.value)||0; break;
      case 'mc-h': el.height = parseFloat(target.value)||0; break;
      case 'mc-txt': el.value = target.value; break;
      case 'mc-border-l': el.borderLeft = target.checked; break;
      case 'mc-border-t': el.borderTop = target.checked; break;
      case 'mc-border-r': el.borderRight = target.checked; break;
      case 'mc-border-b': el.borderBottom = target.checked; break;
      case 'mc-align': el.align = target.value; break;
      case 'mc-fill': el.fill = target.checked; break;
      case 'mc-fontfam': el.fontFamily = target.value; break;
      case 'mc-fontsty': el.fontStyle = target.value; break;
      case 'mc-fontsz': el.fontSize = parseFloat(target.value)||12; break;
      case 'mc-drawcolor':
        const mdc = hexToRgb(target.value);
        el.r = mdc.r; el.g = mdc.g; el.b = mdc.b;
        break;
      case 'mc-fillcolor':
        const mfc = hexToRgb(target.value);
        el.fillR = mfc.r; el.fillG = mfc.g; el.fillB = mfc.b;
        break;
      case 'mc-textcolor':
        const mtc = hexToRgb(target.value);
        el.textR = mtc.r; el.textG = mtc.g; el.textB = mtc.b;
        break;
      case 'mc-linewidth': el.lineWidth = parseFloat(target.value)||0.2; break;
      case 'mc-bind': el.bind = target.value; break;
    }
  }
  else if(el.type === 'Image'){
    switch(target.id){
      case 'image-file': el.image = target.value; break;
      case 'image-x': el.x = parseFloat(target.value)||0; break;
      case 'image-y': el.y = parseFloat(target.value)||0; break;
      case 'image-w': el.width = parseFloat(target.value)||0; break;
      case 'image-h': el.height = parseFloat(target.value)||0; break;
      case 'image-type': el.imgType = target.value; break;
      case 'image-bind': el.bind = target.value; break;
    }
  }
  else if(el.type === 'Text'){
    switch(target.id){
      case 'text-x': el.x = parseFloat(target.value)||0; break;
      case 'text-y': el.y = parseFloat(target.value)||0; break;
      case 'text-value': el.value = target.value; break;
      case 'text-fontfam': el.fontFamily = target.value; break;
      case 'text-fontsty': el.fontStyle = target.value; break;
      case 'text-fontsz': el.fontSize = parseFloat(target.value)||12; break;
      case 'text-color':
        const tc = hexToRgb(target.value);
        el.r = tc.r; el.g = tc.g; el.b = tc.b;
        break;
      case 'text-bind': el.bind = target.value; break;
    }
  }else {
    if(loadedType && window.typeFunctions[loadedType] && typeof window.typeFunctions[loadedType].updateProperty === 'function'){
      window.typeFunctions[loadedType].updateProperty(el, target.id, val);
    }
  }
  // Finally, redraw so changes appear on the page:
   drawElements();
});


propDelete.addEventListener('click',()=>{
  if(activeElementIndex<0)return;
  currentLayout.elements.splice(activeElementIndex,1);
  activeElementIndex=-1;
  drawElements();
  loadProperties(-1);
  updateButtonStates();
  refreshObjectList();
});

function updateButtonStates(){
  const layoutVal=layoutSelect.value.trim();
  const isLayoutSelected=(layoutVal!=="");
  const objectCount=currentLayout.elements.length;
  if(!isLayoutSelected&&objectCount===0){
    btnNewSave.disabled=true;
    btnDeleteLayout.disabled=true;
    btnSave.disabled=true;
    btnGenerate.disabled=true;
    btnImageMgmt.disabled=true;
  }
  else if(!isLayoutSelected&&objectCount>0){
    btnNewSave.disabled=true;
    btnDeleteLayout.disabled=true;
    btnSave.disabled=false;
    btnGenerate.disabled=false;
    btnImageMgmt.disabled=true;
  }
  else{
    btnNewSave.disabled=false;
    btnDeleteLayout.disabled=false;
    btnSave.disabled=false;
    btnGenerate.disabled=false;
    btnImageMgmt.disabled=false;
  }
}
function round2(num){return Math.round(num*100)/100;}
function rgbToHex(r,g,b){
  function c(n){return n.toString(16).padStart(2,'0');}
  return '#'+c(r)+c(g)+c(b);
}
function hexToRgb(hex){
  let c=hex.replace('#','');
  if(c.length===3)c=c[0]+c[0]+c[1]+c[1]+c[2]+c[2];
  let num=parseInt(c,16);
  return {r:(num>>16)&255,g:(num>>8)&255,b:num&255};
}
updatePageSizeFromSelects();
loadProperties(-1);
updateButtonStates();
refreshObjectList();
</script>
<?php require_once('components/footer.php'); ?>
