<?php
/**
 * document_image_management.php
 *
 * Allows user to manage images in /public/documents/images/ folder:
 * - Upload new images
 * - Delete existing images
 */
ini_set('display_errors', 1);
ini_set('display_startup_errors', 1);
error_reporting(E_ALL);

require_once('../config/config.php');
require_once('components/auth.php');
try {
    $pdo = new PDO('sqlite:' . DB_FILE);
    $pdo->setAttribute(PDO::ATTR_ERRMODE, PDO::ERRMODE_EXCEPTION);
} catch (PDOException $e) {
    die("Database Connection Error: " . $e->getMessage());
}

require_once('components/header.php');

$imagesDir = __DIR__ . '/documents/images';
if (!is_dir($imagesDir)) {
    @mkdir($imagesDir, 0777, true);
}

$errors = [];
$success = '';

// Handle uploads
if ($_SERVER['REQUEST_METHOD'] === 'POST' && isset($_FILES['imgfile'])) {
    $fileName = basename($_FILES['imgfile']['name']);
    $ext = strtolower(pathinfo($fileName, PATHINFO_EXTENSION));
    if (in_array($ext, ['jpg','jpeg','png','gif'])) {
        $targetPath = $imagesDir . '/' . $fileName;
        if (move_uploaded_file($_FILES['imgfile']['tmp_name'], $targetPath)) {
            $success = "Image [$fileName] uploaded successfully!";
        } else {
            $errors[] = "Failed to move uploaded file.";
        }
    } else {
        $errors[] = "Unsupported file type. Allowed: jpg, jpeg, png, gif.";
    }
}

// Handle delete
if (isset($_GET['delete'])) {
    $delFile = basename($_GET['delete']);
    $filePath = $imagesDir . '/' . $delFile;
    if (file_exists($filePath)) {
        if (@unlink($filePath)) {
            $success = "Image [$delFile] deleted.";
        } else {
            $errors[] = "Failed to delete [$delFile].";
        }
    } else {
        $errors[] = "File [$delFile] not found.";
    }
}

// Gather image files
$files = [];
$scan = @scandir($imagesDir);
if ($scan) {
    foreach ($scan as $f) {
        if ($f[0] === '.') continue;
        $ext = strtolower(pathinfo($f, PATHINFO_EXTENSION));
        if (in_array($ext, ['jpg','jpeg','png','gif'])) {
            $files[] = $f;
        }
    }
}
?>
<h1>Manage Images</h1>
<?php if (!empty($errors)): ?>
<div class="alert">
    <?php foreach ($errors as $err): ?>
        <p><?php echo htmlspecialchars($err); ?></p>
    <?php endforeach; ?>
</div>
<?php endif; ?>
<?php if ($success): ?>
<div class="success">
    <p><?php echo htmlspecialchars($success); ?></p>
</div>
<?php endif; ?>

<!-- Upload form -->
<div class="form-container" style="max-width:400px;">
    <form action="document_image_management.php" method="POST" enctype="multipart/form-data">
        <label for="imgfile">Upload Image:</label>
        <input type="file" name="imgfile" id="imgfile" required accept=".jpg,.jpeg,.png,.gif">
        <button type="submit">Upload</button>
    </form>
</div>

<!-- List images in a table -->
<?php if (!empty($files)): ?>
<table>
    <thead>
        <tr>
            <th>Preview</th>
            <th>Filename</th>
            <th>Actions</th>
        </tr>
    </thead>
    <tbody>
        <?php foreach ($files as $f): ?>
        <tr>
            <td>
                <img src="documents/images/<?php echo rawurlencode($f); ?>"
                     alt="preview"
                     style="max-width:80px; max-height:80px;">
            </td>
            <td><?php echo htmlspecialchars($f); ?></td>
            <td>
                <a class="button delete-button"
                   href="document_image_management.php?delete=<?php echo urlencode($f); ?>"
                   onclick="return confirm('Delete this image?');">
                   Delete
                </a>
            </td>
        </tr>
        <?php endforeach; ?>
    </tbody>
</table>
<?php else: ?>
<p>No images found in /documents/images/.</p>
<?php endif; ?>

<?php require_once('components/footer.php'); ?>

