<?php
require_once('../config/config.php');
require_once('../core/financial_reports.php');
require_once('../core/financial_periods.php');

function renderBSTable($pdf, $element, $dialogData)
{
    global $pdo;
    $tblBorderShow  = isset($element['borderShow'])  ? (bool)$element['borderShow']  : true;
    $tblBorderColor = isset($element['borderColor']) ? $element['borderColor']       : '#000000';
    $tblBorderWidth = isset($element['borderWidth']) ? floatval($element['borderWidth']) : 1.0;

    $hdrFontFam   = isset($element['headerFontFam'])   ? $element['headerFontFam']   : 'Courier';
    $hdrFontSty   = isset($element['headerFontSty'])   ? $element['headerFontSty']   : '';
    $hdrFontSize  = isset($element['headerFontSize'])  ? floatval($element['headerFontSize']) : 12;
    $hdrFontColor = isset($element['headerFontColor']) ? $element['headerFontColor'] : '#000000';
    $hdrFill      = isset($element['headerFill'])      ? (bool)$element['headerFill'] : true;
    $hdrFillColor = isset($element['headerFillColor']) ? $element['headerFillColor']  : '#cccccc';

    $dataFontFam   = isset($element['dataFontFam'])   ? $element['dataFontFam']      : 'Courier';
    $dataFontSty   = isset($element['dataFontSty'])   ? $element['dataFontSty']      : '';
    $dataFontSize  = isset($element['dataFontSize'])  ? floatval($element['dataFontSize']) : 12;
    $dataFontColor = isset($element['dataFontColor']) ? $element['dataFontColor']    : '#000000';

    $dataFill       = isset($element['dataFill'])       ? $element['dataFill']       : 'stripe';
    $dataFillColor  = isset($element['dataFillColor'])  ? $element['dataFillColor']  : '#ffffff';
    $dataFillColor2 = isset($element['dataFillColor2']) ? $element['dataFillColor2'] : '#f3f3f3';

    $dataHierarchy  = isset($element['dataHierarchy'])  ? (bool)$element['dataHierarchy']  : true;
    $dataMulticolor = isset($element['dataMulticolor']) ? (bool)$element['dataMulticolor'] : true;

    $colorAsset  = isset($element['colorAsset'])  ? $element['colorAsset']  : '#26A269';
    $colorLiab   = isset($element['colorLiab'])   ? $element['colorLiab']   : '#1A5FB4';
    $colorEquity = isset($element['colorEquity']) ? $element['colorEquity'] : '#DC8ADD';
    $colorTotals = isset($element['colorTotals']) ? $element['colorTotals'] : '#000000';

    $hideCode     = isset($element['hideCode'])     ? (bool)$element['hideCode']     : false;
    $hideAcctType = isset($element['hideAcctType']) ? (bool)$element['hideAcctType'] : false;
    $hideNoBalance= isset($element['hideNoBalance'])? (bool)$element['hideNoBalance'] : false;

    $x = isset($element['x']) ? floatval($element['x']) : 10;
    $y = isset($element['y']) ? floatval($element['y']) : 10;
    $w = isset($element['width']) ? floatval($element['width']) : 60;
    $h = isset($element['height'])? floatval($element['height']) : 50;

    $bc = hexToRgb($tblBorderColor);
    $borderR = $bc['r'];
    $borderG = $bc['g'];
    $borderB = $bc['b'];

    // Date
    $asOfDate = !empty($dialogData['as_of_date']) ? $dialogData['as_of_date'] : date('Y-m-d');
    $pdf->PDFName = 'Balance Sheet '.$asOfDate.'.pdf';
    $rawBalanceSheet = generateBalanceSheet($pdo, $asOfDate);
    if (empty($rawBalanceSheet)) {
        return;
    }

    // If there's a financial_period_id, check for closed vs. temporary net income
    if (!empty($dialogData['financial_period_id'])) {
        $fpId = (int)$dialogData['financial_period_id'];
        $fp   = getFinancialPeriod($pdo, $fpId);
        if ($fp) {
            $isClosed = isFinancialPeriodClosed($pdo, $fpId);
            $incomeStmt = generateIncomeStatement($pdo, $fp['start_date'], $fp['end_date']);
            $totalRev=0;
            $totalExp=0;
            foreach ($incomeStmt as $acct) {
                $t = strtolower($acct['account_type']);
                if (in_array($t, ['revenue','income'])) {
                    $totalRev += (float)$acct['total_amount'];
                } elseif ($t==='expense') {
                    $totalExp += (float)$acct['total_amount'];
                }
            }
            $netIncome = $totalRev - $totalExp;
            $netName   = $isClosed ? 'Net Income' : 'Temporary Net Income';

            $foundNI = false;
            foreach ($rawBalanceSheet as &$rbs) {
                if (isset($rbs['code']) && $rbs['code']===NET_INCOME_ACC_CODE) {
                    $rbs['name']         = $netName;
                    $rbs['total_amount'] = $netIncome;
                    $foundNI = true;
                    break;
                }
            }
            unset($rbs);
            if (!$foundNI) {
                $rawBalanceSheet[] = [
                  'account_type'=>'Equity',
                  'code'        => NET_INCOME_ACC_CODE,
                  'name'        => $netName,
                  'parent'      => null,
                  'total_amount'=> $netIncome
                ];
            }
        }
    } else {
        // No period => do yearStart->asOfDate
        $yearStart = date('Y-01-01', strtotime($asOfDate));
        $incomeStmt = generateIncomeStatement($pdo, $yearStart, $asOfDate);
        $totalRev=0;
        $totalExp=0;
        foreach ($incomeStmt as $acct) {
            $t = strtolower($acct['account_type']);
            if (in_array($t, ['revenue','income'])) $totalRev += (float)$acct['total_amount'];
            elseif ($t==='expense')                $totalExp += (float)$acct['total_amount'];
        }
        $netIncome = $totalRev - $totalExp;
        $netName   = 'Net Income (Temporary)';

        $foundNI = false;
        foreach ($rawBalanceSheet as &$rbs) {
            if (isset($rbs['code']) && $rbs['code']===NET_INCOME_ACC_CODE) {
                $rbs['name']         = $netName;
                $rbs['total_amount'] = $netIncome;
                $foundNI = true;
                break;
            }
        }
        unset($rbs);
        if (!$foundNI) {
            $rawBalanceSheet[] = [
              'account_type'=>'Equity',
              'code'        => NET_INCOME_ACC_CODE,
              'name'        => $netName,
              'parent'      => null,
              'total_amount'=> $netIncome
            ];
        }
    }

    $accountTree = buildAccountTree($rawBalanceSheet);
    $flatRows = [];
    traverseAccountTree($accountTree, $flatRows);

    $totals = ['asset'=>0.0, 'liability'=>0.0, 'equity'=>0.0];
    $processedAccounts = [];
    $currentType = null;

    foreach ($flatRows as $acct) {
        $type = strtolower($acct['account_type'] ?? '');
        if (in_array($type, ['asset','liability','equity'])) {
            if ($currentType !== $type) {
                if ($currentType !== null) {
                    $processedAccounts[] = [
                        'account_type'=>'Subtotal',
                        'type'=>'t',
                        'code'=>'',
                        'name'=>'Total '.ucfirst($currentType),
                        'total_amount'=>$totals[$currentType],
                        'row_class'=>'subtotal',
                        'depth'=>0
                    ];
                }
                $currentType = $type;
            }
            $totals[$type] += floatval($acct['total_amount'] ?? 0);
        }
        $processedAccounts[] = $acct;
    }
    if ($currentType !== null) {
        $processedAccounts[] = [
            'account_type'=>'Subtotal',
            'type'=>'t',
            'code'=>'',
            'name'=>'Total '.ucfirst($currentType),
            'total_amount'=>$totals[$currentType],
            'row_class'=>'subtotal',
            'depth'=>0
        ];
    }

    foreach ($processedAccounts as &$p) {
        $p['balance'] = $p['total_amount'] ?? 0.0;
    }
    unset($p);
    $rows = $processedAccounts;

    // Draw
    $pdf->SetLineWidth($tblBorderWidth);
    $pdf->SetDrawColor($borderR, $borderG, $borderB);
    $pdf->SetXY($x, $y);

    if ($tblBorderShow) {
        $pdf->Rect($x, $y, $w, $h, 'D');
    }
    $tblBorder = $tblBorderShow ? 1 : 0;
    $rowHeight = 8;

    $codeW     = 20.0;
    $acctTypeW = 25.0;
    $balanceW  = 25.0;

    // The boolean flags
    $hasCode = !$hideCode;
    $hasAcctType = !$hideAcctType;

    $fixedTotal  = 0;
    if ($hasCode) {
        $fixedTotal += $codeW;
    }
    if ($hasAcctType) {
        $fixedTotal += $acctTypeW;
    }
    $fixedTotal += $balanceW;
    $nameW = $w - $fixedTotal;
    if ($nameW < 10) {
        $nameW = 10;
    }

    $cols = [];
    if ($hasCode) {
        $cols[] = ['id'=>'code','label'=>'Code','width'=>$codeW];
    }
    if ($hasAcctType) {
        $cols[] = ['id'=>'acctType','label'=>'Type','width'=>$acctTypeW];
    }
    $cols[] = ['id'=>'name','label'=>'Name','width'=>$nameW];
    $cols[] = ['id'=>'balance','label'=>'Balance','width'=>$balanceW];

    $colCount = count($cols);
    if ($colCount < 1) {
        return;
    }

    // Header style
    $hc=hexToRgb($hdrFontColor);
    $pdf->SetFont($hdrFontFam, $hdrFontSty, $hdrFontSize);
    $pdf->SetTextColor($hc['r'],$hc['g'],$hc['b']);
    $hf=hexToRgb($hdrFillColor);
    if ($hdrFill) {
        $pdf->SetFillColor($hf['r'],$hf['g'],$hf['b']);
    } else {
        $pdf->SetFillColor(255,255,255);
    }

    // Print header row
    $pdf->SetX($x);
    foreach ($cols as $c) {
        $pdf->Cell($c['width'], $rowHeight, $c['label'], $tblBorder, 0, 'C', $hdrFill);
    }
    $pdf->Ln($rowHeight);

    // Data
    $dfc = hexToRgb($dataFontColor);
    $pdf->SetFont($dataFontFam, $dataFontSty, $dataFontSize);
    $pdf->SetTextColor($dfc['r'],$dfc['g'],$dfc['b']);

    $f1=hexToRgb($dataFillColor);
    $f2=hexToRgb($dataFillColor2);

    $rowCount=0;
    foreach ($rows as $r) {
        if ($hideNoBalance && abs(floatval($r['balance']))<0.00001) {
            continue;
        }
        $rowCount++;

        if ($dataFill==='solid') {
            $pdf->SetFillColor($f1['r'],$f1['g'],$f1['b']);
        } else if ($dataFill==='stripe') {
            if ($rowCount%2===1) {
                $pdf->SetFillColor($f1['r'],$f1['g'],$f1['b']);
            } else {
                $pdf->SetFillColor($f2['r'],$f2['g'],$f2['b']);
            }
        } else {
            $pdf->SetFillColor(255,255,255);
        }

        $typ= strtolower($r['account_type'] ?? '');
        // If row says type:'t' => Subtotal color
        if (!empty($r['type']) && $r['type']==='t') {
            $rowColor = $colorTotals;
        } else {
            switch($typ){
                case 'asset':     $rowColor=$colorAsset;  break;
                case 'liability': $rowColor=$colorLiab;   break;
                case 'equity':    $rowColor=$colorEquity; break;
                case 'subtotal':  $rowColor=$colorTotals; break;
                default:          $rowColor=$dataFontColor; break;
            }
        }
        if (!$dataMulticolor) {
            $rowColor = $dataFontColor;
        }
        $rc=hexToRgb($rowColor);
        $pdf->SetTextColor($rc['r'],$rc['g'],$rc['b']);

        $depth=(int)($r['depth']??0);
        $indent = $dataHierarchy ? str_repeat('  ',$depth) : '';

        $displayCode = $r['code'] ?? '';
        $displayType = '';
        if (!empty($r['type']) && $r['type']==='t') {
            $displayType = 'SubTotal';
        } else {
            $displayType = strtoupper($r['account_type'] ?? '');
        }

        $displayName = $r['name'] ?? '';
        if ($hideCode && $displayCode !== '') {
            // if user wants to hide code col, we might prepend code to name
            $displayName = $displayCode.' '.$displayName;
            $displayCode = '';
        }
        if ($hideAcctType) {
            $displayType = '';
        }
        $balStr= number_format(floatval($r['balance']),2);

        $pdf->SetX($x);
        foreach ($cols as $c) {
            $cellText='';
            $align='L';
            switch($c['id']){
                case 'code':
                    $cellText=$displayCode;
                    break;
                case 'acctType':
                    $cellText=$displayType;
                    break;
                case 'name':
                    $cellText=$indent.$displayName;
                    break;
                case 'balance':
                    $cellText=$balStr;
                    $align='R';
                    break;
            }
            $pdf->Cell($c['width'], $rowHeight, $cellText, $tblBorder, 0, $align, true);
        }
        $pdf->Ln($rowHeight);
        $pdf->SetTextColor($dfc['r'],$dfc['g'],$dfc['b']);
    }

    $pdf->SetTextColor(0,0,0);
    $pdf->Ln(3);
}

function buildAccountTree(array $accounts, $parent=null): array {
    $branch=[];
    foreach($accounts as $acct) {
        $acctParent = (!empty($acct['parent']) && strtolower($acct['parent'])!=='n/a')
            ? $acct['parent'] : null;
        if ($acctParent === $parent) {
            $children = buildAccountTree($accounts, $acct['code']);
            if ($children) {
                $acct['children'] = $children;
            }
            $branch[] = $acct;
        }
    }
    return $branch;
}
function traverseAccountTree(array $tree, array &$sortedList, $depth=0): void {
    foreach($tree as $node) {
        $node['depth'] = $depth;
        $sortedList[]  = $node;
        if (isset($node['children'])) {
            traverseAccountTree($node['children'], $sortedList, $depth+1);
        }
    }
}
function hexToRgb($hex) {
    $hex = str_replace('#','',$hex);
    if(strlen($hex)===3){
        $hex=$hex[0].$hex[0].$hex[1].$hex[1].$hex[2].$hex[2];
    }
    $val=hexdec($hex);
    return [
      'r' => ($val >> 16) & 255,
      'g' => ($val >> 8)  & 255,
      'b' => $val & 255
    ];
}
