window.typeObjectCreators = window.typeObjectCreators || {};

window.typeObjectCreators['BSTable'] = function(nextId) {
    return {
        id: nextId,
        type: 'BSTable',
        x: 10,
        y: 10,
        width: 60,
        height: 50,
        borderShow: true,
        borderColor: '#000000',
        borderWidth: 1.0,
        headerFontFam: 'Courier',
        headerFontSty: '', // May be combined: "B", "I", "U"
        headerFontSize: 12,
        headerFontColor: '#000000',
        headerFill: true,
        headerFillColor: '#cccccc',

        dataFontFam: 'Courier',
        dataFontSty: '', // May be combined: "B", "I", "U"
        dataFontSize: 12,
        dataFontColor: '#000000',
        dataFill: 'stripe',
        dataFillColor: '#ffffff',
        dataFillColor2: '#f3f3f3',

        dataHierarchy: true,
        dataMulticolor: true,

        colorAsset:  '#26A269',
        colorLiab:   '#1A5FB4',
        colorEquity: '#DC8ADD',
        colorTotals: '#000000',

        hideCode: false,
        hideAcctType: false,
        hideNoBalance: false
    };
};

window.typeObjectCreators['updateBSTablePositionSize'] = function(elementObject, div) {
    const leftPx   = (elementObject.x || 0)   * 3;
    const topPx    = (elementObject.y || 0)   * 3;
    const widthPx  = (elementObject.width  || 60) * 3;
    const heightPx = (elementObject.height || 50) * 3;
    div.style.left   = leftPx + 'px';
    div.style.top    = topPx  + 'px';
    div.style.width  = widthPx + 'px';
    div.style.height = heightPx + 'px';
};

window.typeObjectCreators['_renderElementPreview'] = function(el, idx) {
    if (el.type !== 'BSTable') {
        return null;
    }
    const div = document.createElement('div');
    div.className = 'element';
    div.setAttribute('data-index', idx);
    div.style.border  = '1px dashed #aaa';
    div.style.margin  = '0';
    div.style.padding = '0';

    window.typeObjectCreators['updateBSTablePositionSize'](el, div);

    const table = document.createElement('table');
    table.style.margin = '0';
    table.style.padding = '0';
    table.style.borderCollapse = 'collapse';
    table.style.width  = '100%';
    table.style.height = '100%';
    table.style.boxSizing = 'border-box';

    const pxWidth = Math.max(1, Math.round((el.borderWidth || 0.2) * 5));
    const color   = el.borderColor || '#000000';

    if (el.borderShow) {
        table.style.border = pxWidth + 'px solid ' + color;
    } else {
        table.style.border = 'none';
    }

    div.style.display = 'flex';
    div.style.flexDirection = 'column';
    div.style.overflow = 'hidden'; // clip table if bigger than bounding box

    // Build column definitions
    const colDefs = [];
    if (!el.hideCode) {
        colDefs.push({ id:'code', header:'Code' });
    }
    if (!el.hideAcctType) {
        colDefs.push({ id:'type', header:'Type' });
    }
    colDefs.push({ id:'name', header:'Name' });
    colDefs.push({ id:'balance', header:'Balance' });

    // Header style
    const thBg    = el.headerFill       ? (el.headerFillColor || '#cccccc') : 'transparent';
    const thColor = el.headerFontColor  || '#000000';
    const thFam   = el.headerFontFam    || 'Courier';
    const thSize  = (el.headerFontSize  || 12) + 'px';
    // For combined style "B","I","U"
    const hStyle  = el.headerFontSty || '';
    const thWeight= hStyle.includes('B') ? 'bold'   : 'normal';
    const thItalic= hStyle.includes('I') ? 'italic' : 'normal';
    const thUline = hStyle.includes('U') ? 'underline' : 'none';

    const thead = document.createElement('thead');
    const thRow = document.createElement('tr');
    colDefs.forEach(def => {
        const th = document.createElement('th');
        th.textContent = def.header;
        th.style.width = (100 / colDefs.length) + '%';

        if (el.borderShow) {
            th.style.border = pxWidth + 'px solid ' + color;
        } else {
            th.style.border = 'none';
        }

        th.style.background     = thBg;
        th.style.color          = thColor;
        th.style.fontFamily     = thFam;
        th.style.fontSize       = thSize;
        th.style.fontWeight     = thWeight;
        th.style.fontStyle      = thItalic;
        th.style.textDecoration = thUline;
        th.style.textAlign      = 'center';
        thRow.appendChild(th);
    });
    thead.appendChild(thRow);
    table.appendChild(thead);

    // Data style
    const fillMode       = el.dataFill        || 'stripe';
    const fillColor1     = el.dataFillColor   || '#ffffff';
    const fillColor2     = el.dataFillColor2  || '#f3f3f3';
    const dataFontColor  = el.dataFontColor   || '#000000';
    const dataFontFam    = el.dataFontFam     || 'Courier';
    const dataFontSize   = (el.dataFontSize   || 12) + 'px';
    const dataStyle      = el.dataFontSty     || '';
    const dfWeight       = dataStyle.includes('B') ? 'bold'   : 'normal';
    const dfItalic       = dataStyle.includes('I') ? 'italic' : 'normal';
    const dfUline        = dataStyle.includes('U') ? 'underline' : 'none';
    const dataMulticolor = (typeof el.dataMulticolor==='undefined') ? true : el.dataMulticolor;
    const dataHierarchy  = (typeof el.dataHierarchy==='undefined') ? true : el.dataHierarchy;

    const colA = el.colorAsset  || '#26A269';
    const colL = el.colorLiab   || '#1A5FB4';
    const colE = el.colorEquity || '#DC8ADD';
    const colT = el.colorTotals || '#000000';

    const tbody = document.createElement('tbody');
    const sampleRows = [
        { code:'1000', type:'a', level:0, name:'Assets',              balance:'0.00' },
        { code:'1100', type:'a', level:1, name:'Current Assets',      balance:'0.00' },
        { code:'1110', type:'a', level:2, name:'Inventory',           balance:'0.00' },
        { code:'1120', type:'a', level:2, name:'Accounts Receivable', balance:'0.00' },
        { code:'',     type:'t', level:0, name:'Total Assets',        balance:'0.00' },

        { code:'2000', type:'l', level:0, name:'Liabilities',         balance:'0.00' },
        { code:'2100', type:'l', level:1, name:'Current Liabilities', balance:'0.00' },
        { code:'2110', type:'l', level:2, name:'Accounts Payable',    balance:'0.00' },
        { code:'3120', type:'l', level:2, name:'VAT Control Account', balance:'0.00' },
        { code:'',     type:'t', level:0, name:'Total Liability',     balance:'0.00' },

        { code:'3000', type:'e', level:0, name:'Equity',              balance:'0.00' },
        { code:'3100', type:'e', level:1, name:'Capital Contribution',balance:'0.00' },
        { code:'3200', type:'e', level:1, name:'Retained Earnings',   balance:'0.00' },
        { code:'3250', type:'e', level:2, name:'Net Income',          balance:'0.00' },
        { code:'',     type:'t', level:0, name:'Total Equity',        balance:'0.00' }
    ];

    let rowCount = 0;
    sampleRows.forEach(r => {
        rowCount++;
        const tr = document.createElement('tr');
        tr.setAttribute('data-type', r.type);
        tr.setAttribute('data-level', r.level);

        let rowBg = 'transparent';
        if (fillMode === 'solid') {
            rowBg = fillColor1;
        } else if (fillMode === 'stripe') {
            rowBg = (rowCount % 2 === 1) ? fillColor1 : fillColor2;
        }
        tr.style.background = rowBg;

        let rowColor = dataFontColor;
        if (dataMulticolor) {
            if (r.type==='a') rowColor = colA;
            else if (r.type==='l') rowColor = colL;
            else if (r.type==='e') rowColor = colE;
            else if (r.type==='t') rowColor = colT;
        }
        tr.style.color          = rowColor;
        tr.style.fontFamily     = dataFontFam;
        tr.style.fontSize       = dataFontSize;
        tr.style.fontWeight     = dfWeight;
        tr.style.fontStyle      = dfItalic;
        tr.style.textDecoration = dfUline;

        const colCount = colDefs.length;
        colDefs.forEach(def => {
            let cellText = '';
            let tdAlign  = 'left';
            if (def.id === 'code') {
                cellText = r.code || '';
            } else if (def.id === 'type') {
                if (r.type === 't') {
                    cellText = 'SubTotal';
                } else if (r.type==='a') cellText = 'ASSET';
                else if (r.type==='l') cellText = 'LIABILITY';
                else if (r.type==='e') cellText = 'EQUITY';
            } else if (def.id === 'name') {
                let cellTxt = r.name || '';
                if (dataHierarchy) {
                    const level = parseInt(r.level,10) || 0;
                    // Use non-breaking spaces
                    cellTxt = '\u00A0'.repeat(level*3) + cellTxt;
                }
                cellText = cellTxt;
            } else if (def.id === 'balance') {
                cellText = r.balance || '0.00';
                tdAlign = 'right';
            }
            const td = document.createElement('td');
            if (el.borderShow) {
                td.style.border = pxWidth + 'px solid ' + color;
            } else {
                td.style.border = 'none';
            }
            td.style.width = (100 / colCount) + '%';
            td.style.textAlign = tdAlign;

            // preserve spacing for indentation
            td.style.whiteSpace = 'pre';

            td.textContent = cellText;
            tr.appendChild(td);
        });

        tbody.appendChild(tr);
    });

    table.appendChild(tbody);
    div.appendChild(table);

    const resizeHandle = document.createElement('div');
    resizeHandle.className = 'resize-br-handle';
    div.appendChild(resizeHandle);

    resizeHandle.addEventListener('mousedown', e => {
        e.stopPropagation();
        e.preventDefault();
        activeElementIndex = idx;
        resizingElementIndex = idx;
        resizingCorner = 'BR';
        offsetX = e.offsetX;
        offsetY = e.offsetY;
    });

    div.addEventListener('mousedown', e => {
        activeElementDiv = div;
        isDragging = true;
        offsetX = e.offsetX;
        offsetY = e.offsetY;
        activeElementIndex = idx;
        loadProperties(idx);
        e.preventDefault();
        objectListSel.value = (el.id || '') + '';
    });

    return div;
};

window.typeFunctions = window.typeFunctions || {};
window.typeFunctions['balancesheet'] = window.typeFunctions['balancesheet'] || {};

window.typeFunctions['balancesheet'].updateProperty = function(el, propId, newValue) {
    if (!el || el.type !== 'BSTable') return;

    switch (propId) {
        case 'bstable-x':
            el.x = parseFloat(newValue) || 0;
            break;
        case 'bstable-y':
            el.y = parseFloat(newValue) || 0;
            break;
        case 'bstable-w':
            el.width = parseFloat(newValue)||60;
            break;
        case 'bstable-h':
            el.height= parseFloat(newValue)||50;
            break;
        case 'bstable-bordershow':
            el.borderShow = (newValue === 'on');
            break;
        case 'bstable-bordercolor':
            el.borderColor = newValue;
            break;
        case 'bstable-borderwidth':
            el.borderWidth = parseFloat(newValue)||1;
            break;

        // HEADER font style checkboxes:
        case 'bstable-headerBold':
            if (newValue === 'on') {
                if (!el.headerFontSty.includes('B')) el.headerFontSty += 'B';
            } else {
                el.headerFontSty = el.headerFontSty.replace(/B/g, '');
            }
            break;
        case 'bstable-headerItalic':
            if (newValue === 'on') {
                if (!el.headerFontSty.includes('I')) el.headerFontSty += 'I';
            } else {
                el.headerFontSty = el.headerFontSty.replace(/I/g, '');
            }
            break;
        case 'bstable-headerUnderline':
            if (newValue === 'on') {
                if (!el.headerFontSty.includes('U')) el.headerFontSty += 'U';
            } else {
                el.headerFontSty = el.headerFontSty.replace(/U/g, '');
            }
            break;

        case 'bstable-headerfontfam':
            el.headerFontFam = newValue;
            break;
        case 'bstable-headerfontsize':
            el.headerFontSize = parseFloat(newValue)||12;
            break;
        case 'bstable-headerfontcolor':
            el.headerFontColor = newValue;
            break;
        case 'bstable-headerfill':
            el.headerFill = (newValue === 'on');
            break;
        case 'bstable-headerfillcolor':
            el.headerFillColor = newValue;
            break;

        // DATA font style checkboxes:
        case 'bstable-dataBold':
            if (newValue === 'on') {
                if (!el.dataFontSty.includes('B')) el.dataFontSty += 'B';
            } else {
                el.dataFontSty = el.dataFontSty.replace(/B/g, '');
            }
            break;
        case 'bstable-dataItalic':
            if (newValue === 'on') {
                if (!el.dataFontSty.includes('I')) el.dataFontSty += 'I';
            } else {
                el.dataFontSty = el.dataFontSty.replace(/I/g, '');
            }
            break;
        case 'bstable-dataUnderline':
            if (newValue === 'on') {
                if (!el.dataFontSty.includes('U')) el.dataFontSty += 'U';
            } else {
                el.dataFontSty = el.dataFontSty.replace(/U/g, '');
            }
            break;

        case 'bstable-datafontfam':
            el.dataFontFam = newValue;
            break;
        case 'bstable-datafontsize':
            el.dataFontSize = parseFloat(newValue)||12;
            break;
        case 'bstable-datafontcolor':
            el.dataFontColor = newValue;
            break;
        case 'bstable-datafill':
            el.dataFill = newValue;
            break;
        case 'bstable-datafillcolor':
            el.dataFillColor = newValue;
            break;
        case 'bstable-datafillcolor2':
            el.dataFillColor2 = newValue;
            break;
        case 'bstable-datahierarchy':
            el.dataHierarchy = (newValue === 'on');
            break;
        case 'bstable-datamulticolor':
            el.dataMulticolor = (newValue === 'on');
            break;
        case 'bstable-colorasset':
            el.colorAsset = newValue;
            break;
        case 'bstable-colorliab':
            el.colorLiab = newValue;
            break;
        case 'bstable-colorequity':
            el.colorEquity = newValue;
            break;
        case 'bstable-colortotals':
            el.colorTotals = newValue;
            break;
        case 'bstable-hidecode':
            el.hideCode = (newValue === 'on');
            break;
        case 'bstable-hideaccttype':
            el.hideAcctType = (newValue === 'on');
            break;
        case 'bstable-hidenobalance':
            el.hideNoBalance = (newValue === 'on');
            break;

        default:
            console.warn("Unhandled BSTable property:", propId, "value:", newValue);
    }
};

window.typeFunctions['balancesheet'].populateProperties = function(el) {
    if (!el || el.type !== 'BSTable') return;

    const xInput = document.getElementById('bstable-x');
    if (xInput) xInput.value = el.x || 0;

    const yInput = document.getElementById('bstable-y');
    if (yInput) yInput.value = el.y || 0;

    const wInput = document.getElementById('bstable-w');
    if (wInput) wInput.value = el.width || 60;

    const hInput = document.getElementById('bstable-h');
    if (hInput) hInput.value = el.height || 50;

    const borderShow = document.getElementById('bstable-bordershow');
    if (borderShow) borderShow.checked = !!el.borderShow;

    const borderColor = document.getElementById('bstable-bordercolor');
    if (borderColor) borderColor.value = el.borderColor || '#000000';

    const borderWidth = document.getElementById('bstable-borderwidth');
    if (borderWidth) borderWidth.value = el.borderWidth || 1;

    // Header
    const hfam = document.getElementById('bstable-headerfontfam');
    if (hfam) hfam.value = el.headerFontFam || 'Courier';

    const hsize = document.getElementById('bstable-headerfontsize');
    if (hsize) hsize.value = el.headerFontSize || 12;

    const hcolor = document.getElementById('bstable-headerfontcolor');
    if (hcolor) hcolor.value = el.headerFontColor || '#000000';

    const hfill = document.getElementById('bstable-headerfill');
    if (hfill) hfill.checked = !!el.headerFill;

    const hfillcolor = document.getElementById('bstable-headerfillcolor');
    if (hfillcolor) hfillcolor.value = el.headerFillColor || '#cccccc';

    // Header style checkboxes
    const hBold = document.getElementById('bstable-headerBold');
    const hItal = document.getElementById('bstable-headerItalic');
    const hUndr = document.getElementById('bstable-headerUnderline');

    if (hBold)  hBold.checked  = (el.headerFontSty || '').includes('B');
    if (hItal)  hItal.checked  = (el.headerFontSty || '').includes('I');
    if (hUndr)  hUndr.checked = (el.headerFontSty || '').includes('U');

    // Data
    const dfam = document.getElementById('bstable-datafontfam');
    if (dfam) dfam.value = el.dataFontFam || 'Courier';

    const dsize = document.getElementById('bstable-datafontsize');
    if (dsize) dsize.value = el.dataFontSize || 12;

    const dcolor = document.getElementById('bstable-datafontcolor');
    if (dcolor) dcolor.value = el.dataFontColor || '#000000';

    const datafill = document.getElementById('bstable-datafill');
    if (datafill) datafill.value = el.dataFill || 'stripe';

    const datafill1 = document.getElementById('bstable-datafillcolor');
    if (datafill1) datafill1.value = el.dataFillColor || '#ffffff';

    const datafill2 = document.getElementById('bstable-datafillcolor2');
    if (datafill2) datafill2.value = el.dataFillColor2 || '#f3f3f3';

    const hier = document.getElementById('bstable-datahierarchy');
    if (hier) hier.checked = !!el.dataHierarchy;

    const multi = document.getElementById('bstable-datamulticolor');
    if (multi) multi.checked = !!el.dataMulticolor;

    const colA = document.getElementById('bstable-colorasset');
    if (colA) colA.value = el.colorAsset || '#26A269';

    const colL = document.getElementById('bstable-colorliab');
    if (colL) colL.value = el.colorLiab || '#1A5FB4';

    const colE = document.getElementById('bstable-colorequity');
    if (colE) colE.value = el.colorEquity || '#DC8ADD';

    const colT = document.getElementById('bstable-colortotals');
    if (colT) colT.value = el.colorTotals || '#000000';

    const hideC = document.getElementById('bstable-hidecode');
    if (hideC) hideC.checked = !!el.hideCode;

    const hideA = document.getElementById('bstable-hideaccttype');
    if (hideA) hideA.checked = !!el.hideAcctType;

    const hideNB = document.getElementById('bstable-hidenobalance');
    if (hideNB) hideNB.checked = !!el.hideNoBalance;
};

window.typeFunctions['balancesheet'].showTypeDialog = function(dialogObj, dialogCss) {
    if (dialogCss) {
        const styleTag = document.getElementById('typeDialogDynamicCss') || document.createElement('style');
        styleTag.id = 'typeDialogDynamicCss';
        styleTag.innerHTML = dialogCss;
        document.head.appendChild(styleTag);
    }
    const dialogTitle = document.getElementById('type-dialog-title');
    const dialogBody  = document.getElementById('type-dialog-body');
    const modal       = document.getElementById('type-dialog-modal');
    const btnCancel   = document.getElementById('type-dialog-cancel');
    const btnOk       = document.getElementById('type-dialog-ok');

    dialogTitle.textContent = dialogObj.title || 'Parameters';
    dialogBody.innerHTML    = '';

    const fieldElements = {};
    dialogObj.fields.forEach(field => {
        const labelEl = document.createElement('label');
        labelEl.textContent = field.label || field.name;
        labelEl.style.display = 'block';
        dialogBody.appendChild(labelEl);

        let inputEl;
        if (field.type === 'select') {
            inputEl = document.createElement('select');
            if (field.optionsEndpoint) {
                const url = `documents/type/balancesheet/dialog.php?${field.optionsEndpoint}`;
                fetch(url)
                .then(r => r.json())
                .then(opts => {
                    if (Array.isArray(opts)) {
                        const defaultOption = document.createElement('option');
                        defaultOption.value = '';
                        defaultOption.textContent = '-- Custom Date Range --';
                        inputEl.appendChild(defaultOption);
                        opts.forEach(opt => {
                            const optionNode = document.createElement('option');
                            optionNode.value = opt.id;
                            optionNode.textContent = opt.name || opt.id;
                            inputEl.appendChild(optionNode);
                        });
                    }
                })
                .catch(err => console.error("Error loading select options:", err));
            }
        } else {
            inputEl = document.createElement('input');
            inputEl.type = field.type || 'text';
        }
        inputEl.style.width = '100%';
        dialogBody.appendChild(inputEl);
        fieldElements[field.name] = inputEl;
    });

    modal.style.display = 'flex';

    btnCancel.onclick = () => {
        modal.style.display = 'none';
    };
    btnOk.onclick = () => {
        const formData = new FormData();
        dialogObj.fields.forEach(field => {
            const val = fieldElements[field.name].value;
            formData.append(field.name, val);
        });
        fetch('documents/type/balancesheet/dialog.php', {
            method: 'POST',
            body: formData
        })
        .then(r => r.json())
        .then(resp => {
            if (!resp.success) {
                console.error("Dialog submission failed:", resp.message);
                modal.style.display = 'none';
                return;
            }
            currentLayout.dialogData = resp.data || {};
            modal.style.display = 'none';
            postLayoutToPdf();
        })
        .catch(err => {
            console.error("Error in dialog submission:", err);
            modal.style.display = 'none';
        });
    };
};
