window.typeObjectCreators = window.typeObjectCreators || {};

//
// This creates a new "CFTable" object with certain default properties
// (similar to how 'BS Table' is done for the Balance Sheet).
//
window.typeObjectCreators['CFTable'] = function(nextId) {
  return {
    id: nextId,
    type: 'CFTable',
    x: 10,
    y: 10,
    width: 60,
    height: 50,
    bordershow: true,
    bordercolor: '#000000',
    borderwidth: 1.0,

    headerfontfam: 'Courier',
    headerFontSty: '',      // e.g. 'B', 'I', 'U'
    headerfontsize: 12,
    headerfontcolor: '#000000',
    headerfill: true,
    headerfillcolor: '#cccccc',

    datafontfam: 'Courier',
    dataFontSty: '',
    datafontsize: 12,
    datafontcolor: '#000000',
    datafill: 'stripe',
    datafillcolor: '#ffffff',
    datafillcolor2: '#f3f3f3'
  };
};

//
// Called by the editor to render a preview of the CF Table
//
window.typeObjectCreators['_renderElementPreview'] = window.typeObjectCreators['_renderElementPreview'] || function(el, idx) {
  // If some other type has a specialized preview, it might handle it.
  // We'll check if it's CFTable here:
  if (el.type === 'CFTable') {
    // Create a container DIV
    const div = document.createElement('div');
    div.className = 'element';
    div.setAttribute('data-index', idx);
    div.style.border = '1px dashed #aaa';
    div.style.margin = '0';
    div.style.padding= '0';

    // Helper to position/size the main container
    function updateCFTablePositionSize(elementObject, container) {
      const leftPx   = (elementObject.x || 0) * 3;
      const topPx    = (elementObject.y || 0) * 3;
      const widthPx  = (elementObject.width  || 60) * 3;
      const heightPx = (elementObject.height || 50) * 3;
      container.style.left   = leftPx + 'px';
      container.style.top    = topPx  + 'px';
      container.style.width  = widthPx + 'px';
      container.style.height = heightPx + 'px';
    }
    updateCFTablePositionSize(el, div);

    // Build a quick table:
    const table = document.createElement('table');
    table.style.margin = '0';
    table.style.padding= '0';
    table.style.borderCollapse = 'collapse';
    table.style.width  = '100%';
    table.style.height = '100%';
    table.style.boxSizing = 'border-box';

    // Border style
    const borderWidthPx = Math.max(1, Math.round((el.borderwidth || 1.0) * 3));
    const borderColor   = el.bordercolor || '#000000';
    if (el.bordershow) {
      table.style.border = borderWidthPx + 'px solid ' + borderColor;
    } else {
      table.style.border = 'none';
    }

    // Create THEAD
    const thead = document.createElement('thead');
    const thr   = document.createElement('tr');

    const th1   = document.createElement('th');
    th1.textContent = 'Cash Flow Activities';
    th1.style.textAlign = 'center';

    const th2   = document.createElement('th');
    th2.textContent = 'Amount';
    th2.style.textAlign = 'center';

    // Style the header
    const hcolor = el.headerfontcolor || '#000000';
    const hsize  = (el.headerfontsize || 12) + 'px';
    const fillColor= el.headerfillcolor || '#cccccc';
    const headStyles = (el.headerFontSty || '');
    const isBold   = headStyles.includes('B');
    const isItalic = headStyles.includes('I');
    const isUnderl = headStyles.includes('U');

    if (el.headerfill) {
      thr.style.background = fillColor;
    } else {
      thr.style.background = 'transparent';
    }
    thr.style.color = hcolor;
    thr.style.fontFamily = el.headerfontfam || 'Courier';
    thr.style.fontSize   = hsize;
    thr.style.fontWeight = isBold   ? 'bold'      : 'normal';
    thr.style.fontStyle  = isItalic ? 'italic'    : 'normal';
    thr.style.textDecoration = isUnderl ? 'underline' : 'none';

    // border for each th if table border is on
    if (el.bordershow) {
      th1.style.border = borderWidthPx + 'px solid ' + borderColor;
      th2.style.border = borderWidthPx + 'px solid ' + borderColor;
    } else {
      th1.style.border = 'none';
      th2.style.border = 'none';
    }

    thr.appendChild(th1);
    thr.appendChild(th2);
    thead.appendChild(thr);
    table.appendChild(thead);

    // TBody with sample rows
    const tbody = document.createElement('tbody');

    // Data style
    const dataColor   = el.datafontcolor || '#000000';
    const dataSize    = (el.datafontsize || 12) + 'px';
    const dataSty     = (el.dataFontSty  || '');
    const dBold       = dataSty.includes('B');
    const dItalic     = dataSty.includes('I');
    const dUnderl     = dataSty.includes('U');
    const fill1       = el.datafillcolor  || '#ffffff';
    const fill2       = el.datafillcolor2 || '#f3f3f3';
    const fillMode    = el.datafill       || 'stripe';

    const sampleRows = [
      { label:'Operating Activities', amount:'0.00' },
      { label:'Investing Activities', amount:'0.00' },
      { label:'Financing Activities', amount:'0.00' },
      { label:'Net Increase in Cash', amount:'0.00' }
    ];

    let rowCount=0;
    sampleRows.forEach(sr => {
      rowCount++;
      const tr   = document.createElement('tr');

      // fill color logic
      let rowBg = 'transparent';
      if (fillMode === 'solid') {
        rowBg = fill1;
      } else if (fillMode === 'stripe') {
        rowBg = (rowCount % 2 === 1) ? fill1 : fill2;
      }
      tr.style.background = rowBg;

      tr.style.color       = dataColor;
      tr.style.fontSize    = dataSize;
      tr.style.fontFamily  = el.datafontfam || 'Courier';
      tr.style.fontWeight  = dBold   ? 'bold' : 'normal';
      tr.style.fontStyle   = dItalic ? 'italic' : 'normal';
      tr.style.textDecoration = dUnderl ? 'underline' : 'none';

      // Create TDs
      const td1 = document.createElement('td');
      td1.textContent = sr.label;
      td1.style.textAlign = 'left';

      const td2 = document.createElement('td');
      td2.textContent = sr.amount;
      td2.style.textAlign = 'right';

      if (el.bordershow) {
        td1.style.border = borderWidthPx + 'px solid ' + borderColor;
        td2.style.border = borderWidthPx + 'px solid ' + borderColor;
      } else {
        td1.style.border = 'none';
        td2.style.border = 'none';
      }

      tr.appendChild(td1);
      tr.appendChild(td2);
      tbody.appendChild(tr);
    });

    table.appendChild(tbody);
    div.appendChild(table);

    // A resize handle
    const resizeHandle = document.createElement('div');
    resizeHandle.className = 'resize-br-handle';
    div.appendChild(resizeHandle);
    resizeHandle.addEventListener('mousedown', e=>{
      e.stopPropagation();
      e.preventDefault();
      activeElementIndex=idx;
      resizingElementIndex=idx;
      resizingCorner='BR';
      offsetX=e.offsetX;
      offsetY=e.offsetY;
    });

    // Draggable
    div.addEventListener('mousedown', e=>{
      activeElementDiv=div;
      isDragging=true;
      offsetX=e.offsetX;
      offsetY=e.offsetY;
      activeElementIndex=idx;
      loadProperties(idx);
      e.preventDefault();
      objectListSel.value = (el.id||'')+'';
    });

    return div;
  }

  // If not CFTable, return null and let other type objects handle
  return null;
};

// A container for specific per-type logic:
window.typeFunctions = window.typeFunctions || {};
window.typeFunctions['cashflow'] = window.typeFunctions['cashflow'] || {};

// Update property from the property form
window.typeFunctions['cashflow'].updateProperty = function(el, propId, newValue) {
  if (!el || el.type !== 'CFTable') return;
  switch (propId) {
    case 'cftable-x':
      el.x = parseFloat(newValue) || 0;
      break;
    case 'cftable-y':
      el.y = parseFloat(newValue) || 0;
      break;
    case 'cftable-w':
      el.width = parseFloat(newValue) || 60;
      break;
    case 'cftable-h':
      el.height= parseFloat(newValue) || 50;
      break;
    case 'cftable-bordershow':
      el.bordershow = (newValue === 'on');
      break;
    case 'cftable-bordercolor':
      el.bordercolor = newValue;
      break;
    case 'cftable-borderwidth':
      el.borderwidth = parseFloat(newValue) || 1.0;
      break;

    // Header:
    case 'cftable-headerBold':
      if (newValue === 'on') {
        if (!el.headerFontSty.includes('B')) el.headerFontSty += 'B';
      } else {
        el.headerFontSty = el.headerFontSty.replace(/B/g,'');
      }
      break;
    case 'cftable-headerItalic':
      if (newValue === 'on') {
        if (!el.headerFontSty.includes('I')) el.headerFontSty += 'I';
      } else {
        el.headerFontSty = el.headerFontSty.replace(/I/g,'');
      }
      break;
    case 'cftable-headerUnderline':
      if (newValue === 'on') {
        if (!el.headerFontSty.includes('U')) el.headerFontSty += 'U';
      } else {
        el.headerFontSty = el.headerFontSty.replace(/U/g,'');
      }
      break;
    case 'cftable-headerfontfam':
      el.headerfontfam = newValue;
      break;
    case 'cftable-headerfontsize':
      el.headerfontsize = parseFloat(newValue) || 12;
      break;
    case 'cftable-headerfontcolor':
      el.headerfontcolor = newValue;
      break;
    case 'cftable-headerfill':
      el.headerfill = (newValue === 'on');
      break;
    case 'cftable-headerfillcolor':
      el.headerfillcolor = newValue;
      break;

    // Data:
    case 'cftable-dataBold':
      if (newValue === 'on') {
        if (!el.dataFontSty.includes('B')) el.dataFontSty += 'B';
      } else {
        el.dataFontSty = el.dataFontSty.replace(/B/g,'');
      }
      break;
    case 'cftable-dataItalic':
      if (newValue === 'on') {
        if (!el.dataFontSty.includes('I')) el.dataFontSty += 'I';
      } else {
        el.dataFontSty = el.dataFontSty.replace(/I/g,'');
      }
      break;
    case 'cftable-dataUnderline':
      if (newValue === 'on') {
        if (!el.dataFontSty.includes('U')) el.dataFontSty += 'U';
      } else {
        el.dataFontSty = el.dataFontSty.replace(/U/g,'');
      }
      break;
    case 'cftable-datafontfam':
      el.datafontfam = newValue;
      break;
    case 'cftable-datafontsize':
      el.datafontsize = parseFloat(newValue) || 12;
      break;
    case 'cftable-datafontcolor':
      el.datafontcolor = newValue;
      break;
    case 'cftable-datafill':
      el.datafill = newValue;
      break;
    case 'cftable-datafillcolor':
      el.datafillcolor = newValue;
      break;
    case 'cftable-datafillcolor2':
      el.datafillcolor2 = newValue;
      break;

    default:
      console.warn("Unhandled CFTable property:", propId, "value:", newValue);
  }
};

// Populate the property form for the CF Table
window.typeFunctions['cashflow'].populateProperties = function(el) {
  if (!el || el.type !== 'CFTable') return;

  const xInput = document.getElementById('cftable-x');
  if (xInput) xInput.value = el.x || 0;

  const yInput = document.getElementById('cftable-y');
  if (yInput) yInput.value = el.y || 0;

  const wInput = document.getElementById('cftable-w');
  if (wInput) wInput.value = el.width || 60;

  const hInput = document.getElementById('cftable-h');
  if (hInput) hInput.value = el.height || 50;

  const borderShow = document.getElementById('cftable-bordershow');
  if (borderShow) borderShow.checked = !!el.bordershow;

  const borderColor= document.getElementById('cftable-bordercolor');
  if (borderColor) borderColor.value = el.bordercolor || '#000000';

  const borderWidth= document.getElementById('cftable-borderwidth');
  if (borderWidth) borderWidth.value = el.borderwidth || 1.0;

  // Header
  const hfam = document.getElementById('cftable-headerfontfam');
  if (hfam) hfam.value = el.headerfontfam || 'Courier';

  const hfsize= document.getElementById('cftable-headerfontsize');
  if (hfsize) hfsize.value = el.headerfontsize || 12;

  const hfcolor= document.getElementById('cftable-headerfontcolor');
  if (hfcolor) hfcolor.value = el.headerfontcolor || '#000000';

  const hfill = document.getElementById('cftable-headerfill');
  if (hfill) hfill.checked = !!el.headerfill;

  const hfillcolor = document.getElementById('cftable-headerfillcolor');
  if (hfillcolor) hfillcolor.value = el.headerfillcolor || '#cccccc';

  const hBold = document.getElementById('cftable-headerBold');
  const hItal = document.getElementById('cftable-headerItalic');
  const hUndr = document.getElementById('cftable-headerUnderline');
  if (hBold) hBold.checked  = (el.headerFontSty||'').includes('B');
  if (hItal) hItal.checked  = (el.headerFontSty||'').includes('I');
  if (hUndr) hUndr.checked = (el.headerFontSty||'').includes('U');

  // Data
  const dfam = document.getElementById('cftable-datafontfam');
  if (dfam) dfam.value = el.datafontfam || 'Courier';

  const dfsize= document.getElementById('cftable-datafontsize');
  if (dfsize) dfsize.value = el.datafontsize || 12;

  const dfcolor= document.getElementById('cftable-datafontcolor');
  if (dfcolor) dfcolor.value = el.datafontcolor || '#000000';

  const dBold = document.getElementById('cftable-dataBold');
  const dItal = document.getElementById('cftable-dataItalic');
  const dUndr = document.getElementById('cftable-dataUnderline');
  if (dBold) dBold.checked  = (el.dataFontSty||'').includes('B');
  if (dItal) dItal.checked  = (el.dataFontSty||'').includes('I');
  if (dUndr) dUndr.checked = (el.dataFontSty||'').includes('U');

  const dFill = document.getElementById('cftable-datafill');
  if (dFill) dFill.value = el.datafill || 'stripe';

  const dFillC1 = document.getElementById('cftable-datafillcolor');
  if (dFillC1) dFillC1.value = el.datafillcolor || '#ffffff';

  const dFillC2 = document.getElementById('cftable-datafillcolor2');
  if (dFillC2) dFillC2.value = el.datafillcolor2 || '#f3f3f3';
};

// Show the type dialog (cashflow)
window.typeFunctions['cashflow'].showTypeDialog = function(dialogObj, dialogCss) {
  // Similar approach to the balancesheet showTypeDialog:
  if (dialogCss) {
    const styleTag = document.getElementById('typeDialogDynamicCss') || document.createElement('style');
    styleTag.id = 'typeDialogDynamicCss';
    styleTag.innerHTML = dialogCss;
    document.head.appendChild(styleTag);
  }
  const dialogTitle = document.getElementById('type-dialog-title');
  const dialogBody  = document.getElementById('type-dialog-body');
  const modal       = document.getElementById('type-dialog-modal');
  const btnCancel   = document.getElementById('type-dialog-cancel');
  const btnOk       = document.getElementById('type-dialog-ok');

  dialogTitle.textContent = dialogObj.title || 'Parameters';
  dialogBody.innerHTML    = '';

  const fieldElements = {};
  dialogObj.fields.forEach(field => {
    const labelEl = document.createElement('label');
    labelEl.textContent = field.label || field.name;
    labelEl.style.display = 'block';
    dialogBody.appendChild(labelEl);

    let inputEl;
    if (field.type === 'select') {
      inputEl = document.createElement('select');
      if (field.optionsEndpoint) {
        const url = `documents/type/cashflow/dialog.php?${field.optionsEndpoint}`;
        fetch(url)
          .then(r => r.json())
          .then(opts => {
            if (Array.isArray(opts)) {
              const defaultOption = document.createElement('option');
              defaultOption.value = '';
              defaultOption.textContent = '-- Custom Date Range --';
              inputEl.appendChild(defaultOption);

              opts.forEach(opt => {
                const optionNode = document.createElement('option');
                optionNode.value = opt.id;
                optionNode.textContent = opt.name || opt.id;
                inputEl.appendChild(optionNode);
              });
            }
          })
          .catch(err => console.error("Error loading select options:", err));
      }
    } else {
      inputEl = document.createElement('input');
      inputEl.type = field.type || 'text';
    }
    inputEl.style.width = '100%';
    dialogBody.appendChild(inputEl);
    fieldElements[field.name] = inputEl;
  });

  modal.style.display = 'flex';

  btnCancel.onclick = () => {
    modal.style.display = 'none';
  };
  btnOk.onclick = () => {
    const formData = new FormData();
    dialogObj.fields.forEach(field => {
      const val = fieldElements[field.name].value;
      formData.append(field.name, val);
    });
    fetch('documents/type/cashflow/dialog.php', {
      method: 'POST',
      body: formData
    })
    .then(r => r.json())
    .then(resp => {
      if (!resp.success) {
        console.error("Dialog submission failed:", resp.message);
        modal.style.display = 'none';
        return;
      }
      currentLayout.dialogData = resp.data || {};
      modal.style.display = 'none';
      postLayoutToPdf();
    })
    .catch(err => {
      console.error("Error in dialog submission:", err);
      modal.style.display = 'none';
    });
  };
};
