window.typeObjectCreators = window.typeObjectCreators || {};

/**
 * Creates a new "ISTable" object, similar to BSTable but for Income Statement.
 */
window.typeObjectCreators['ISTable'] = function(nextId) {
  return {
    id: nextId,
    type: 'ISTable',
    x: 10,
    y: 10,
    width: 60,
    height: 50,
    borderShow: true,
    borderColor: '#000000',
    borderWidth: 1.0,
    headerFontFam: 'Courier',
    headerFontSty: '',
    headerFontSize: 12,
    headerFontColor: '#000000',
    headerFill: true,
    headerFillColor: '#cccccc',

    dataFontFam: 'Courier',
    dataFontSty: '',
    dataFontSize: 12,
    dataFontColor: '#000000',
    dataFill: 'stripe',
    dataFillColor: '#ffffff',
    dataFillColor2: '#f3f3f3',

    // New "hierarchy" and "multicolor" flags
    dataHierarchy:  false,
    dataMulticolor: true,

    // Unique for Income Statement:
    colorRevenue: '#20c997',
    colorExpense: '#dc3545',
    colorTotals:  '#000000',

    // Hide no-balance rows
    hideNoBalance: false
  };
};

/**
 * Renders a basic preview of ISTable within the editor.
 */
window.typeObjectCreators['_renderElementPreview'] = window.typeObjectCreators['_renderElementPreview'] || function(el, idx){
  if (el.type !== 'ISTable') {
    return null; // Not our type
  }

  const div = document.createElement('div');
  div.className = 'element';
  div.setAttribute('data-index', idx);
  div.style.border = '1px dashed #aaa';
  div.style.margin = '0';
  div.style.padding= '0';

  // Position & size
  updateISTablePositionSize(el, div);

  // Create a table to show sample data
  const table = document.createElement('table');
  table.style.margin = '0';
  table.style.padding= '0';
  table.style.borderCollapse = 'collapse';
  table.style.width  = '100%';
  table.style.height = '100%';
  table.style.boxSizing = 'border-box';

  const pxWidth = Math.max(1, Math.round((el.borderWidth || 0.2) * 5));
  const color   = el.borderColor || '#000000';
  if (el.borderShow) {
    table.style.border = pxWidth + 'px solid ' + color;
  } else {
    table.style.border = 'none';
  }

  div.style.display = 'flex';
  div.style.flexDirection = 'column';
  div.style.overflow = 'hidden'; // clip if bigger

  // Determine header font style in preview
  const hfSty = el.headerFontSty || '';
  const headerStyleBold = hfSty.includes('B');
  const headerStyleItalic = hfSty.includes('I');
  const headerStyleUnderline = hfSty.includes('U');

  // Build columns: "Account Name" (70%), "Amount" (30%)
  const thead = document.createElement('thead');
  const thr   = document.createElement('tr');

  const th1   = document.createElement('th');
  th1.textContent = 'Account Name';
  const th2   = document.createElement('th');
  th2.textContent = 'Amount';

  [th1, th2].forEach(th => {
    if (el.borderShow) {
      th.style.border = pxWidth + 'px solid ' + color;
    } else {
      th.style.border = 'none';
    }
    th.style.background = el.headerFill ? (el.headerFillColor || '#cccccc') : 'transparent';
    th.style.color      = el.headerFontColor || '#000000';

    // mimic font style for header
    th.style.fontFamily  = el.headerFontFam || 'Courier';
    th.style.fontSize    = (el.headerFontSize || 12) + 'px';
    th.style.fontWeight  = headerStyleBold ? 'bold'   : 'normal';
    th.style.fontStyle   = headerStyleItalic ? 'italic' : 'normal';
    th.style.textDecoration = headerStyleUnderline ? 'underline' : 'none';

    // Make header text center
    th.style.textAlign = 'center';
  });

  thr.appendChild(th1);
  thr.appendChild(th2);
  thead.appendChild(thr);
  table.appendChild(thead);

  const tbody = document.createElement('tbody');

  // Example rows for preview
  const sampleRows = [
    { group:'R',code:'4000',parent:null, level:0, name:'Income',         amount:'0.00' },
    { group:'R',code:'4100',parent:'4000', level:1, name:'Sales',          amount:'0.00' },
    { group:'R',code:'4200',parent:'4000', level:1, name:'Other Income',   amount:'0.00' },
    { group:'E',code:'5000',parent:null, level:0, name:'Expenses',       amount:'0.00' },
    { group:'E',code:'5100',parent:'5000', level:1, name:'Cost of Sales',  amount:'0.00' },
    { group:'E',code:'5200',parent:'5000', level:1, name:'General Expenses', amount:'0.00' },
    { group:'T',code:'1000',parent:null, level:0, name:'Total Revenue',  amount:'0.00' },
    { group:'T',code:'1000',parent:null, level:0, name:'Total Expenses', amount:'0.00' },
    { group:'T',code:'1000',parent:null, level:0, name:'Net Income',     amount:'0.00' }
  ];

  const dSty  = el.dataFontSty || '';
  const dataStyleBold      = dSty.includes('B');
  const dataStyleItalic    = dSty.includes('I');
  const dataStyleUnderline = dSty.includes('U');

  const pxFontSize = (el.dataFontSize || 12) + 'px';
  const fillMode   = el.dataFill || 'stripe';
  const f1         = el.dataFillColor || '#ffffff';
  const f2         = el.dataFillColor2|| '#f3f3f3';
  let rowCount = 0;

  sampleRows.forEach(r => {
    rowCount++;
    // Possibly skip zero-balance row if hideNoBalance (but in preview, let's show all)
    // if (el.hideNoBalance && (r.amount==='0.00')) { ... }

    const tr = document.createElement('tr');
    // Stripe or no fill
    let rowBg = 'transparent';
    if (fillMode === 'solid') {
      rowBg = f1;
    } else if (fillMode === 'stripe') {
      rowBg = (rowCount % 2 === 1) ? f1 : f2;
    }
    tr.style.background = rowBg;

    let rowColor = el.dataFontColor || '#000000';
    if (el.dataMulticolor) {
      if (r.group==='R') {
        rowColor = el.colorRevenue || '#20c997';
      } else if (r.group==='E') {
        rowColor = el.colorExpense || '#dc3545';
      } else if (r.group==='T') {
        rowColor = el.colorTotals  || '#000000';
      }
    }

    tr.style.color       = rowColor;
    tr.style.fontFamily  = el.dataFontFam || 'Courier';
    tr.style.fontSize    = pxFontSize;
    tr.style.fontWeight  = dataStyleBold ? 'bold'   : 'normal';
    tr.style.fontStyle   = dataStyleItalic ? 'italic' : 'normal';
    tr.style.textDecoration = dataStyleUnderline ? 'underline' : 'none';

    // Hierarchy indentation
    let rowName = r.name;
    if (el.dataHierarchy && typeof r.level==='number') {
      rowName = '\u00A0'.repeat(r.level) + rowName;
    }

    // Create TDs
    const tdName = document.createElement('td');
    tdName.textContent = rowName;

    const tdAmount= document.createElement('td');
    tdAmount.textContent = r.amount;
    tdAmount.style.textAlign = 'right';

    [tdName, tdAmount].forEach(td => {
      if (el.borderShow) {
        td.style.border = pxWidth + 'px solid ' + color;
      } else {
        td.style.border = 'none';
      }
    });

    tr.appendChild(tdName);
    tr.appendChild(tdAmount);
    tbody.appendChild(tr);
  });

  table.appendChild(tbody);
  div.appendChild(table);

  // Add resize handle
  const resizeHandle = document.createElement('div');
  resizeHandle.className = 'resize-br-handle';
  div.appendChild(resizeHandle);

  resizeHandle.addEventListener('mousedown', e => {
    e.stopPropagation();
    e.preventDefault();
    activeElementIndex = idx;
    resizingElementIndex = idx;
    resizingCorner = 'BR';
    offsetX = e.offsetX;
    offsetY = e.offsetY;
  });

  div.addEventListener('mousedown', e => {
    activeElementDiv = div;
    isDragging = true;
    offsetX = e.offsetX;
    offsetY = e.offsetY;
    activeElementIndex = idx;
    loadProperties(idx);
    e.preventDefault();
    objectListSel.value = (el.id || '') + '';
  });

  return div;
};

// Helper for positioning the preview
function updateISTablePositionSize(elementObject, div) {
  const leftPx   = (elementObject.x || 0)   * 3;
  const topPx    = (elementObject.y || 0)   * 3;
  const widthPx  = (elementObject.width  || 60) * 3;
  const heightPx = (elementObject.height || 50) * 3;
  div.style.left   = leftPx + 'px';
  div.style.top    = topPx  + 'px';
  div.style.width  = widthPx + 'px';
  div.style.height = heightPx + 'px';
}

window.typeFunctions = window.typeFunctions || {};
window.typeFunctions['incomestatement'] = window.typeFunctions['incomestatement'] || {};

/**
 * Called when the user changes a property in the property-box for the ISTable.
 */
window.typeFunctions['incomestatement'].updateProperty = function(el, propId, newValue) {
  if (!el || el.type !== 'ISTable') return;
  switch (propId) {
    case 'istable-x':
      el.x = parseFloat(newValue) || 0;
      break;
    case 'istable-y':
      el.y = parseFloat(newValue) || 0;
      break;
    case 'istable-w':
      el.width = parseFloat(newValue) || 60;
      break;
    case 'istable-h':
      el.height = parseFloat(newValue) || 50;
      break;
    case 'istable-bordershow':
      el.borderShow = (newValue === 'on');
      break;
    case 'istable-bordercolor':
      el.borderColor = newValue;
      break;
    case 'istable-borderwidth':
      el.borderWidth = parseFloat(newValue) || 1;
      break;

    // Header
    case 'istable-headerBold':
      if (newValue === 'on') {
        if (!el.headerFontSty.includes('B')) el.headerFontSty += 'B';
      } else {
        el.headerFontSty = el.headerFontSty.replace(/B/g, '');
      }
      break;
    case 'istable-headerItalic':
      if (newValue === 'on') {
        if (!el.headerFontSty.includes('I')) el.headerFontSty += 'I';
      } else {
        el.headerFontSty = el.headerFontSty.replace(/I/g, '');
      }
      break;
    case 'istable-headerUnderline':
      if (newValue === 'on') {
        if (!el.headerFontSty.includes('U')) el.headerFontSty += 'U';
      } else {
        el.headerFontSty = el.headerFontSty.replace(/U/g, '');
      }
      break;
    case 'istable-headerfontfam':
      el.headerFontFam = newValue;
      break;
    case 'istable-headerfontsize':
      el.headerFontSize = parseFloat(newValue) || 12;
      break;
    case 'istable-headerfontcolor':
      el.headerFontColor = newValue;
      break;
    case 'istable-headerfill':
      el.headerFill = (newValue === 'on');
      break;
    case 'istable-headerfillcolor':
      el.headerFillColor = newValue;
      break;

    // Data
    case 'istable-dataBold':
      if (newValue === 'on') {
        if (!el.dataFontSty.includes('B')) el.dataFontSty += 'B';
      } else {
        el.dataFontSty = el.dataFontSty.replace(/B/g, '');
      }
      break;
    case 'istable-dataItalic':
      if (newValue === 'on') {
        if (!el.dataFontSty.includes('I')) el.dataFontSty += 'I';
      } else {
        el.dataFontSty = el.dataFontSty.replace(/I/g, '');
      }
      break;
    case 'istable-dataUnderline':
      if (newValue === 'on') {
        if (!el.dataFontSty.includes('U')) el.dataFontSty += 'U';
      } else {
        el.dataFontSty = el.dataFontSty.replace(/U/g, '');
      }
      break;
    case 'istable-datafontfam':
      el.dataFontFam = newValue;
      break;
    case 'istable-datafontsize':
      el.dataFontSize = parseFloat(newValue) || 12;
      break;
    case 'istable-datafontcolor':
      el.dataFontColor = newValue;
      break;
    case 'istable-datafill':
      el.dataFill = newValue;
      break;
    case 'istable-datafillcolor':
      el.dataFillColor = newValue;
      break;
    case 'istable-datafillcolor2':
      el.dataFillColor2 = newValue;
      break;

    // Data Hierarchy / Multicolor
    case 'istable-datahierarchy':
      el.dataHierarchy = (newValue === 'on');
      break;
    case 'istable-datamulticolor':
      el.dataMulticolor = (newValue === 'on');
      break;

    // Income Statement Colors
    case 'istable-colorrevenue':
      el.colorRevenue = newValue;
      break;
    case 'istable-colorexpense':
      el.colorExpense = newValue;
      break;
    case 'istable-colortotals':
      el.colorTotals = newValue;
      break;

    // Hide no-balance rows
    case 'istable-hidenobalance':
      el.hideNoBalance = (newValue === 'on');
      break;

    default:
      console.warn("Unhandled ISTable property:", propId, "value:", newValue);
  }
};

/**
 * Populate the property fields from the current ISTable element.
 */
window.typeFunctions['incomestatement'].populateProperties = function(el) {
  if (!el || el.type !== 'ISTable') return;

  const getById = id => document.getElementById(id);

  const xInput = getById('istable-x');
  if (xInput) xInput.value = el.x || 0;
  const yInput = getById('istable-y');
  if (yInput) yInput.value = el.y || 0;
  const wInput = getById('istable-w');
  if (wInput) wInput.value = el.width || 60;
  const hInput = getById('istable-h');
  if (hInput) hInput.value = el.height || 50;

  const borderShow = getById('istable-bordershow');
  if (borderShow) borderShow.checked = !!el.borderShow;
  const borderColor = getById('istable-bordercolor');
  if (borderColor) borderColor.value = el.borderColor || '#000000';
  const borderWidth = getById('istable-borderwidth');
  if (borderWidth) borderWidth.value = el.borderWidth || 1;

  // Header
  const hfam = getById('istable-headerfontfam');
  if (hfam) hfam.value = el.headerFontFam || 'Courier';
  const hsize = getById('istable-headerfontsize');
  if (hsize) hsize.value = el.headerFontSize || 12;
  const hcolor = getById('istable-headerfontcolor');
  if (hcolor) hcolor.value = el.headerFontColor || '#000000';
  const hfill = getById('istable-headerfill');
  if (hfill) hfill.checked = !!el.headerFill;
  const hfillcolor = getById('istable-headerfillcolor');
  if (hfillcolor) hfillcolor.value = el.headerFillColor || '#cccccc';

  const hBold = getById('istable-headerBold');
  if (hBold) hBold.checked = (el.headerFontSty || '').includes('B');
  const hItal = getById('istable-headerItalic');
  if (hItal) hItal.checked = (el.headerFontSty || '').includes('I');
  const hUndr = getById('istable-headerUnderline');
  if (hUndr) hUndr.checked = (el.headerFontSty || '').includes('U');

  // Data
  const dfam = getById('istable-datafontfam');
  if (dfam) dfam.value = el.dataFontFam || 'Courier';
  const dsize = getById('istable-datafontsize');
  if (dsize) dsize.value = el.dataFontSize || 12;
  const dcolor = getById('istable-datafontcolor');
  if (dcolor) dcolor.value = el.dataFontColor || '#000000';
  const datafill = getById('istable-datafill');
  if (datafill) datafill.value = el.dataFill || 'stripe';
  const datafill1 = getById('istable-datafillcolor');
  if (datafill1) datafill1.value = el.dataFillColor || '#ffffff';
  const datafill2 = getById('istable-datafillcolor2');
  if (datafill2) datafill2.value = el.dataFillColor2 || '#f3f3f3';

  const dBold = getById('istable-dataBold');
  if (dBold) dBold.checked = (el.dataFontSty || '').includes('B');
  const dItal = getById('istable-dataItalic');
  if (dItal) dItal.checked = (el.dataFontSty || '').includes('I');
  const dUndr = getById('istable-dataUnderline');
  if (dUndr) dUndr.checked = (el.dataFontSty || '').includes('U');

  // Data hierarchy, multicolor
  const hier = getById('istable-datahierarchy');
  if (hier) hier.checked = !!el.dataHierarchy;
  const mcol = getById('istable-datamulticolor');
  if (mcol) mcol.checked = !!el.dataMulticolor;

  // Income Statement colors
  const revC = getById('istable-colorrevenue');
  if (revC) revC.value = el.colorRevenue || '#20c997';
  const expC = getById('istable-colorexpense');
  if (expC) expC.value = el.colorExpense || '#dc3545';
  const totC = getById('istable-colortotals');
  if (totC) totC.value = el.colorTotals || '#000000';

  // Hide no-balance rows
  const hideNB = getById('istable-hidenobalance');
  if (hideNB) hideNB.checked = !!el.hideNoBalance;
};

/**
 * Show the type dialog for "incomestatement"
 */
window.typeFunctions['incomestatement'].showTypeDialog = function(dialogObj, dialogCss) {
  if (dialogCss) {
    const styleTag = document.getElementById('typeDialogDynamicCss') || document.createElement('style');
    styleTag.id = 'typeDialogDynamicCss';
    styleTag.innerHTML = dialogCss;
    document.head.appendChild(styleTag);
  }

  const dialogTitle = document.getElementById('type-dialog-title');
  const dialogBody  = document.getElementById('type-dialog-body');
  const modal       = document.getElementById('type-dialog-modal');
  const btnCancel   = document.getElementById('type-dialog-cancel');
  const btnOk       = document.getElementById('type-dialog-ok');

  dialogTitle.textContent = dialogObj.title || 'Parameters';
  dialogBody.innerHTML    = '';

  const fieldElements = {};

  // Build the form
  dialogObj.fields.forEach(field => {
    const labelEl = document.createElement('label');
    labelEl.textContent = field.label || field.name;
    labelEl.style.display = 'block';
    dialogBody.appendChild(labelEl);

    let inputEl;
    if (field.type === 'select') {
      inputEl = document.createElement('select');
      if (field.optionsEndpoint) {
        const url = `documents/type/incomestatement/dialog.php?${field.optionsEndpoint}`;
        fetch(url)
        .then(r => r.json())
        .then(opts => {
          if (Array.isArray(opts)) {
            const defaultOption = document.createElement('option');
            defaultOption.value = '';
            defaultOption.textContent = '-- Use Custom Dates --';
            inputEl.appendChild(defaultOption);
            opts.forEach(opt => {
              const optionNode = document.createElement('option');
              optionNode.value = opt.id;
              optionNode.textContent = opt.name || opt.id;
              inputEl.appendChild(optionNode);
            });
          }
        })
        .catch(err => console.error("Error loading select options:", err));
      }
    } else {
      inputEl = document.createElement('input');
      inputEl.type = field.type || 'text';
    }
    inputEl.style.width = '100%';
    dialogBody.appendChild(inputEl);
    fieldElements[field.name] = inputEl;
  });

  // Show the modal
  modal.style.display = 'flex';

  btnCancel.onclick = () => {
    modal.style.display = 'none';
  };
  btnOk.onclick = () => {
    const formData = new FormData();
    dialogObj.fields.forEach(field => {
      const val = fieldElements[field.name].value;
      formData.append(field.name, val);
    });
    fetch('documents/type/incomestatement/dialog.php', {
      method: 'POST',
      body: formData
    })
    .then(r => r.json())
    .then(resp => {
      if (!resp.success) {
        console.error("Dialog submission failed:", resp.message);
        modal.style.display = 'none';
        return;
      }
      currentLayout.dialogData = resp.data || {};
      modal.style.display = 'none';
      postLayoutToPdf();
    })
    .catch(err => {
      console.error("Error in dialog submission:", err);
      modal.style.display = 'none';
    });
  };
};
