<?php
require_once('../../../../config/config.php');
require_once('../../../../core/financial_periods.php');

/**
 * AJAX endpoint to list available financial periods
 * Called by "optionsEndpoint": "ajax=periods"
 */
if ($_SERVER['REQUEST_METHOD'] === 'GET' && isset($_GET['ajax']) && $_GET['ajax'] === 'periods') {
    header('Content-Type: application/json');
    $financialPeriods = listFinancialPeriods($pdo);
    $output = [];
    foreach ($financialPeriods as $fp) {
        $output[] = [
            'id'         => (string)$fp['id'],
            'name'       => $fp['name'],
            'start_date' => $fp['start_date'],
            'end_date'   => $fp['end_date']
        ];
    }
    echo json_encode($output);
    exit;
}

/**
 * Handle POST: gather period_id, start_date, end_date and respond with JSON.
 */
if ($_SERVER['REQUEST_METHOD'] === 'POST') {
    $fpId      = $_POST['financial_period_id'] ?? '';
    $startDate = $_POST['start_date'] ?? '';
    $endDate   = $_POST['end_date']   ?? '';
    echo json_encode([
        'success' => true,
        'data'    => [
            'financial_period_id' => $fpId,
            'start_date'         => $startDate,
            'end_date'           => $endDate
        ]
    ]);
    exit;
}

// If request is otherwise invalid:
echo json_encode(['success' => false, 'message' => 'Invalid request']);

