window.typeObjectCreators = window.typeObjectCreators || {};

/**
 * VRTable object with extended H1 properties and data fill logic
 */
window.typeObjectCreators['VRTable'] = function(nextId) {
  return {
    id: nextId,
    type: 'VRTable',
    x: 10,
    y: 10,
    width: 80,
    height: 60,

    // H1 properties (for big headings)
    h1FontFam: 'Courier',
    h1FontSty: 'B',   // default bold
    h1FontSize: 16,
    h1FontColor: '#000000',
    h1Fill: true,
    h1FillColor: '#ffffff',

    // Table border
    borderShow: true,
    borderColor: '#000000',
    borderWidth: 1.0,

    // Table header styling
    headerFontFam: 'Courier',
    headerFontSty: '',
    headerFontSize: 12,
    headerFontColor: '#000000',
    headerFill: true,
    headerFillColor: '#cccccc',

    // Data font
    dataFontFam: 'Courier',
    dataFontSty: '',
    dataFontSize: 12,
    dataFontColor: '#000000',

    // Data fill
    dataFill: 'none',
    dataFillColor: '#ffffff',
    dataFillColor2: '#f3f3f3',

    // Summary only => hide transaction listing
    summaryOnly: false
  };
};

/**
 * Custom preview function for VRTable
 */
window.typeObjectCreators['_renderElementPreview'] = window.typeObjectCreators['_renderElementPreview'] || function(el, idx) {
  if (el.type !== 'VRTable') return null;

  const div = document.createElement('div');
  div.className = 'element';
  div.setAttribute('data-index', idx);
  div.style.border  = '1px dashed #aaa';
  div.style.margin  = '0';
  div.style.padding = '0';

  // Position & size
  const leftPx   = (el.x || 0) * 3;
  const topPx    = (el.y || 0) * 3;
  const widthPx  = (el.width  || 80) * 3;
  const heightPx = (el.height || 60) * 3;
  div.style.left   = leftPx + 'px';
  div.style.top    = topPx  + 'px';
  div.style.width  = widthPx + 'px';
  div.style.height = heightPx + 'px';

  // We'll create a small "preview" of the structure
  // 1) "Transaction Listing by VAT Rate" heading
  const heading1 = document.createElement('div');
  heading1.textContent = 'Transaction Listing by VAT Rate';
  heading1.style.fontFamily = el.h1FontFam || 'Courier';
  heading1.style.fontSize   = (el.h1FontSize || 16) + 'px';
  heading1.style.fontWeight = (el.h1FontSty||'').includes('B') ? 'bold' : 'normal';
  heading1.style.fontStyle  = (el.h1FontSty||'').includes('I') ? 'italic' : 'normal';
  heading1.style.textDecoration = (el.h1FontSty||'').includes('U') ? 'underline' : 'none';
  heading1.style.background = el.h1Fill ? (el.h1FillColor || '#ffffff') : 'transparent';
  heading1.style.textAlign  = 'center';
  div.appendChild(heading1);

  // If summaryOnly => skip transaction listing preview
  if (!el.summaryOnly) {
    const table1 = document.createElement('table');
    table1.style.width  = '100%';
    table1.style.borderCollapse = 'collapse';
    // Next row = small header
    const thRow = document.createElement('tr');
    ['Transaction Date','Debit Account','Credit Account','Amount','Description'].forEach(txt=>{
      const th = document.createElement('th');
      th.textContent = txt;
      th.style.fontFamily = el.headerFontFam||'Courier';
      th.style.fontSize   = (el.headerFontSize||12) + 'px';
      if ((el.headerFontSty||'').includes('B')) th.style.fontWeight='bold';
      if ((el.headerFontSty||'').includes('I')) th.style.fontStyle='italic';
      if ((el.headerFontSty||'').includes('U')) th.style.textDecoration='underline';
      thRow.appendChild(th);
    });
    const thead = document.createElement('thead');
    thead.appendChild(thRow);
    table1.appendChild(thead);

    // Body => 2 sample rows
    const tBody1 = document.createElement('tbody');
    const r1 = document.createElement('tr');
    ['2024-11-01','Cost of Sales','Cash/Bank Account','0.00','TEST description'].forEach(txt=>{
      const td = document.createElement('td');
      td.textContent = txt;
      td.style.fontFamily = el.dataFontFam||'Courier';
      td.style.fontSize=(el.dataFontSize||12)+'px';
      r1.appendChild(td);
    });
    const r2 = document.createElement('tr');
    ['2024-11-01','VAT Receivable SUP','Cash/Bank Account','0.00','TEST description (VAT)'].forEach(txt=>{
      const td = document.createElement('td');
      td.textContent = txt;
      td.style.fontFamily = el.dataFontFam||'Courier';
      td.style.fontSize=(el.dataFontSize||12)+'px';
      r2.appendChild(td);
    });
    // apply dataFill logic
    let rowCount=0;
    [r1,r2].forEach(rr=>{
      rowCount++;
      let bg = 'transparent';
      if (el.dataFill==='solid') {
        bg = el.dataFillColor||'#ffffff';
      } else if (el.dataFill==='stripe') {
        bg = (rowCount%2===1)?(el.dataFillColor||'#ffffff'):(el.dataFillColor2||'#f3f3f3');
      }
      rr.style.background=bg;
      tBody1.appendChild(rr);
    });
    table1.appendChild(tBody1);
    div.appendChild(table1);
  }

  // 2) "VAT Report Results" heading
  const heading2 = document.createElement('div');
  heading2.textContent = 'VAT Report Results';
  heading2.style.fontFamily = el.h1FontFam || 'Courier';
  heading2.style.fontSize   = (el.h1FontSize || 16) + 'px';
  heading2.style.fontWeight = (el.h1FontSty||'').includes('B') ? 'bold' : 'normal';
  heading2.style.fontStyle  = (el.h1FontSty||'').includes('I') ? 'italic' : 'normal';
  heading2.style.textDecoration = (el.h1FontSty||'').includes('U') ? 'underline' : 'none';
  heading2.style.background = el.h1Fill ? (el.h1FillColor || '#ffffff') : 'transparent';
  heading2.style.textAlign  = 'center';
  heading2.style.marginTop  = '8px';
  div.appendChild(heading2);

  // Next table for the summary
  const table2 = document.createElement('table');
  table2.style.width='100%';
  table2.style.borderCollapse='collapse';
  const thead2=document.createElement('thead');
  const thr2=document.createElement('tr');
  ['Account Code','Account Name','Account Type','Total Debit','Total Credit','Net Change'].forEach(txt=>{
    const th=document.createElement('th');
    th.textContent=txt;
    th.style.fontFamily=el.headerFontFam||'Courier';
    th.style.fontSize=(el.headerFontSize||12)+'px';
    thr2.appendChild(th);
  });
  thead2.appendChild(thr2);
  table2.appendChild(thead2);

  const tBody2=document.createElement('tbody');
  const r3=document.createElement('tr');
  ['2121','VAT Payable CLIENT','Liability','0.00','0.00','0.00'].forEach(txt=>{
    const td=document.createElement('td');
    td.textContent=txt;
    td.style.fontFamily=el.dataFontFam||'Courier';
    td.style.fontSize=(el.dataFontSize||12)+'px';
    r3.appendChild(td);
  });
  const r4=document.createElement('tr');
  ['2122','VAT Receivable SUP','Liability','0.00','0','0.00'].forEach(txt=>{
    const td=document.createElement('td');
    td.textContent=txt;
    td.style.fontFamily=el.dataFontFam||'Courier';
    td.style.fontSize=(el.dataFontSize||12)+'px';
    r4.appendChild(td);
  });
  tBody2.appendChild(r3);
  tBody2.appendChild(r4);
  table2.appendChild(tBody2);

  // last row => total net change
  const r5=document.createElement('tr');
  const td1=document.createElement('td');
  td1.colSpan=5;
  td1.textContent='Total Net Change:';
  td1.style.fontWeight='bold';
  td1.style.textAlign='right';
  const td2=document.createElement('td');
  td2.textContent='0.00';
  td2.style.fontWeight='bold';
  td2.style.textAlign='right';
  r5.appendChild(td1);
  r5.appendChild(td2);
  tBody2.appendChild(r5);

  div.appendChild(table2);

  // Resizer handle
  const resizeHandle = document.createElement('div');
  resizeHandle.className = 'resize-br-handle';
  div.appendChild(resizeHandle);
  resizeHandle.addEventListener('mousedown', e=>{
    e.stopPropagation();
    e.preventDefault();
    activeElementIndex=idx;
    resizingElementIndex=idx;
    resizingCorner='BR';
    offsetX=e.offsetX;
    offsetY=e.offsetY;
  });

  // Drag
  div.addEventListener('mousedown', e=>{
    activeElementDiv=div;
    isDragging=true;
    offsetX=e.offsetX;
    offsetY=e.offsetY;
    activeElementIndex=idx;
    loadProperties(idx);
    e.preventDefault();
    objectListSel.value=(el.id||'')+'';
  });

  return div;
};

window.typeFunctions = window.typeFunctions || {};
window.typeFunctions['vatreport'] = window.typeFunctions['vatreport'] || {};

/**
 * Update property + populate for VRTable
 */
window.typeFunctions['vatreport'].updateProperty = function(el, propId, newValue) {
  if (!el || el.type !== 'VRTable') return;

  switch(propId){
    // --- H1 properties ---
    case 'vrtable-h1fontfam':
      el.h1FontFam = newValue;
      break;
    case 'vrtable-h1Bold':
      if (newValue==='on') {
        if (!el.h1FontSty.includes('B')) el.h1FontSty+='B';
      } else {
        el.h1FontSty=el.h1FontSty.replace(/B/g,'');
      }
      break;
    case 'vrtable-h1Italic':
      if (newValue==='on') {
        if (!el.h1FontSty.includes('I')) el.h1FontSty+='I';
      } else {
        el.h1FontSty=el.h1FontSty.replace(/I/g,'');
      }
      break;
    case 'vrtable-h1Underline':
      if (newValue==='on') {
        if (!el.h1FontSty.includes('U')) el.h1FontSty+='U';
      } else {
        el.h1FontSty=el.h1FontSty.replace(/U/g,'');
      }
      break;
    case 'vrtable-h1fontsize':
      el.h1FontSize = parseFloat(newValue)||16;
      break;
    case 'vrtable-h1fontcolor':
      el.h1FontColor = newValue;
      break;
    case 'vrtable-h1fill':
      el.h1Fill = (newValue==='on');
      break;
    case 'vrtable-h1fillcolor':
      el.h1FillColor = newValue;
      break;

    // --- border / table settings
    case 'vrtable-bordershow':
      el.borderShow = (newValue==='on');
      break;
    case 'vrtable-bordercolor':
      el.borderColor = newValue;
      break;
    case 'vrtable-borderwidth':
      el.borderWidth = parseFloat(newValue)||1;
      break;

    // --- table header styling
    case 'vrtable-headerBold':
      if (newValue==='on') {
        if (!el.headerFontSty.includes('B')) el.headerFontSty+='B';
      } else {
        el.headerFontSty=el.headerFontSty.replace(/B/g,'');
      }
      break;
    case 'vrtable-headerItalic':
      if (newValue==='on') {
        if (!el.headerFontSty.includes('I')) el.headerFontSty+='I';
      } else {
        el.headerFontSty=el.headerFontSty.replace(/I/g,'');
      }
      break;
    case 'vrtable-headerUnderline':
      if (newValue==='on') {
        if (!el.headerFontSty.includes('U')) el.headerFontSty+='U';
      } else {
        el.headerFontSty=el.headerFontSty.replace(/U/g,'');
      }
      break;
    case 'vrtable-headerfontfam':
      el.headerFontFam = newValue;
      break;
    case 'vrtable-headerfontsize':
      el.headerFontSize = parseFloat(newValue)||12;
      break;
    case 'vrtable-headerfontcolor':
      el.headerFontColor = newValue;
      break;
    case 'vrtable-headerfill':
      el.headerFill = (newValue==='on');
      break;
    case 'vrtable-headerfillcolor':
      el.headerFillColor = newValue;
      break;

    // --- data font
    case 'vrtable-dataBold':
      if (newValue==='on') {
        if (!el.dataFontSty.includes('B')) el.dataFontSty+='B';
      } else {
        el.dataFontSty=el.dataFontSty.replace(/B/g,'');
      }
      break;
    case 'vrtable-dataItalic':
      if (newValue==='on') {
        if (!el.dataFontSty.includes('I')) el.dataFontSty+='I';
      } else {
        el.dataFontSty=el.dataFontSty.replace(/I/g,'');
      }
      break;
    case 'vrtable-dataUnderline':
      if (newValue==='on') {
        if (!el.dataFontSty.includes('U')) el.dataFontSty+='U';
      } else {
        el.dataFontSty=el.dataFontSty.replace(/U/g,'');
      }
      break;
    case 'vrtable-datafontfam':
      el.dataFontFam = newValue;
      break;
    case 'vrtable-datafontsize':
      el.dataFontSize = parseFloat(newValue)||12;
      break;
    case 'vrtable-datafontcolor':
      el.dataFontColor = newValue;
      break;

    // data fill
    case 'vrtable-datafill':
      el.dataFill = newValue;
      break;
    case 'vrtable-datafillcolor':
      el.dataFillColor = newValue;
      break;
    case 'vrtable-datafillcolor2':
      el.dataFillColor2 = newValue;
      break;

    // coords / summaryOnly
    case 'vrtable-x':
      el.x = parseFloat(newValue)||0;
      break;
    case 'vrtable-y':
      el.y = parseFloat(newValue)||0;
      break;
    case 'vrtable-w':
      el.width = parseFloat(newValue)||80;
      break;
    case 'vrtable-h':
      el.height= parseFloat(newValue)||60;
      break;
    case 'vrtable-summaryonly':
      el.summaryOnly = (newValue==='on');
      break;
    default:
      console.warn("[VRTable] Unhandled property:", propId, newValue);
  }
};

window.typeFunctions['vatreport'].populateProperties = function(el){
  if (!el || el.type!=='VRTable') return;

  // --- H1 props
  const h1Fam = document.getElementById('vrtable-h1fontfam');
  if(h1Fam) h1Fam.value = el.h1FontFam || 'Courier';
  const h1Bold = document.getElementById('vrtable-h1Bold');
  if(h1Bold) h1Bold.checked = (el.h1FontSty||'').includes('B');
  const h1Ital = document.getElementById('vrtable-h1Italic');
  if(h1Ital) h1Ital.checked = (el.h1FontSty||'').includes('I');
  const h1Und  = document.getElementById('vrtable-h1Underline');
  if(h1Und)  h1Und.checked = (el.h1FontSty||'').includes('U');
  const h1Size= document.getElementById('vrtable-h1fontsize');
  if(h1Size) h1Size.value = el.h1FontSize||16;
  const h1Colr= document.getElementById('vrtable-h1fontcolor');
  if(h1Colr) h1Colr.value = el.h1FontColor||'#000000';
  const h1F   = document.getElementById('vrtable-h1fill');
  if(h1F)   h1F.checked   = !!el.h1Fill;
  const h1Fcol= document.getElementById('vrtable-h1fillcolor');
  if(h1Fcol) h1Fcol.value = el.h1FillColor||'#ffffff';

  // border
  const bShow = document.getElementById('vrtable-bordershow');
  if(bShow) bShow.checked = !!el.borderShow;
  const bCol = document.getElementById('vrtable-bordercolor');
  if(bCol) bCol.value = el.borderColor||'#000000';
  const bWid = document.getElementById('vrtable-borderwidth');
  if(bWid) bWid.value = el.borderWidth||1;

  // table header
  const hfam = document.getElementById('vrtable-headerfontfam');
  if(hfam) hfam.value = el.headerFontFam||'Courier';
  const hfsz= document.getElementById('vrtable-headerfontsize');
  if(hfsz) hfsz.value = el.headerFontSize||12;
  const hfclr = document.getElementById('vrtable-headerfontcolor');
  if(hfclr) hfclr.value = el.headerFontColor||'#000000';
  const hfll  = document.getElementById('vrtable-headerfill');
  if(hfll)  hfll.checked= !!el.headerFill;
  const hfllc = document.getElementById('vrtable-headerfillcolor');
  if(hfllc) hfllc.value= el.headerFillColor||'#cccccc';
  const hBold = document.getElementById('vrtable-headerBold');
  if(hBold) hBold.checked= (el.headerFontSty||'').includes('B');
  const hItal = document.getElementById('vrtable-headerItalic');
  if(hItal) hItal.checked= (el.headerFontSty||'').includes('I');
  const hUndr = document.getElementById('vrtable-headerUnderline');
  if(hUndr) hUndr.checked=(el.headerFontSty||'').includes('U');

  // data font
  const dFam = document.getElementById('vrtable-datafontfam');
  if(dFam) dFam.value = el.dataFontFam||'Courier';
  const dSize= document.getElementById('vrtable-datafontsize');
  if(dSize) dSize.value = el.dataFontSize||12;
  const dColr= document.getElementById('vrtable-datafontcolor');
  if(dColr) dColr.value= el.dataFontColor||'#000000';
  const dBld= document.getElementById('vrtable-dataBold');
  if(dBld) dBld.checked= (el.dataFontSty||'').includes('B');
  const dItl= document.getElementById('vrtable-dataItalic');
  if(dItl) dItl.checked=(el.dataFontSty||'').includes('I');
  const dUnd= document.getElementById('vrtable-dataUnderline');
  if(dUnd) dUnd.checked=(el.dataFontSty||'').includes('U');

  // data fill
  const dfSel= document.getElementById('vrtable-datafill');
  if(dfSel) dfSel.value = el.dataFill||'none';
  const dfCol= document.getElementById('vrtable-datafillcolor');
  if(dfCol) dfCol.value= el.dataFillColor||'#ffffff';
  const dfCol2= document.getElementById('vrtable-datafillcolor2');
  if(dfCol2) dfCol2.value= el.dataFillColor2||'#f3f3f3';

  // coords
  const xEl = document.getElementById('vrtable-x');
  if(xEl) xEl.value= el.x||0;
  const yEl = document.getElementById('vrtable-y');
  if(yEl) yEl.value= el.y||0;
  const wEl = document.getElementById('vrtable-w');
  if(wEl) wEl.value= el.width||80;
  const hEl = document.getElementById('vrtable-h');
  if(hEl) hEl.value= el.height||60;

  const sOnly = document.getElementById('vrtable-summaryonly');
  if(sOnly) sOnly.checked=!!el.summaryOnly;
};

/**
 * showTypeDialog for vatreport
 */
window.typeFunctions['vatreport'].showTypeDialog = function(dialogObj, dialogCss) {
  // If there's custom dialog CSS, inject it
  if (dialogCss) {
    const styleTag = document.getElementById('typeDialogDynamicCss') || document.createElement('style');
    styleTag.id = 'typeDialogDynamicCss';
    styleTag.innerHTML = dialogCss;
    document.head.appendChild(styleTag);
  }

  const dialogTitle = document.getElementById('type-dialog-title');
  const dialogBody  = document.getElementById('type-dialog-body');
  const modal       = document.getElementById('type-dialog-modal');
  const btnCancel   = document.getElementById('type-dialog-cancel');
  const btnOk       = document.getElementById('type-dialog-ok');

  dialogTitle.textContent = dialogObj.title || 'Parameters';
  dialogBody.innerHTML    = '';

  const fieldElements = {};
  dialogObj.fields.forEach(field => {
    const labelEl = document.createElement('label');
    labelEl.textContent = field.label || field.name;
    labelEl.style.display = 'block';
    dialogBody.appendChild(labelEl);

    let inputEl;
    if (field.type === 'select') {
      inputEl = document.createElement('select');
      if (field.optionsEndpoint) {
        // e.g. "ajax=periods" => fetch from dialog.php
        const url = `documents/type/vatreport/dialog.php?${field.optionsEndpoint}`;
        fetch(url)
          .then(r => r.json())
          .then(opts => {
            if (Array.isArray(opts)) {
              const defaultOption = document.createElement('option');
              defaultOption.value = '';
              defaultOption.textContent = '-- Custom Date Range --';
              inputEl.appendChild(defaultOption);

              opts.forEach(opt => {
                const optionNode = document.createElement('option');
                optionNode.value = opt.id;
                optionNode.textContent = opt.name || opt.id;
                inputEl.appendChild(optionNode);
              });
            }
          })
          .catch(err => console.error("Error loading select options:", err));
      }
    } else {
      inputEl = document.createElement('input');
      inputEl.type = field.type || 'text';
    }
    inputEl.style.width = '100%';
    dialogBody.appendChild(inputEl);
    fieldElements[field.name] = inputEl;
  });

  // Show modal
  modal.style.display = 'flex';

  // Cancel
  btnCancel.onclick = () => {
    modal.style.display = 'none';
  };

  // OK => post form => store => PDF
  btnOk.onclick = () => {
    const formData = new FormData();
    dialogObj.fields.forEach(field => {
      const val = fieldElements[field.name].value;
      formData.append(field.name, val);
    });

    fetch('documents/type/vatreport/dialog.php', {
      method: 'POST',
      body: formData
    })
    .then(r => r.json())
    .then(resp => {
      if (!resp.success) {
        console.error("Dialog submission failed:", resp.message);
        modal.style.display = 'none';
        return;
      }
      // Store user input in layout's dialogData
      currentLayout.dialogData = resp.data || {};
      modal.style.display = 'none';
      // Generate PDF
      postLayoutToPdf();
    })
    .catch(err => {
      console.error("Error in dialog submission:", err);
      modal.style.display = 'none';
    });
  };
};
