<?php
ini_set('display_errors', 1);
ini_set('display_startup_errors', 1);
error_reporting(E_ALL);
require_once('../core/accounts.php');
require_once('components/auth.php');
require_once('components/header.php');
require_once('components/form_component.php');
$errors = [];

// Add role check for 'accounts_edit'
if (!checkRole(['accounts_edit'])) {
    $errors[] = $_SESSION['error_message'];
    unset($_SESSION['error_message']);
    ?>
    <h1>Edit Account</h1>
    <?php if (!empty($errors)): ?>
    <div class="alert">
        <?php foreach ($errors as $error): ?>
            <p><?php echo htmlspecialchars($error); ?></p>
        <?php endforeach; ?>
    </div>
    <?php endif; ?>
    <?php require_once('components/footer.php'); ?>
    <?php
    exit;
}

if (!isset($_GET['id']) || empty(trim($_GET['id']))) {
    $_SESSION['error_message'] = "Invalid account ID.";
    header('Location: accounts.php');
    exit;
}
$accountId = trim($_GET['id']);
$account = getAccount($pdo, $accountId);
if (!$account) {
    $_SESSION['error_message'] = "Account not found.";
    header('Location: accounts.php');
    exit;
}
$currentCode = $account['code'];
$currentName = $account['name'];
$currentDescription = $account['description'];
$currentAccountType = $account['account_type'];
$currentParent = $account['parent'];
$accountTypes = [
    'asset' => 'Asset',
    'liability' => 'Liability',
    'equity' => 'Equity',
    'income' => 'Income',
    'expense' => 'Expense'
];
$parentAccounts = ['' => 'None'];
$accounts = listAccounts($pdo);
foreach ($accounts as $acct) {
    if ($acct['code'] !== $currentCode) {
        $parentAccounts[$acct['code']] = $acct['name'] . ' (' . $acct['code'] . ')';
    }
}
if ($_SERVER['REQUEST_METHOD'] === 'POST') {
    $code = trim($_POST['code']);
    $name = trim($_POST['name']);
    $description = trim($_POST['description']);
    if (empty($code) || empty($name)) {
        $errors[] = 'Code, Name, and Account Type are required.';
    }
    if (empty($errors)) {
        $result = updateAccount($pdo, $code, $name, $description, $_SESSION['user_id']);
        if ($result) {
            $_SESSION['success_message'] = "Account [$code] has been successfully updated!";
            header('Location: accounts.php');
            exit;
        } else {
            $errors[] = 'Failed to update account. Please try again.';
        }
    }
    $currentCode = $code;
    $currentName = $name;
    $currentDescription = $description;
    $currentAccountType = $account_type;
    $currentParent = $parent;
}
?>
<h1>Edit Account</h1>
<?php if (!empty($errors)): ?>
<div class="alert">
    <?php foreach ($errors as $error): ?>
        <p><?php echo htmlspecialchars($error); ?></p>
    <?php endforeach; ?>
</div>
<?php endif; ?>
<div class="form-container">
    <form action="edit_account.php?id=<?php echo urlencode($accountId); ?>" method="POST">
        <?php
        renderInput('text', 'code', 'Code', htmlspecialchars($currentCode), true, ['disabled' => 'disabled']);
        echo "<input type=\"hidden\" name=\"code\" value=\"" .  htmlspecialchars($currentCode) . "\">";
        renderInput('text', 'name', 'Name', htmlspecialchars($currentName), true, ['disabled' => 'disabled']);
        echo "<input type=\"hidden\" name=\"name\" value=\"" . htmlspecialchars($currentName) . "\">";
        renderTextarea('description', 'Description', htmlspecialchars($currentDescription));
        renderSelect('account_type', 'Account Type', $accountTypes, $currentAccountType, true, ['disabled' => 'disabled']);
        renderSelect('parent', 'Parent Account', $parentAccounts, $currentParent, false, ['disabled' => 'disabled']);
        ?>
        <div class="button-group">
            <button type="submit">Save Changes</button>
            <a href="accounts.php" class="button">Back to Manage Accounts</a>
        </div>
    </form>
</div>
<?php require_once('components/footer.php'); ?>
