<?php
require_once('../core/error_handling.php');
require_once('components/auth.php');
require_once('components/header.php');
$errors = [];
$success = '';

if (!checkRole(['error_handling'])) {
    $errors[] = $_SESSION['error_message'];
    unset($_SESSION['error_message']);
    ?>
    <h1>Error Handling</h1>
    <?php if (!empty($errors)): ?>
    <div class="alert">
        <?php foreach ($errors as $error): ?>
        <p><?php echo htmlspecialchars($error); ?></p>
        <?php endforeach; ?>
    </div>
    <?php endif; ?>
    <?php require_once('components/footer.php'); ?>
    <?php
    exit;
}

// Handle delete error action
if ($_SERVER['REQUEST_METHOD'] === 'POST') {
    if (isset($_POST['delete_error_id']) && hasRole('error_handling_delete')) {
        $deleteErrorId = (int)$_POST['delete_error_id'];
        if ($deleteErrorId > 0) {
            $result = deleteError($pdo, $deleteErrorId);
            if ($result) {
                $success = 'Error has been deleted successfully.';
            } else {
                $errors[] = 'Failed to delete error.';
            }
        }
    }
}

// Get list of errors
$errorList = listErrors($pdo);

?>
<h1>Error Handling</h1>
<?php if (!empty($errors)): ?>
<div class="alert">
    <?php foreach ($errors as $error): ?>
    <p><?php echo htmlspecialchars($error); ?></p>
    <?php endforeach; ?>
</div>
<?php endif; ?>
<?php if ($success): ?>
<div class="success">
    <p><?php echo htmlspecialchars($success); ?></p>
</div>
<?php endif; ?>
<table>
    <thead>
        <tr>
            <th>ID</th>
            <th>Message</th>
            <th>Error Level</th>
            <th>Created At</th>
            <?php if (hasRole('error_handling_log') || hasRole('error_handling_delete')): ?>
            <th>Actions</th>
            <?php endif; ?>
        </tr>
    </thead>
    <tbody>
        <?php foreach ($errorList as $errorItem): ?>
        <tr>
            <td><?php echo htmlspecialchars($errorItem['id']); ?></td>
            <td><?php echo htmlspecialchars($errorItem['message']); ?></td>
            <td><?php echo htmlspecialchars($errorItem['error_level']); ?></td>
            <td><?php echo htmlspecialchars($errorItem['created_at']); ?></td>
            <?php if (hasRole('error_handling_log') || hasRole('error_handling_delete')): ?>
            <td>
                <?php if (hasRole('error_handling_log')): ?>
                <a href="log_error_handling.php?id=<?php echo $errorItem['id']; ?>" class="button">View Details</a>
                <?php endif; ?>
                <?php if (hasRole('error_handling_delete')): ?>
                <form action="error_handling.php" method="POST" style="display:inline;">
                    <input type="hidden" name="delete_error_id" value="<?php echo $errorItem['id']; ?>">
                    <button type="submit" class="button delete-button" onclick="return confirm('Are you sure you want to delete this error?');">Delete</button>
                </form>
                <?php endif; ?>
            </td>
            <?php endif; ?>
        </tr>
        <?php endforeach; ?>
    </tbody>
</table>
<?php require_once('components/footer.php'); ?>

