<?php
ini_set('display_errors', 1);
ini_set('display_startup_errors', 1);
error_reporting(E_ALL);
require_once('../core/accounts.php');
require_once('components/auth.php');
require_once('components/header.php');
require_once('components/form_component.php');
$errors = [];
$importResults = null;

// Add role check for 'accounts_import'
if (!checkRole(['accounts_import'])) {
    $errors[] = $_SESSION['error_message'];
    unset($_SESSION['error_message']);
    ?>
    <h1>Import Accounts</h1>
    <?php if (!empty($errors)): ?>
    <div class="alert">
        <?php foreach ($errors as $error): ?>
            <p><?php echo htmlspecialchars($error); ?></p>
        <?php endforeach; ?>
    </div>
    <?php endif; ?>
    <?php require_once('components/footer.php'); ?>
    <?php
    exit;
}

if ($_SERVER['REQUEST_METHOD'] === 'POST') {
    if (isset($_FILES['accounts_csv']) && $_FILES['accounts_csv']['error'] === UPLOAD_ERR_OK) {
        $fileTmpPath = $_FILES['accounts_csv']['tmp_name'];
        $fileName = $_FILES['accounts_csv']['name'];
        $fileSize = $_FILES['accounts_csv']['size'];
        $fileType = $_FILES['accounts_csv']['type'];
        $fileNameCmps = explode(".", $fileName);
        $fileExtension = strtolower(end($fileNameCmps));
        if ($fileExtension !== 'csv') {
            $errors[] = "Invalid file type. Please upload a CSV file.";
        } else {
            if (($handle = fopen($fileTmpPath, 'r')) !== FALSE) {
                $header = fgetcsv($handle, 1000, ",");
                $expectedHeaders = ['code', 'name', 'description', 'account_type', 'parent'];
                $header = array_map('strtolower', $header);
                if ($header !== $expectedHeaders) {
                    $errors[] = "Invalid CSV format. Expected headers: " . implode(",", $expectedHeaders);
                } else {
                    $imported = 0;
                    $skipped = 0;
                    while (($row = fgetcsv($handle, 1000, ",")) !== FALSE) {
                        $accountData = array_combine($expectedHeaders, $row);
                        $accountData = array_map('trim', $accountData);
                        if (empty($accountData['code']) || empty($accountData['name']) || empty($accountData['account_type'])) {
                            $skipped++;
                            continue;
                        }
                        $existingAccount = getAccount($pdo, $accountData['code']);
                        if ($existingAccount) {
                            $skipped++;
                            continue;
                        }
                        $validAccountTypes = ['asset', 'liability', 'equity', 'income', 'expense'];
                        if (!in_array(strtolower($accountData['account_type']), $validAccountTypes)) {
                            $skipped++;
                            continue;
                        }
                        $parent = !empty($accountData['parent']) && strtolower($accountData['parent']) !== 'n/a' ? $accountData['parent'] : null;
                        if ($parent !== null) {
                            $parentAccount = getAccount($pdo, $parent);
                            if (!$parentAccount) {
                                $skipped++;
                                continue;
                            }
                        }
                        $created = createAccount($pdo, $accountData['code'], $accountData['name'], $accountData['account_type'],$_SESSION['user_id'], $accountData['description'], $parent, 0);
                        if ($created) {
                            $imported++;
                        } else {
                            $skipped++;
                        }
                    }
                    fclose($handle);
                    $_SESSION['import_message'] = "Import completed. $imported accounts were imported.\n$skipped accounts were skipped.";
                    header('Location: accounts.php');
                    exit;
                }
            } else {
                $errors[] = "Failed to open the uploaded file.";
            }
        }
    } else {
        $errors[] = "No file uploaded or there was an upload error.";
    }
}
?>
<h1>Import Accounts</h1>
<?php if (!empty($errors)): ?>
<div class="alert">
    <?php foreach ($errors as $error): ?>
        <p><?php echo htmlspecialchars($error); ?></p>
    <?php endforeach; ?>
</div>
<?php endif; ?>
<div class="form-container">
    <form action="import_accounts.php" method="POST" enctype="multipart/form-data">
        <label for="accounts_csv">Select CSV File:</label>
        <input type="file" id="accounts_csv" name="accounts_csv" accept=".csv" required>
        <div class="button-group">
            <button type="submit">Import Accounts</button>
            <a href="accounts.php" class="button">Back to Manage Accounts</a>
        </div>
    </form>
</div>
<?php require_once('components/footer.php'); ?>
