<?php
// public/index.php
require_once('../config/config.php');
if (!file_exists(DB_FILE)) {
    header('Location: install.php');
    exit;
}
require_once('components/auth.php');
require_once('../core/financial_reports.php');

// Read local_app_id
$localAppIdFile = __DIR__ . '/../data/local_app_id.txt';
$localAppId     = 'NOT_SET';
if (file_exists($localAppIdFile)) {
    $localAppId = trim(file_get_contents($localAppIdFile));
}

// LICENSE CHECK
$licenseRow = $pdo->query("SELECT license_type, unique_registration_number, start_date, end_date, registration_code
                           FROM license_info LIMIT 1")->fetch(PDO::FETCH_ASSOC);
$showLicenseModal = false;
if ($licenseRow && strtolower($licenseRow['license_type']) === 'unregistered') {
    $showLicenseModal = true;
} else {
    $type = strtolower($licenseRow['license_type'] ?? 'unregistered');
    if ($type !== 'unregistered') {
        // Check if expired
        $end = $licenseRow['end_date'] ?? null;
        if ($end && $end < date('Y-m-d')) {
            $licenseRow['license_type'] = 'Expired';
            $showLicenseModal = true;
        }
    }
}

// WIDGETS Logic
if (isset($_GET['action']) && $_GET['action'] === 'list_widgets') {
    header('Content-Type: application/json; charset=utf-8');
    $widgetDir = __DIR__ . '/dashboard';
    if (!is_dir($widgetDir)) {
        echo json_encode(['success' => false, 'message' => 'No widget folder found.']);
        exit;
    }
    $files = scandir($widgetDir);
    $widgetFiles = [];
    foreach ($files as $f) {
        if ($f === '.' || $f === '..') continue;
        if (strtolower(pathinfo($f, PATHINFO_EXTENSION)) === 'php') {
            $widgetFiles[] = $f;
        }
    }
    if (empty($widgetFiles)) {
        echo json_encode(['success' => false, 'message' => 'No widgets found.']);
        exit;
    }
    echo json_encode(['success' => true, 'widgets' => $widgetFiles]);
    exit;
}
if (isset($_GET['action']) && $_GET['action'] === 'load_widget') {
    $file = $_GET['file'] ?? '';
    $widgetPath = __DIR__ . '/dashboard/' . basename($file);
    if (!file_exists($widgetPath)) {
        echo "Widget file not found.";
        exit;
    }
    readfile($widgetPath);
    exit;
}

require_once('components/header.php');
?>
<h1>Dashboard</h1>

<!-- Add Widget Button at top-right -->
<div style="float: right; margin-top: -40px; margin-right: 10px;">
    <button id="addWidgetBtn" class="button">Add Widget</button>
</div>

<!-- Container for dynamically loaded widgets -->
<div id="widgetsContainer" style="position: relative; min-height: 200px; margin-top: 30px;">
    <!-- Widget DIVs will be appended here via JS -->
</div>

<?php // Show license modal if needed ?>
<?php if ($showLicenseModal): ?>
    <div id="licenseModalOverlay" style="
        position: fixed;
        top: 0; left: 0;
        width: 100%; height: 100%;
        background: rgba(0,0,0,0.8);
        z-index: 99999;
        display: flex;
        justify-content: center;
        align-items: center;
    ">
        <div style="
            background: #fff;
            max-width: 600px;
            padding: 20px;
            border-radius: 4px;
            position: relative;
        ">
            <h2>License Agreement</h2>
            <p>Your software is currently <strong><?php echo $licenseRow['license_type'] ?></strong>.</p>
            <p>
              Please register on
              <a href="https://donnotec.com/activation.php" target="_blank">
                donnotec.com
              </a>
              to activate.
            </p>
            <div id="licenseTextContainer" style="margin-top:20px;overflow:auto;height:200px;border:1px solid black;">
<h1>DONNOTEC Double Entry Accounting System License</h1>
  <p><strong>Version 1.0</strong></p>
  <p><strong>Effective Date:</strong> (Date of Publication)</p>

  <h2>1. Definitions</h2>
  <p>
    1.1 <strong>“Software”</strong> refers to the DONNOTEC Double Entry Accounting System,
    including its source code, associated files, documentation, or portions thereof,
    made available under this License.
  </p>
  <p>
    1.2 <strong>“Non-Commercial Use”</strong> means personal, educational,
    academic research, nonprofit, or other use where <em>no direct or indirect
    commercial advantage</em> is gained. Examples include:
  </p>
  <ul>
    <li>Personal record-keeping for oneself</li>
    <li>Nonprofit or charitable organization usage</li>
    <li>Use in academic institutions for teaching or research</li>
  </ul>
  <p>
    1.3 <strong>“Commercial Use”</strong> means any use of the Software (or a
    modified version thereof) in exchange for monetary or other compensation, or
    for the commercial advantage or financial benefit of the user or any third
    party. This includes, but is not limited to, use by for-profit businesses,
    consultants, or service providers.
  </p>
  <p>
    1.4 <strong>“You” (or “Your”)</strong> means the individual or legal entity exercising
    permissions granted by this License.
  </p>

  <h2>2. Grant of License</h2>
  <p>
    2.1 <strong>Non-Commercial License Grant.</strong> Subject to the terms and conditions
    of this License, You are granted a worldwide, non-exclusive, royalty-free license
    to <em>use, copy, modify, merge, publish, and distribute</em> the Software (in part or
    in whole) for <strong>Non-Commercial Use</strong> only. You must retain a copy of this
    License in all distributions and make clear references to the Software’s name
    (“DONNOTEC Double Entry Accounting System”) and this License.
  </p>
  <p>
    2.2 <strong>Commercial License Requirement.</strong> Any Commercial Use of the Software
    requires obtaining a <strong>yearly paid license key</strong> from the original licensor
    of the Software (“DONNOTEC” or an authorized provider). Without a valid commercial
    license key, any Commercial Use is <strong>strictly prohibited</strong>.
  </p>

  <h2>3. Modifications and Contributions</h2>
  <p>
    3.1 <strong>Right to Modify.</strong> You may create modifications or derivative
    works based on the Software, and You may use, copy, or distribute such
    modifications for <strong>Non-Commercial Use</strong>, provided they remain under
    the same terms of this License.
  </p>
  <p>
    3.2 <strong>Distribution of Modifications.</strong> When You distribute or publish
    any derivative work:
  </p>
  <ul>
    <li>You must retain notice of this License.</li>
    <li>You must clearly inform recipients that modified portions are Your own contribution
        and are not part of the official Software.</li>
  </ul>

  <h2>4. License Key for Commercial Use</h2>
  <p>
    4.1 <strong>Obtaining a Commercial License Key.</strong> Organizations or individuals
    intending to use the Software for commercial purposes must purchase a
    <strong>valid license key</strong> from DONNOTEC or an authorized reseller.
    Instructions for obtaining said license key are provided at
    <a href="https://donnotec.com" target="_blank" rel="noopener">https://donnotec.com</a>.
  </p>
  <p>
    4.2 <strong>Term and Renewal.</strong> Commercial license keys are generally valid
    for one year unless otherwise specified. Continued commercial usage beyond the
    license term requires renewal of the license key.
  </p>

  <h2>5. Disclaimer of Warranty</h2>
  <p>
    THE SOFTWARE IS PROVIDED “AS IS,” WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
    INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
    PARTICULAR PURPOSE, AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
    HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES, OR OTHER LIABILITY, WHETHER IN AN ACTION
    OF CONTRACT, TORT, OR OTHERWISE, ARISING FROM, OUT OF, OR IN CONNECTION WITH THE
    SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
  </p>

  <h2>6. Limitation of Liability</h2>
  <p>
    TO THE MAXIMUM EXTENT PERMITTED BY APPLICABLE LAW, IN NO EVENT SHALL DONNOTEC OR
    ITS CONTRIBUTORS BE LIABLE FOR ANY SPECIAL, INCIDENTAL, INDIRECT, OR CONSEQUENTIAL
    DAMAGES WHATSOEVER ARISING OUT OF OR IN CONNECTION WITH THE USE OF THE SOFTWARE
    UNDER ANY THEORY OF LIABILITY, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
  </p>

  <h2>7. Termination</h2>
  <p>
    7.1 <strong>Termination of Rights.</strong> If You fail to comply with any term(s) of
    this License, Your rights under this License terminate automatically, and You
    must cease all further use, copying, modification, and distribution of the
    Software.
  </p>
  <p>
    7.2 <strong>Survival.</strong> License provisions regarding Disclaimer of Warranty,
    Limitation of Liability, and any other terms intended to survive shall remain
    in effect after termination.
  </p>

  <h2>8. General Provisions</h2>
  <p>
    8.1 <strong>No Endorsement.</strong> Use of DONNOTEC’s name, trademarks, or service
    marks in any advertising or publicity relating to the Software is prohibited
    without specific, written prior permission.
  </p>
  <p>
    8.2 <strong>Entire Agreement.</strong> This License constitutes the entire agreement
    between You and DONNOTEC concerning the Software and supersedes any prior
    agreements or understandings.
  </p>
  <p>
    8.3 <strong>Severability.</strong> If any provision of this License is held to be
    unenforceable, such provision shall be reformed only to the extent necessary to
    make it enforceable, and the remaining provisions of this License remain in full
    force and effect.
  </p>
  <p>
    8.4 <strong>Governing Law.</strong> This License shall be governed by and interpreted
    in accordance with the laws of the jurisdiction in which DONNOTEC primarily
    conducts business, excluding its conflict-of-law provisions.
  </p>

  <p>
    By using the DONNOTEC Double Entry Accounting System in any way, You acknowledge
    that You have read, understood, and agree to be bound by the terms of this
    License. If You do not agree, You must not use, copy, modify, or distribute the
    Software.
  </p>
            </div>
            <div style="text-align: right;margin-top:20px;">
                <button id="closeUnregisteredModalBtn" disabled>I Agree / Continue</button>
            </div>
        </div>
    </div>
    <script>
    document.addEventListener('DOMContentLoaded', function() {
        var closeBtn = document.getElementById('closeUnregisteredModalBtn');
        var textContainer = document.getElementById('licenseTextContainer');
        function enableButtonIfScrolledToEnd() {
          if (textContainer.scrollTop + textContainer.clientHeight >= textContainer.scrollHeight) {
            closeBtn.disabled = false;
          }
        }
        textContainer.addEventListener('scroll', enableButtonIfScrolledToEnd);
        enableButtonIfScrolledToEnd();

        closeBtn.addEventListener('click', function() {
            document.getElementById('licenseModalOverlay').style.display = 'none';
        });
    });
    </script>
<?php endif; ?>

<!-- Modal to choose widget file -->
<div id="widgetModalOverlay" style="
    display: none;
    position: fixed;
    top: 0; left: 0;
    width: 100%; height: 100%;
    background: rgba(0,0,0,0.5);
    z-index: 9999;
    align-items: center;
    justify-content: center;
">
  <div style="background: #fff; padding: 20px; border-radius: 5px; min-width: 300px;">
    <h2>Select a Widget</h2>
    <div id="widgetModalError" style="color: red; font-weight: bold; display:none;"></div>
    <select id="widgetSelect" style="width: 100%; margin-bottom: 15px;"></select>
    <div style="text-align: right;">
      <button id="widgetCancelBtn" class="button" style="margin-right:10px;">Cancel</button>
      <button id="widgetOkBtn" class="button">OK</button>
    </div>
  </div>
</div>

<script src="https://code.jquery.com/ui/1.13.2/jquery-ui.min.js"></script>
<style>

    #closeUnregisteredModalBtn:disabled,
    #closeUnregisteredModalBtn[disabled]{
        border: 1px solid #999999;
        background-color: #cccccc;
        color: #666666;
    }

</style>
<script>
document.addEventListener('DOMContentLoaded', function() {
    const addWidgetBtn  = document.getElementById('addWidgetBtn');
    const widgetModal   = document.getElementById('widgetModalOverlay');
    const widgetSelect  = document.getElementById('widgetSelect');
    const widgetCancel  = document.getElementById('widgetCancelBtn');
    const widgetOk      = document.getElementById('widgetOkBtn');
    const widgetErr     = document.getElementById('widgetModalError');
    const widgetsContainer = document.getElementById('widgetsContainer');

    // Show widget selection
    addWidgetBtn.addEventListener('click', function() {
        widgetSelect.innerHTML = '';
        widgetErr.style.display = 'none';
        fetch('index.php?action=list_widgets')
          .then(r => r.json())
          .then(data => {
            if (!data.success) {
                widgetErr.textContent = data.message || 'Error listing widgets.';
                widgetErr.style.display = 'block';
                return;
            }
            const files = data.widgets || [];
            files.forEach(f => {
                const opt = document.createElement('option');
                opt.value = f;
                opt.textContent = f;
                widgetSelect.appendChild(opt);
            });
            widgetModal.style.display = 'flex';
          })
          .catch(err => {
            console.error(err);
            widgetErr.textContent = 'Error fetching widget list.';
            widgetErr.style.display = 'block';
          });
    });

    // Cancel
    widgetCancel.onclick = () => {
        widgetModal.style.display = 'none';
    };

    // OK -> load chosen widget
    widgetOk.onclick = () => {
        const chosen = widgetSelect.value;
        if (!chosen) {
            widgetErr.textContent = 'Please select a widget.';
            widgetErr.style.display = 'block';
            return;
        }
        widgetModal.style.display = 'none';
        fetch('index.php?action=load_widget&file=' + encodeURIComponent(chosen))
          .then(r => r.text())
          .then(htmlContent => {
            const widgetDiv = document.createElement('div');
            widgetDiv.innerHTML = htmlContent;
            widgetDiv.classList.add('dashboard-widget');
            widgetDiv.style.position = 'absolute';
            widgetDiv.style.top = '0px';
            widgetDiv.style.left = '0px';
            widgetDiv.style.border = '1px solid #ccc';
            widgetDiv.style.padding = '10px';
            widgetDiv.style.borderRadius = '5px';
            widgetDiv.style.background = '#fafafa';
            widgetDiv.style.cursor = 'move';
            widgetDiv.style.zIndex = '1';
            widgetsContainer.appendChild(widgetDiv);

            $(widgetDiv).draggable({
                containment: "#widgetsContainer",
                start: function(e, ui) {
                    $('.dashboard-widget').css('z-index','1');
                    $(this).css('z-index','999');
                }
            }).on('click', function(){
                $('.dashboard-widget').css('z-index','1');
                $(this).css('z-index','999');
            });
          })
          .catch(err => {
            console.error(err);
            alert('Failed to load the widget content.');
          });
    };
});
</script>

<?php require_once('components/footer.php'); ?>
