<?php
// public/install.php
// Basic Installation Script

ini_set('display_errors', 1);
ini_set('display_startup_errors', 1);
error_reporting(E_ALL);

require_once(__DIR__ . '/../config/config.php');
require_once(__DIR__ . '/../core/initialization.php'); // after changes

// 1) If DB file already exists, skip
if (file_exists(__DIR__ . '/../data/accounting_system.sqlite')) {
    echo "<h2>Database already exists. Installation not needed.</h2>";
    exit;
}

// Process form
$errors = [];
$successMsg = '';

if ($_SERVER['REQUEST_METHOD'] === 'POST' && isset($_POST['install_submit'])) {
    $masterUser     = trim($_POST['master_username'] ?? '');
    $masterPass     = trim($_POST['master_password'] ?? '');
    $masterPassConf = trim($_POST['master_password_conf'] ?? '');
    $masterEmail    = trim($_POST['master_email'] ?? '');

    if (!$masterUser || !$masterPass || !$masterPassConf || !$masterEmail) {
        $errors[] = "All fields are required.";
    } elseif ($masterPass !== $masterPassConf) {
        $errors[] = "Passwords do not match.";
    } else {
        // 2) Now create the DB schema with the user credentials
        //    License info is set to unregistered by default in initialization.php
        if (initializeDatabase($masterUser, $masterPass, $masterEmail)) {
            $successMsg = "Installation completed!<br>License: unregistered";
        } else {
            $errors[] = "Failed to initialize database. Check logs or permissions.";
        }
    }
}
?>
<!DOCTYPE html>
<html>
<head>
    <meta charset="utf-8">
    <title>DONNOTEC Installation</title>
</head>
<body>
<h1>DONNOTEC Installation</h1>
<?php if (!empty($errors)): ?>
    <div style="color:red;">
    <ul>
    <?php foreach($errors as $e) echo "<li>".htmlspecialchars($e)."</li>"; ?>
    </ul>
    </div>
<?php endif; ?>

<?php if (!empty($successMsg)): ?>
    <div style="color:green;"><?php echo $successMsg; ?></div>
    <p><a href="index.php">Go to Dashboard</a></p>
<?php else: ?>
    <p>Welcome! Please configure your (admin) user:</p>
    <form method="POST" action="install.php">

        <label for="master_username">Administrator Username:</label>
        <input type="text" name="master_username" id="master_username" required>
        <br><br>

        <label for="master_password">Administrator Password:</label>
        <input type="password" name="master_password" id="master_password" required>
        <button type="button" onclick="togglePasswordVisibility('master_password', this)">Show</button>
        <br><br>

        <label for="master_password_conf">Confirm Password:</label>
        <input type="password" name="master_password_conf" id="master_password_conf" required>
        <button type="button" onclick="togglePasswordVisibility('master_password_conf', this)">Show</button>
        <br><br>

        <label for="master_email">Administrator Email:</label>
        <input type="email" name="master_email" id="master_email" required>
        <br><br>

        <button type="submit" name="install_submit" value="1">Install</button>
    </form>

    <script>
    function togglePasswordVisibility(fieldId, btn) {
        const field = document.getElementById(fieldId);
        if (field.type === 'password') {
            field.type = 'text';
            btn.textContent = 'Hide';
        } else {
            field.type = 'password';
            btn.textContent = 'Show';
        }
    }
    </script>
<?php endif; ?>
</body>
</html>
