<?php
// public/license.php
require_once('components/auth.php');
require_once('../core/error_handling.php');
require_once('../core/users.php');
require_once('components/header.php');

/**
 * Decrypt function for the activation code.
 * If you have more advanced logic, adjust here.
 */
function decryptLicenseData($encryptedString) {
    $decoded = base64_decode($encryptedString);
    return $decoded; // Basic approach
}

$errors = [];
$success = '';

// Offline activation
if ($_SERVER['REQUEST_METHOD'] === 'POST' && isset($_POST['activation_code'])) {
    $activationCode = trim($_POST['activation_code']);
    if ($activationCode === '') {
        $errors[] = "Please provide a valid activation code.";
    } else {
        $decoded = decryptLicenseData($activationCode);
        $parts   = explode(',', $decoded);
        if (count($parts) < 6) {
            $errors[] = "Invalid license.";
        } else {
            $regNumber   = $parts[0];
            $licenseType = $parts[1];
            $nameCompany = $parts[2];
            $email       = $parts[3];
            $payDate     = $parts[4];
            $endDate     = $parts[5];

            if (strpos($endDate, '|') !== false) {
                $tmpArr  = explode('|', $endDate, 2);
                $endDate = $tmpArr[0];
            }

            // Check if row with id=1 exists
            $stmtSel = $pdo->prepare("SELECT * FROM license_info WHERE id=1");
            $stmtSel->execute();
            $row = $stmtSel->fetch(PDO::FETCH_ASSOC);

            if (!$row) {
                // Insert new row with id=1
                $ins = $pdo->prepare("
                    INSERT INTO license_info (
                        id,
                        unique_registration_number,
                        license_type,
                        name_or_company,
                        email,
                        pay_date,
                        end_date,
                        start_date,
                        registration_code
                    ) VALUES (1,?,?,?,?,?,?,?,?)
                ");
                $ins->execute([
                    $regNumber,
                    $licenseType,
                    $nameCompany,
                    $email,
                    $payDate ?: null,
                    $endDate ?: null,
                    // We'll set start_date to payDate or null here
                    $payDate ?: null,
                    $activationCode
                ]);
                $success = "License Activated Successfully.";
            } else {
                // Try an UPDATE first
                $update = $pdo->prepare("
                    UPDATE license_info
                    SET unique_registration_number = :urn,
                        license_type = :lictype,
                        name_or_company = :nc,
                        email = :em,
                        pay_date = :pd,
                        end_date = :ed,
                        start_date = :sd,
                        registration_code = :rc
                    WHERE id = 1
                ");
                $update->execute([
                    ':urn'     => $regNumber,
                    ':lictype' => $licenseType,
                    ':nc'      => $nameCompany,
                    ':em'      => $email,
                    ':pd'      => $payDate ?: null,
                    ':ed'      => $endDate ?: null,
                    ':sd'      => $payDate ?: null,
                    ':rc'      => $activationCode
                ]);

                if ($update->rowCount() < 1) {
                    // fallback insert if no row updated
                    $ins2 = $pdo->prepare("
                        INSERT INTO license_info (
                            id,
                            unique_registration_number,
                            license_type,
                            name_or_company,
                            email,
                            pay_date,
                            end_date,
                            start_date,
                            registration_code
                        ) VALUES (1,?,?,?,?,?,?,?,?)
                    ");
                    $ins2->execute([
                        $regNumber,
                        $licenseType,
                        $nameCompany,
                        $email,
                        $payDate ?: null,
                        $endDate ?: null,
                        $payDate ?: null,
                        $activationCode
                    ]);
                }
                $success = "License Activated Successfully.";
            }
        }
    }
}

// Query row id=1 for display
$licenseRow = $pdo->query("SELECT * FROM license_info WHERE id=1")->fetch(PDO::FETCH_ASSOC);
?>

<h1>License Information</h1>

<div class="form-container">
  <?php if (!empty($errors)): ?>
    <div class="alert">
      <?php foreach ($errors as $err): ?>
        <p><?php echo htmlspecialchars($err); ?></p>
      <?php endforeach; ?>
    </div>
  <?php endif; ?>

  <?php if ($success): ?>
    <div class="success">
      <p><?php echo htmlspecialchars($success); ?></p>
    </div>
  <?php endif; ?>

  <form method="POST" action="license.php">
    <label for="activation_code">Enter License Activation Code (Offline):</label>
    <input type="text" name="activation_code" id="activation_code" style="width: 100%;">

    <div class="button-group">
      <button type="submit">Verify / Update License</button>
    </div>
  </form>
</div>

<?php if ($licenseRow): ?>
    <h2>Current License Status</h2>
    <table>
      <tr><th>Unique Registration Number</th>
          <td><?php echo htmlspecialchars($licenseRow['unique_registration_number'] ?? ''); ?></td>
      </tr>
      <tr><th>License Type</th>
          <td><?php echo htmlspecialchars($licenseRow['license_type'] ?? ''); ?></td>
      </tr>
      <tr><th>Name or Company</th>
          <td><?php echo htmlspecialchars($licenseRow['name_or_company'] ?? ''); ?></td>
      </tr>
      <tr><th>Email</th>
          <td><?php echo htmlspecialchars($licenseRow['email'] ?? ''); ?></td>
      </tr>
      <tr><th>Pay Date</th>
          <td><?php echo htmlspecialchars($licenseRow['pay_date'] ?? ''); ?></td>
      </tr>
      <tr><th>Start Date</th>
          <td><?php echo htmlspecialchars($licenseRow['start_date'] ?? ''); ?></td>
      </tr>
      <tr><th>End Date</th>
          <td><?php echo htmlspecialchars($licenseRow['end_date'] ?? ''); ?></td>
      </tr>
    </table>
<?php else: ?>
    <p>No license row found with id=1 in local DB.</p>
<?php endif; ?>

<?php require_once('components/footer.php'); ?>
