<?php
require_once('../core/error_handling.php');
require_once('components/auth.php');
require_once('components/header.php');
$errors = [];
$success = '';

if (!checkRole(['error_handling_log'])) {
    $errors[] = $_SESSION['error_message'];
    unset($_SESSION['error_message']);
    ?>
    <h1>Error Details</h1>
    <?php if (!empty($errors)): ?>
    <div class="alert">
        <?php foreach ($errors as $error): ?>
        <p><?php echo htmlspecialchars($error); ?></p>
        <?php endforeach; ?>
    </div>
    <?php endif; ?>
    <?php require_once('components/footer.php'); ?>
    <?php
    exit;
}

if (!isset($_GET['id']) || empty(trim($_GET['id']))) {
    $_SESSION['error_message'] = "Invalid error ID.";
    header('Location: error_handling.php');
    exit;
}

$errorId = (int)$_GET['id'];
$errorData = getError($pdo, $errorId);
if (!$errorData) {
    $_SESSION['error_message'] = "Error not found.";
    header('Location: error_handling.php');
    exit;
}

?>
<h1>Error Details</h1>
<div class="error-details">
    <h2>Error ID: <?php echo htmlspecialchars($errorData['id']); ?></h2>
    <p><strong>Message:</strong> <?php echo htmlspecialchars($errorData['message']); ?></p>
    <p><strong>Error Level:</strong> <?php echo htmlspecialchars($errorData['error_level']); ?></p>
    <p><strong>Context:</strong></p>
    <pre><?php echo htmlspecialchars($errorData['context']); ?></pre>
    <p><strong>Created At:</strong> <?php echo htmlspecialchars($errorData['created_at']); ?></p>
</div>
<a href="error_handling.php" class="button">Back to Error List</a>
<?php require_once('components/footer.php'); ?>

