<?php
session_start();
require_once('../core/users.php');

$errors = [];
if ($_SERVER['REQUEST_METHOD'] === 'POST') {
    if (isset($_POST['reset_password'])) {
        // Handle password reset
        $new_password = $_POST['new_password'] ?? '';
        $confirm_password = $_POST['confirm_password'] ?? '';

        if (empty($new_password) || empty($confirm_password)) {
            $errors[] = 'Both password fields are required.';
        } elseif ($new_password !== $confirm_password) {
            $errors[] = 'Passwords do not match.';
        } else {
            // Update the user's password
            $user_id = $_SESSION['user_id'];
            if (updateUserPassword($pdo, $user_id, $new_password)) {
                // Set is_reset to 0
                $stmt = $pdo->prepare("UPDATE users SET is_reset = 0 WHERE id = :id");
                $stmt->execute([':id' => $user_id]);

                // Redirect to the dashboard or desired page
                header('Location: index.php');
                exit;
            } else {
                $errors[] = 'Failed to update password. Please try again.';
            }
        }
    } else {
        // Handle normal login
        $username = trim($_POST['username']);
        $password = $_POST['password'];
        if (authenticateUser($pdo, $username, $password)) {
            $user = getUserByUsername($pdo, $username);
            $_SESSION['username'] = $username;
            $_SESSION['user_id'] = $user['id'];
            $_SESSION['roles'] = getUserRoles($pdo, $user['id']);

            // SNIPPET: clear user-specific temp folder after login
            $userTempDir = __DIR__ . '/temp/' . $username;
            if (is_dir($userTempDir)) {
                $files = glob($userTempDir . '/*');
                foreach ($files as $f) {
                    if (is_file($f)) {
                        @unlink($f);
                    }
                }
            }

            if ($user['is_reset'] == 1) {
                // Redirect to password reset form
                header('Location: login.php?reset=1');
                exit;
            } else {
                // Redirect to the dashboard
                header('Location: index.php');
                exit;
            }
        } else {
            $errors[] = 'Invalid username, password, or your account is not yet authorized.';
        }
    }
}

require_once('components/header.php');

function hasRole($role) {
    if (!isset($_SESSION['username'])) {
        return false;
    }

    // Get the user's roles from the session
    $userRoles = $_SESSION['roles'] ?? [];

    // If the user has the 'admin' role, they have access to everything
    if (in_array('admin', $userRoles)) {
        return true;
    }

    // Check if the user has the specified role
    return in_array($role, $userRoles);
}

?>
<h1>Login</h1>
<?php if (!empty($errors)): ?>
    <div class="alert">
        <?php foreach($errors as $error): ?>
            <p><?php echo htmlspecialchars($error); ?></p>
        <?php endforeach; ?>
    </div>
<?php endif; ?>

<?php if (isset($_GET['reset']) && $_GET['reset'] == 1): ?>
    <!-- Password Reset Form -->
    <div class="form-container">
        <h2 style="text-align:center;">DONNOTEC double entry accounting system</h2>
        <img src="css/donnotec_logo.png" alt="donnotec_logo" width="300" height="150" style="display: block;margin-left: auto;margin-right: auto;width: 50%;">
        <form action="login.php" method="POST">
            <label for="new_password">New Password</label>
            <input type="password" id="new_password" name="new_password" required>

            <label for="confirm_password">Confirm New Password</label>
            <input type="password" id="confirm_password" name="confirm_password" required>

            <button type="submit" name="reset_password">Reset Password</button>
        </form>
    </div>
<?php else: ?>
    <!-- Login Form -->
    <div class="form-container">
        <h2 style="text-align:center;">DONNOTEC double entry accounting system</h2>
        <img src="css/donnotec_logo.png" alt="donnotec_logo" width="300" height="150" style="display: block;margin-left: auto;margin-right: auto;width: 50%;">
        <form action="login.php" method="POST">
            <label for="username">Username</label>
            <input type="text" id="username" name="username" required>

            <label for="password">Password</label>
            <input type="password" id="password" name="password" required>

            <button type="submit">Login</button>
        </form>
    </div>
<?php endif; ?>

<?php require_once('components/footer.php'); ?>
