<?php
require_once('../core/users.php');
require_once('components/auth.php');
require_once('components/header.php');

try {
    $pdo = new PDO('sqlite:' . DB_FILE);
    $pdo->setAttribute(PDO::ATTR_ERRMODE, PDO::ERRMODE_EXCEPTION);
} catch (PDOException $e) {
    die("Database Connection Failed: " . $e->getMessage());
}

$errors = [];
$success = '';

if (!checkRole(['clients_manage'])) {
    $errors[] = $_SESSION['error_message'] ?? 'Access denied. You do not have permission to manage clients.';
    unset($_SESSION['error_message']);
?>
<h1>Manage Clients</h1>
<?php if (!empty($errors)): ?>
<div class="alert">
<?php foreach ($errors as $error): ?>
<p><?php echo htmlspecialchars($error); ?></p>
<?php endforeach; ?>
</div>
<?php endif; ?>
<?php require_once('components/footer.php'); ?>
<?php
    exit;
}

$clients = listUsers($pdo, ['role_id' => 1]); // Assuming role ID for clients is 1

?>
<h1>Manage Clients</h1>
<?php if (!empty($success)): ?>
<div class="success">
<p><?php echo htmlspecialchars($success); ?></p>
</div>
<?php endif; ?>
<table>
<thead>
<tr>
<th>Name</th>
<th>Email</th>
<th>Role</th>
<th>Actions</th>
</tr>
</thead>
<tbody>
<?php foreach ($clients as $client): ?>
<tr>
<td><?php echo htmlspecialchars($client['username']); ?></td>
<td><?php echo htmlspecialchars($client['email']); ?></td>
<td><?php echo htmlspecialchars(getUserRoles($pdo, $client['id'])[0]['name']); ?></td>
<td>
<a href="edit_client.php?id=<?php echo htmlspecialchars($client['id']); ?>" class="button">Edit</a>
<form action="delete_client.php" method="POST" style="display:inline;" onsubmit="return confirm('Are you sure you want to delete this client?');">
<input type="hidden" name="id" value="<?php echo htmlspecialchars($client['id']); ?>">
<button type="submit" class="button delete-button">Delete</button>
</form>
</td>
</tr>
<?php endforeach; ?>
</tbody>
</table>
<?php require_once('components/footer.php'); ?>
