<?php
require_once('../core/users.php');
require_once('components/header.php');

$errors = [];
$success = '';

if ($_SERVER['REQUEST_METHOD'] === 'POST') {
    $username = trim($_POST['username']);
    $email = trim($_POST['email']);
    $password = $_POST['password'];
    $confirmPassword = $_POST['confirm_password'];

    // Basic validation
    if (empty($username) || empty($email) || empty($password) || empty($confirmPassword)) {
        $errors[] = 'All fields are required.';
    }

    if ($password !== $confirmPassword) {
        $errors[] = 'Passwords do not match.';
    }

    // Check if username or email already exists
    $existingUsers = listUsers($pdo, ['username' => $username]);
    if (!empty($existingUsers)) {
        $errors[] = 'Username is already taken.';
    }

    $existingEmails = listUsers($pdo, ['email' => $email]);
    if (!empty($existingEmails)) {
        $errors[] = 'An account with this email already exists.';
    }

    if (empty($errors)) {
        $userId = createUser($pdo, $username, $password, $email,0);
        if ($userId) {
            $success = 'Registration successful! Your account will be activated after approval.';
        } else {
            $errors[] = 'Registration failed. Please try again.';
        }
    }
}
?>
<h1>Register</h1>
<?php if (!empty($errors)): ?>
    <div class="alert">
        <?php foreach ($errors as $error): ?>
            <p><?php echo htmlspecialchars($error); ?></p>
        <?php endforeach; ?>
    </div>
<?php endif; ?>
<?php if ($success): ?>
    <div class="success">
        <p><?php echo htmlspecialchars($success); ?></p>
        <a href="login.php" class="button">Go to Login</a>
    </div>
<?php else: ?>
    <div class="form-container">
        <h2 style="text-align:center;">DONNOTEC double entry accounting system</h2>
        <img src="css/donnotec_logo.png" alt="donnotec_logo" width="300" height="150" style="display: block;margin-left: auto;margin-right: auto;width: 50%;">
        <form action="register.php" method="POST">
            <label for="username">Username</label>
            <input type="text" id="username" name="username" required>

            <label for="email">Email</label>
            <input type="email" id="email" name="email" required>

            <label for="password">Password</label>
            <input type="password" id="password" name="password" required>

            <label for="confirm_password">Confirm Password</label>
            <input type="password" id="confirm_password" name="confirm_password" required>

            <button type="submit">Register</button>
        </form>
    </div>
<?php endif; ?>
<?php require_once('components/footer.php'); ?>
