<?php
require_once('../core/users.php');
require_once('components/auth.php');
require_once('components/header.php');
$errors = [];
$success = [];

// Update role check to 'roles'
if (!checkRole(['roles'])) {
    $errors[] = $_SESSION['error_message'];
    unset($_SESSION['error_message']);
    ?>
    <h1>Manage Roles</h1>
    <?php if (!empty($errors)): ?>
    <div class="alert">
        <?php foreach ($errors as $error): ?>
            <p><?php echo htmlspecialchars($error); ?></p>
        <?php endforeach; ?>
    </div>
    <?php endif; ?>
    <?php require_once('components/footer.php'); ?>
    <?php
    exit;
}

if ($_SERVER['REQUEST_METHOD'] === 'POST') {
    if (isset($_POST['delete_role_id']) && hasRole('roles_delete')) {
        $roleId = (int)$_POST['delete_role_id'];
        if ($roleId > 0) {
            $result = deleteRole($pdo, $roleId, $_SESSION['user_id']);
            if ($result) {
                $success = 'Role deleted successfully.';
            } else {
                $errors[] = 'Failed to delete role.';
            }
        }
    }
}

$roles = listRoles($pdo);

if (isset($_SESSION['success_message'])) {
    $success = $_SESSION['success_message'];
    unset($_SESSION['success_message']);
}
?>
<h1>Manage Roles</h1>
<?php if (!empty($errors)): ?>
<div class="alert">
    <?php foreach ($errors as $error): ?>
        <p><?php echo htmlspecialchars($error); ?></p>
    <?php endforeach; ?>
</div>
<?php endif; ?>
<?php if ($success): ?>
<div class="success">
    <p><?php echo htmlspecialchars($success); ?></p>
</div>
<?php endif; ?>
<div class="button-container">
    <?php if (hasRole('roles_add')): ?>
        <a href="add_roles.php" class="button">Add Role</a>
    <?php endif; ?>
</div>
<table>
<thead>
<tr>
    <th>Role Name</th>
    <th>Actions</th>
</tr>
</thead>
<tbody>
<?php foreach ($roles as $role): ?>
<tr>
    <td><?php echo htmlspecialchars($role['name']); ?></td>
    <td>
        <?php if (hasRole('roles_delete') && isset($role['is_system']) && $role['is_system'] != 1): ?>
            <form action="roles.php" method="POST" style="display:inline;">
                <input type="hidden" name="delete_role_id" value="<?php echo $role['id']; ?>">
                <button type="submit" class="button delete-button" onclick="return confirm('Are you sure you want to delete this role?');">Delete</button>
            </form>
        <?php endif; ?>
    </td>
</tr>
<?php endforeach; ?>
</tbody>
</table>
<?php require_once('components/footer.php'); ?>
