<?php
/**
 * /templates/transactions.php
 *
 * This file outputs JavaScript defining the TransactionsTemplates object,
 * which includes templates for both No-VAT and With-VAT transactions.
 */

header('Content-Type: application/javascript');

// Include configuration to access constants
require_once('../../config/config.php');
?>

window.TransactionsTemplates = window.TransactionsTemplates || {};

// 1) Templates for "Add Transaction" (No VAT)
TransactionsTemplates.noVAT = {
    templates: [
        { label: "Purchase with Cash", value: "Purchase_Cash" },
        { label: "Purchase with Credit", value: "Purchase_Credit" },
        { label: "Sale with Cash", value: "Sale_Cash" },
        { label: "Sale with Credit", value: "Sale_Credit" },
        { label: "Purchase Return with Cash", value: "PurchaseReturn_Cash" },
        { label: "Purchase Return with Credit", value: "PurchaseReturn_Credit" },
        { label: "Sale Return with Cash", value: "SaleReturn_Cash" },
        { label: "Sale Return with Credit", value: "SaleReturn_Credit" }
    ],

    applyTemplate: function(templateKey, debitAccountElem, creditAccountElem) {
        switch(templateKey){
            case "Purchase_Cash":
                debitAccountElem.value  = "<?php echo EXPENCE_ACC_CODE; ?>";
                creditAccountElem.value = "<?php echo CASH_BANK_ACC_CODE; ?>";
                break;
            case "Purchase_Credit":
                debitAccountElem.value  = "<?php echo EXPENCE_ACC_CODE; ?>";
                creditAccountElem.value = "<?php echo ACCOUNTS_PAYABLE_ACC_CODE; ?>";
                break;
            case "Sale_Cash":
                debitAccountElem.value  = "<?php echo CASH_BANK_ACC_CODE; ?>";
                creditAccountElem.value = "<?php echo INCOME_ACC_CODE; ?>";
                break;
            case "Sale_Credit":
                debitAccountElem.value  = "<?php echo ACCOUNTS_RECEIVABLE_ACC_CODE; ?>";
                creditAccountElem.value = "<?php echo INCOME_ACC_CODE; ?>";
                break;
            case "PurchaseReturn_Cash":
                debitAccountElem.value  = "<?php echo CASH_BANK_ACC_CODE; ?>";
                creditAccountElem.value = "<?php echo EXPENCE_ACC_CODE; ?>";
                break;
            case "PurchaseReturn_Credit":
                debitAccountElem.value  = "<?php echo ACCOUNTS_PAYABLE_ACC_CODE; ?>";
                creditAccountElem.value = "<?php echo EXPENCE_ACC_CODE; ?>";
                break;
            case "SaleReturn_Cash":
                debitAccountElem.value  = "<?php echo INCOME_ACC_CODE; ?>";
                creditAccountElem.value = "<?php echo CASH_BANK_ACC_CODE; ?>";
                break;
            case "SaleReturn_Credit":
                debitAccountElem.value  = "<?php echo INCOME_ACC_CODE; ?>";
                creditAccountElem.value = "<?php echo ACCOUNTS_RECEIVABLE_ACC_CODE; ?>";
                break;
            default:
                // "Custom Transaction" => do nothing
                break;
        }
    }
};

// 2) Templates for "Add Transaction With VAT"
TransactionsTemplates.withVAT = {
    templates: [
        { label: "Purchase with Cash", value: "Purchase_Cash" },
        { label: "Purchase with Credit", value: "Purchase_Credit" },
        { label: "Sale with Cash", value: "Sale_Cash" },
        { label: "Sale with Credit", value: "Sale_Credit" },
        { label: "Purchase Return with Cash", value: "PurchaseReturn_Cash" },
        { label: "Purchase Return with Credit", value: "PurchaseReturn_Credit" },
        { label: "Sale Return with Cash", value: "SaleReturn_Cash" },
        { label: "Sale Return with Credit", value: "SaleReturn_Credit" }
    ],

    applyTemplate: function(templateKey, primaryAccountElem, primaryPositionElem, secondaryAccountElem) {

        // We also want to set the Reversal checkbox for Return cases
        var reversalCheckbox = document.getElementById('vat_reversal');
        if (reversalCheckbox) {
            reversalCheckbox.checked = false; // default
        }

        switch(templateKey){
            case "Purchase_Cash":
                primaryAccountElem.value   = "<?php echo CASH_BANK_ACC_CODE; ?>";
                primaryPositionElem.value  = "credit";
                secondaryAccountElem.value = "<?php echo EXPENCE_ACC_CODE; ?>";
                if (reversalCheckbox) reversalCheckbox.checked = false;
                break;
            case "Purchase_Credit":
                primaryAccountElem.value   = "<?php echo ACCOUNTS_PAYABLE_ACC_CODE; ?>";
                primaryPositionElem.value  = "credit";
                secondaryAccountElem.value = "<?php echo EXPENCE_ACC_CODE; ?>";
                if (reversalCheckbox) reversalCheckbox.checked = false;
                break;
            case "Sale_Cash":
                primaryAccountElem.value   = "<?php echo CASH_BANK_ACC_CODE; ?>";
                primaryPositionElem.value  = "debit";
                secondaryAccountElem.value = "<?php echo INCOME_ACC_CODE; ?>";
                if (reversalCheckbox) reversalCheckbox.checked = false;
                break;
            case "Sale_Credit":
                primaryAccountElem.value   = "<?php echo ACCOUNTS_RECEIVABLE_ACC_CODE; ?>";
                primaryPositionElem.value  = "debit";
                secondaryAccountElem.value = "<?php echo INCOME_ACC_CODE; ?>";
                if (reversalCheckbox) reversalCheckbox.checked = false;
                break;
            case "PurchaseReturn_Cash":
                primaryAccountElem.value   = "<?php echo CASH_BANK_ACC_CODE; ?>";
                primaryPositionElem.value  = "debit";
                secondaryAccountElem.value = "<?php echo EXPENCE_ACC_CODE; ?>";
                if (reversalCheckbox) reversalCheckbox.checked = true; // Return => reversal
                break;
            case "PurchaseReturn_Credit":
                primaryAccountElem.value   = "<?php echo ACCOUNTS_PAYABLE_ACC_CODE; ?>";
                primaryPositionElem.value  = "debit";
                secondaryAccountElem.value = "<?php echo EXPENCE_ACC_CODE; ?>";
                if (reversalCheckbox) reversalCheckbox.checked = true;
                break;
            case "SaleReturn_Cash":
                primaryAccountElem.value   = "<?php echo CASH_BANK_ACC_CODE; ?>";
                primaryPositionElem.value  = "credit";
                secondaryAccountElem.value = "<?php echo INCOME_ACC_CODE; ?>";
                if (reversalCheckbox) reversalCheckbox.checked = true;
                break;
            case "SaleReturn_Credit":
                primaryAccountElem.value   = "<?php echo ACCOUNTS_RECEIVABLE_ACC_CODE; ?>";
                primaryPositionElem.value  = "credit";
                secondaryAccountElem.value = "<?php echo INCOME_ACC_CODE; ?>";
                if (reversalCheckbox) reversalCheckbox.checked = true;
                break;
            default:
                // "Custom Transaction" => do nothing
                break;
        }
    }
};
