 <?php
require_once('../core/financial_periods.php');
require_once('components/auth.php');
require_once('components/header.php');
require_once('components/form_component.php');

$errors = [];
$success = '';

if (!checkRole(['financial_period_update'])) {
    $errors[] = $_SESSION['error_message'];
    unset($_SESSION['error_message']);
    ?>
    <h1>Update Financial Period</h1>
    <?php if (!empty($errors)): ?>
        <div class="alert">
            <?php foreach ($errors as $error): ?>
                <p><?php echo htmlspecialchars($error); ?></p>
            <?php endforeach; ?>
        </div>
    <?php endif; ?>
    <?php require_once('components/footer.php'); ?>
    <?php
    exit;
}

if (!isset($_GET['id']) || empty(trim($_GET['id']))) {
    $_SESSION['error_message'] = "Invalid financial period ID.";
    header('Location: financial_periods.php');
    exit;
}

$periodId = (int)$_GET['id'];
$financialPeriod = getFinancialPeriod($pdo, $periodId);

if (!$financialPeriod) {
    $_SESSION['error_message'] = "Financial period not found.";
    header('Location: financial_periods.php');
    exit;
}

if ($_SERVER['REQUEST_METHOD'] === 'POST') {
    $name = trim($_POST['name']);
    $startDate = trim($_POST['start_date']);
    $endDate = trim($_POST['end_date']);

    if (empty($name) || empty($startDate) || empty($endDate)) {
        $errors[] = 'Name, Start Date, and End Date are required.';
    }

    if (strtotime($startDate) === false || strtotime($endDate) === false) {
        $errors[] = 'Invalid date format.';
    }

    if (strtotime($startDate) > strtotime($endDate)) {
        $errors[] = 'Start Date cannot be after End Date.';
    }

    if (empty($errors)) {
        function updateFinancialPeriodFull($pdo, int $financialPeriodId, string $name, string $startDate, string $endDate): bool {
            try {
                $stmt = $pdo->prepare("UPDATE financial_periods SET name = ?, start_date = ?, end_date = ? WHERE id = ?");
                $stmt->execute([$name, $startDate, $endDate, $financialPeriodId]);
                return true;
            } catch (PDOException $e) {
                error_log('Update Financial Period Error: ' . $e->getMessage(), 3, LOG_FILE);
                return false;
            }
        }
        $result = updateFinancialPeriodFull($pdo, $periodId, $name, $startDate, $endDate,$_SESSION['user_id']);
        if ($result) {
            $success = 'Financial period updated successfully.';
            $financialPeriod = getFinancialPeriod($pdo, $periodId);
        } else {
            $errors[] = 'Failed to update financial period.';
        }
    }
}

?>
<h1>Update Financial Period</h1>
<?php if (!empty($errors)): ?>
    <div class="alert">
        <?php foreach ($errors as $error): ?>
            <p><?php echo htmlspecialchars($error); ?></p>
        <?php endforeach; ?>
    </div>
<?php endif; ?>
<?php if ($success): ?>
    <div class="success">
        <p><?php echo htmlspecialchars($success); ?></p>
        <a href="financial_periods.php" class="button">Back to Financial Periods</a>
    </div>
<?php else: ?>
    <div class="form-container">
        <form action="update_financial_period.php?id=<?php echo $periodId; ?>" method="POST">
            <?php
            renderInput('text', 'name', 'Name', htmlspecialchars($financialPeriod['name']), true);
            renderInput('date', 'start_date', 'Start Date', htmlspecialchars($financialPeriod['start_date']), true);
            renderInput('date', 'end_date', 'End Date', htmlspecialchars($financialPeriod['end_date']), true);
            ?>
            <div class="button-group">
                <button type="submit">Update Financial Period</button>
                <a href="financial_periods.php" class="button">Cancel</a>
            </div>
        </form>
    </div>
<?php endif; ?>
<?php require_once('components/footer.php'); ?>

