<?php
require_once('../core/vat_periods.php');
require_once('components/auth.php');
require_once('components/header.php');

$errors = [];
$success = '';

if (!checkRole(['vat_period_update'])) {
    $errors[] = 'Access denied. You do not have permission to update VAT periods.';
    ?>
    <h1>Update VAT Period</h1>
    <?php if (!empty($errors)): ?>
    <div class="alert">
        <?php foreach ($errors as $error): ?>
        <p><?php echo htmlspecialchars($error); ?></p>
        <?php endforeach; ?>
    </div>
    <?php endif; ?>
    <?php
    require_once('components/footer.php');
    exit;
}

if (!isset($_GET['id']) || empty(trim($_GET['id']))) {
    $_SESSION['error_message'] = 'Invalid VAT Period ID.';
    header('Location: vat_periods.php');
    exit;
}

$vatPeriodId = (int)$_GET['id'];
$vatPeriod = getVATPeriod($pdo, $vatPeriodId);

if (!$vatPeriod) {
    $_SESSION['error_message'] = 'VAT Period not found.';
    header('Location: vat_periods.php');
    exit;
}

if ($_SERVER['REQUEST_METHOD'] === 'POST') {
    if (isset($_POST['update_vat_period'])) {
        $name = trim($_POST['name']);
        $start_date = trim($_POST['start_date']);
        $end_date = trim($_POST['end_date']);

        if (empty($name) || empty($start_date) || empty($end_date)) {
            $errors[] = 'All fields are required.';
        } elseif (strtotime($start_date) > strtotime($end_date)) {
            $errors[] = 'Start date cannot be after end date.';
        }

        // Check for overlapping VAT periods
        $existingPeriods = listVATPeriods($pdo);
        foreach ($existingPeriods as $period) {
            if ($period['id'] != $vatPeriodId && (strtotime($start_date) <= strtotime($period['end_date'])) && (strtotime($end_date) >= strtotime($period['start_date']))) {
                $errors[] = 'The date range overlaps with an existing VAT period.';
                break;
            }
        }

        if (empty($errors)) {
            $result = updateVATPeriod($pdo, $vatPeriodId, $name, $start_date, $end_date,$_SESSION['user_id']);
            if ($result) {
                $success = 'VAT Period updated successfully.';
                $vatPeriod = getVATPeriod($pdo, $vatPeriodId);
            } else {
                $errors[] = 'Failed to update VAT Period.';
            }
        }
    }
}

?>
<h1>Update VAT Period</h1>
<?php if (!empty($errors)): ?>
<div class="alert">
    <?php foreach ($errors as $error): ?>
    <p><?php echo htmlspecialchars($error); ?></p>
    <?php endforeach; ?>
</div>
<?php endif; ?>
<?php if ($success): ?>
<div class="success">
    <p><?php echo htmlspecialchars($success); ?></p>
</div>
<?php endif; ?>
<div class="form-container">
    <form action="update_vat_periods.php?id=<?php echo $vatPeriodId; ?>" method="POST">
        <label for="name">VAT Period Name</label>
        <input type="text" id="name" name="name" value="<?php echo htmlspecialchars($vatPeriod['name']); ?>" required>

        <label for="start_date">Start Date</label>
        <input type="date" id="start_date" name="start_date" value="<?php echo htmlspecialchars($vatPeriod['start_date']); ?>" required>

        <label for="end_date">End Date</label>
        <input type="date" id="end_date" name="end_date" value="<?php echo htmlspecialchars($vatPeriod['end_date']); ?>" required>

        <div class="button-group">
            <button type="submit" name="update_vat_period">Save Changes</button>
            <a href="vat_periods.php" class="button">Back</a>
        </div>
    </form>
</div>
<?php require_once('components/footer.php'); ?>
