<?php
require_once('../core/users.php');
require_once('components/auth.php');
require_once('components/header.php');
require_once('components/form_component.php');
$errors = [];
$success = '';

$userId = $_SESSION['user_id'] ?? 0;
$userRoles = $_SESSION['roles'] ?? [];

// Update role check to 'users'
if (!checkRole(['users'])) {
    $errors[] = 'Access denied. You do not have permission to view this page.';
    ?>
    <h1>Manage Users</h1>
    <?php if (!empty($errors)): ?>
    <div class="alert">
        <?php foreach ($errors as $error): ?>
            <p><?php echo htmlspecialchars($error); ?></p>
        <?php endforeach; ?>
    </div>
    <?php endif; ?>
    <?php require_once('components/footer.php'); ?>
    <?php
    exit;
}

if ($_SERVER['REQUEST_METHOD'] === 'POST') {
    if (isset($_POST['authorize_user_id']) && hasRole('users')) {
        $authorizeUserId = (int)$_POST['authorize_user_id'];
        if ($authorizeUserId > 0) {
            $result = updateUser($pdo, $authorizeUserId, ['is_auth' => 1], $_SESSION['user_id']);
            if ($result) {
                $success = 'User has been authorized successfully.';
            } else {
                $errors[] = 'Failed to authorize user.';
            }
        }
    } elseif (isset($_POST['delete_user_id']) && hasRole('users_delete')) {
        $deleteUserId = (int)$_POST['delete_user_id'];
        if ($deleteUserId > 0) {
            $result = deleteUser($pdo, $deleteUserId, $_SESSION['user_id']);
            if ($result) {
                $success = 'User has been deleted successfully.';
            } else {
                $errors[] = 'Failed to delete user.';
            }
        }
    } elseif (isset($_POST['reset_user_id']) && hasRole('users_reset')) {
        $resetUserId = (int)$_POST['reset_user_id'];
        if ($resetUserId > 0) {
            $newPassword = bin2hex(random_bytes(5));
            $result = updateUser($pdo, $resetUserId, ['password' => $newPassword], $_SESSION['user_id']);
            if ($result) {
                $success = 'Password reset successfully. New password: ' . htmlspecialchars($newPassword);
            } else {
                $errors[] = 'Failed to reset password.';
            }
        }
    }
}

$users = listUsers($pdo);
?>
<h1>Manage Users</h1>
<?php if (!empty($errors)): ?>
<div class="alert">
    <?php foreach ($errors as $error): ?>
        <p><?php echo htmlspecialchars($error); ?></p>
    <?php endforeach; ?>
</div>
<?php endif; ?>
<?php if ($success): ?>
<div class="success">
    <p><?php echo htmlspecialchars($success); ?></p>
</div>
<?php endif; ?>
<div class="button-container">
    <?php if (hasRole('users_add')): ?>
        <a href="add_user.php" class="button">Add User</a>
    <?php endif; ?>
    <?php if (hasRole('roles')): ?>
        <a href="roles.php" class="button">Manage Roles</a>
    <?php endif; ?>
</div>
<table>
<thead>
<tr>
    <th>Username</th>
    <th>Email</th>
    <th>Authorized</th>
    <th>Roles</th>
    <th>Actions</th>
</tr>
</thead>
<tbody>
<?php foreach ($users as $user): ?>
<?php
    $userRolesList = getUserRoles($pdo, $user['id']);
    $isAuth = $user['is_auth'] == 1 ? 'Yes' : 'No';
    $isSystem = isset($user['is_system']) ? (int)$user['is_system'] : 0;
?>
<tr>
    <td><?php echo htmlspecialchars($user['username']); ?></td>
    <td><?php echo htmlspecialchars($user['email']); ?></td>
    <td><?php echo $isAuth; ?></td>
    <td><?php echo htmlspecialchars(implode(', ', $userRolesList)); ?></td>
    <td>
        <?php if ($isSystem !== 1): ?>
            <?php if ($user['is_auth'] == 0 && hasRole('users')): ?>
                <form action="users.php" method="POST" style="display:inline;">
                    <input type="hidden" name="authorize_user_id" value="<?php echo $user['id']; ?>">
                    <button type="submit" class="button">Authorize</button>
                </form>
            <?php endif; ?>
            <?php if (hasRole('users_edit')): ?>
                <a href="edit_user.php?id=<?php echo $user['id']; ?>" class="button edit-button">Edit</a>
            <?php endif; ?>
            <?php if (hasRole('users_delete')): ?>
                <form action="users.php" method="POST" style="display:inline;">
                    <input type="hidden" name="delete_user_id" value="<?php echo $user['id']; ?>">
                    <button type="submit" class="button delete-button" onclick="return confirm('Are you sure you want to delete this user?');">Delete</button>
                </form>
            <?php endif; ?>
            <?php if (hasRole('users_reset')): ?>
                <form action="users.php" method="POST" style="display:inline;">
                    <input type="hidden" name="reset_user_id" value="<?php echo $user['id']; ?>">
                    <button type="submit" class="button reset-button" onclick="return confirm('Are you sure you want to reset this user\'s password?');">Reset Password</button>
                </form>
            <?php endif; ?>
        <?php endif; ?>
    </td>
</tr>
<?php endforeach; ?>
</tbody>
</table>
<?php require_once('components/footer.php'); ?>
