<?php
ini_set('display_errors', 1);
ini_set('display_startup_errors', 1);
error_reporting(E_ALL);

require_once('../core/vat_periods.php');
require_once('components/auth.php');
require_once('components/header.php');

$errors = [];
$success = '';

if (!checkRole(['vat_period'])) {
    $errors[] = 'Access denied. You do not have permission to view this page.';
    ?>
    <h1>Manage VAT Periods</h1>
    <?php if (!empty($errors)): ?>
    <div class="alert">
        <?php foreach ($errors as $error): ?>
        <p><?php echo htmlspecialchars($error); ?></p>
        <?php endforeach; ?>
    </div>
    <?php endif; ?>
    <?php
    require_once('components/footer.php');
    exit;
}

if ($_SERVER['REQUEST_METHOD'] === 'POST') {

    // Close
    if (isset($_POST['close_vat_period_id']) && hasRole('vat_period_close')) {
        $vatPeriodId = (int)$_POST['close_vat_period_id'];
        $result = closeVATPeriod($pdo, $vatPeriodId, $_SESSION['user_id']);
        if ($result['success']) {
            $success = $result['message'];
        } else {
            $errors[] = $result['message'];
        }
    }

    // Reopen
    elseif (isset($_POST['reopen_vat_period_id']) && hasRole('vat_period_reopen')) {
        $vatPeriodId = (int)$_POST['reopen_vat_period_id'];
        $result = reopenVATPeriod($pdo, $vatPeriodId, $_SESSION['user_id']);
        if ($result['success']) {
            $success = $result['message'];
        } else {
            $errors[] = $result['message'];
        }
    }

    // Delete
    elseif (isset($_POST['delete_vat_period_id']) && hasRole('vat_period_delete')) {
        $vatPeriodId = (int)$_POST['delete_vat_period_id'];
        $result = deleteVATPeriod($pdo, $vatPeriodId, $_SESSION['user_id']);
        if ($result['success']) {
            $success = $result['message'];
        } else {
            $errors[] = $result['message'];
        }
    }
}

$vatPeriods = listVATPeriods($pdo);

if (isset($_SESSION['success_message'])) {
    $success = $_SESSION['success_message'];
    unset($_SESSION['success_message']);
}
if (isset($_SESSION['error_message'])) {
    $errors[] = $_SESSION['error_message'];
    unset($_SESSION['error_message']);
}
?>

<h1>Manage VAT Periods</h1>

<?php if (!empty($errors)): ?>
<div class="alert">
    <?php foreach ($errors as $error): ?>
    <p><?php echo htmlspecialchars($error); ?></p>
    <?php endforeach; ?>
</div>
<?php endif; ?>

<?php if ($success): ?>
<div class="success">
    <p><?php echo htmlspecialchars($success); ?></p>
</div>
<?php endif; ?>

<div class="button-container">
    <?php if (hasRole('vat_period_add')): ?>
    <a href="add_vat_periods.php" class="button">Add VAT Period</a>
    <?php endif; ?>
</div>

<table>
    <thead>
        <tr>
            <th>Name</th>
            <th>Start Date</th>
            <th>End Date</th>
            <th>Status</th>
            <th>Actions</th>
        </tr>
    </thead>
    <tbody>
        <?php foreach ($vatPeriods as $vatPeriod): ?>
        <tr>
            <td><?php echo htmlspecialchars($vatPeriod['name']); ?></td>
            <td><?php echo htmlspecialchars($vatPeriod['start_date']); ?></td>
            <td><?php echo htmlspecialchars($vatPeriod['end_date']); ?></td>
            <td><?php echo htmlspecialchars(ucfirst($vatPeriod['status'])); ?></td>
            <td>
                <?php if (hasRole('vat_period_update')): ?>
                <a href="update_vat_periods.php?id=<?php echo $vatPeriod['id']; ?>" class="button edit-button">Edit</a>
                <?php endif; ?>

                <?php if ($vatPeriod['status'] == 'open' && hasRole('vat_period_close')): ?>
                <form action="vat_periods.php" method="POST" style="display:inline;">
                    <input type="hidden" name="close_vat_period_id" value="<?php echo $vatPeriod['id']; ?>">
                    <button type="submit" class="button close-button"
                            onclick="return confirm('Are you sure you want to close this VAT period?');">
                        Close
                    </button>
                </form>
                <?php elseif ($vatPeriod['status'] == 'closed' && hasRole('vat_period_reopen')): ?>
                <form action="vat_periods.php" method="POST" style="display:inline;">
                    <input type="hidden" name="reopen_vat_period_id" value="<?php echo $vatPeriod['id']; ?>">
                    <button type="submit" class="button reopen-button"
                            onclick="return confirm('Are you sure you want to reopen this VAT period?');">
                        Reopen
                    </button>
                </form>
                <?php endif; ?>

                <?php if (hasRole('vat_period_delete')): ?>
                <form action="vat_periods.php" method="POST" style="display:inline;">
                    <input type="hidden" name="delete_vat_period_id" value="<?php echo $vatPeriod['id']; ?>">
                    <button type="submit" class="button delete-button"
                            onclick="return confirm('Are you sure you want to delete this VAT period?');">
                        Delete
                    </button>
                </form>
                <?php endif; ?>
            </td>
        </tr>
        <?php endforeach; ?>
    </tbody>
</table>

<?php require_once('components/footer.php'); ?>
