<?php
require_once('../core/vat_rates.php');
require_once('components/auth.php');
require_once('components/header.php');
$errors = [];
$success = '';

if (!checkRole(['vat_rates'])) {
    $errors[] = $_SESSION['error_message'];
    unset($_SESSION['error_message']);
    ?>
    <h1>Manage VAT Rates</h1>
    <?php if (!empty($errors)): ?>
        <div class="alert">
            <?php foreach ($errors as $error): ?>
                <p><?php echo htmlspecialchars($error); ?></p>
            <?php endforeach; ?>
        </div>
    <?php endif; ?>
    <?php require_once('components/footer.php'); ?>
    <?php
    exit;
}

// Retrieve and display success and error messages from the session
if (isset($_SESSION['success_message'])) {
    $success = $_SESSION['success_message'];
    unset($_SESSION['success_message']);
}

if ($_SERVER['REQUEST_METHOD'] === 'POST') {
    if (isset($_POST['delete_vat_rate_id']) && hasRole('vat_rates_delete')) {
        $vatRateId = (int)$_POST['delete_vat_rate_id'];
        if ($vatRateId > 0) {
            $result = deleteVATRate($pdo, $vatRateId,$_SESSION['user_id']);
            if ($result) {
                $success = 'VAT Rate deleted successfully.';
            } else {
                $errors[] = 'Failed to delete VAT Rate.';
            }
        }
    } elseif (isset($_POST['set_default_vat_rate_id']) && hasRole('vat_rates_default')) {
        $vatRateId = (int)$_POST['set_default_vat_rate_id'];
        if ($vatRateId > 0) {
            $result = setDefaultVATRate($pdo, $vatRateId,$_SESSION['user_id']);
            if ($result) {
                $success = 'Default VAT Rate set successfully.';
            } else {
                $errors[] = 'Failed to set default VAT Rate.';
            }
        }
    }
}

$vatRates = listVATRates($pdo);
?>
<h1>Manage VAT Rates</h1>
<?php if (!empty($errors)): ?>
    <div class="alert">
        <?php foreach ($errors as $error): ?>
            <p><?php echo htmlspecialchars($error); ?></p>
        <?php endforeach; ?>
    </div>
<?php endif; ?>
<?php if ($success): ?>
    <div class="success">
        <p><?php echo htmlspecialchars($success); ?></p>
    </div>
<?php endif; ?>
<div class="button-container">
    <?php if (hasRole('vat_rates_add')): ?>
        <a href="add_vat_rates.php" class="button">Add VAT Rate</a>
    <?php endif; ?>
</div>
<table>
    <thead>
        <tr>
            <th>Rate Name</th>
            <th>Rate Value (%)</th>
            <th>Default</th>
            <th>Actions</th>
        </tr>
    </thead>
    <tbody>
        <?php foreach ($vatRates as $vatRate): ?>
            <tr>
                <td><?php echo htmlspecialchars($vatRate['rate_name']); ?></td>
                <td><?php echo htmlspecialchars(number_format($vatRate['rate_value'], 2)); ?></td>
                <td><?php echo $vatRate['is_default'] ? 'Yes' : 'No'; ?></td>
                <td>
                    <?php if (hasRole('vat_rates_update')): ?>
                        <a href="update_vat_rates.php?id=<?php echo $vatRate['id']; ?>" class="button edit-button">Edit</a>
                    <?php endif; ?>
                    <?php if (hasRole('vat_rates_delete')): ?>
                        <form action="vat_rates.php" method="POST" style="display:inline;">
                            <input type="hidden" name="delete_vat_rate_id" value="<?php echo $vatRate['id']; ?>">
                            <button type="submit" class="button delete-button" onclick="return confirm('Are you sure you want to delete this VAT Rate?');">Delete</button>
                        </form>
                    <?php endif; ?>
                    <?php if (hasRole('vat_rates_default') && !$vatRate['is_default']): ?>
                        <form action="vat_rates.php" method="POST" style="display:inline;">
                            <input type="hidden" name="set_default_vat_rate_id" value="<?php echo $vatRate['id']; ?>">
                            <button type="submit" class="button default-button">Set as Default</button>
                        </form>
                    <?php endif; ?>
                </td>
            </tr>
        <?php endforeach; ?>
    </tbody>
</table>
<?php require_once('components/footer.php'); ?>

